unit wave_dlg;

{

   COPYRIGHT 1999 .. 2014 Blue Hell / Jan Punter

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License version 2 as
  published by the Free Software Foundation;

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

  For all listed email addresses :

    _dot. to be substituted by a dot      '.'
    2@t2  to be substituted by an at sign '@'


  Blue Hell is a trade mark owned by

    Jan Punter
    Oogstplein 6
    7545 HP Enschede
    the Netherlands
    http://www.iaf.nl/Users/BlueHell/
    http://bluehell.electro-music.com/
    j_dot.punter2@t2iaf_dot.nl

  All rights attributed to Blue Hell are owned by Jan Punter.
}

interface

uses

  WinApi.Windows, WinApi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics, Vcl.Controls,
  Vcl.Forms, Vcl.Dialogs, Vcl.StdCtrls, Vcl.Buttons, WinApi.MMSystem,

  Globals, KnobsUtils, KnobsConversions;

type
  TFormWaveDeviceSelect = class(TForm)
    BitBtnOk: TBitBtn;
    BitBtnCancel: TBitBtn;
    Label1: TLabel;
    Label2: TLabel;
    ComboBoxWaveIn: TComboBox;
    ComboBoxWaveOut: TComboBox;
    procedure ComboBoxWaveInChange(Sender: TObject);
    procedure ComboBoxWaveOutChange(Sender: TObject);
  private
    FSelectedInput    : string;
    FSelectedOutput   : string;
    FSelectedInputId  : DWORD;
    FSelectedOutputId : DWORD;
    FInitialized      : Boolean;
  private
    procedure   SetSelectedInput   ( const aValue: string);
    procedure   SetSelectedOutput  ( const aValue: string);
    procedure   SetSelectedInputId ( aValue: DWORD);
    procedure   SetSelectedOutputId( aValue: DWORD);
  public
    procedure   Initialize;
    function    Execute: Boolean;
  public
    property    SelectedInput    : string  read FSelectedInput    write SetSelectedInput   ;
    property    SelectedOutput   : string  read FSelectedOutput   write SetSelectedOutput  ;
    property    SelectedInputId  : DWORD   read FSelectedInputId  write SetSelectedInputId ;
    property    SelectedOutputId : DWORD   read FSelectedOutputId write SetSelectedOutputId;
  end;

var
  FormWaveDeviceSelect: TFormWaveDeviceSelect;

implementation

{$R *.dfm}


// User area

//  private

    procedure   TFormWaveDeviceSelect.SetSelectedInput( const aValue: string);
    var
      i     : Integer;
      Found : Integer;
    begin
      if not FInitialized
      then Initialize;
      if aValue <> FSelectedInput
      then begin
        with ComboBoxWaveIn
        do begin
          Found := -1;
          for i := 0 to Items.Count - 1
          do begin
            if SameText( aValue, Items[ i])
            then begin
              Found := i;
              Break;
            end;
          end;
          if Found >= 0
          then begin
            ItemIndex        := Found;
            Text             := Items[ Found];
            FSelectedInput   := Text;
            FSelectedInputId := Found;
          end;
        end;
      end;
    end;

    procedure   TFormWaveDeviceSelect.SetSelectedOutput( const aValue: string);
    var
      i     : Integer;
      Found : Integer;
    begin
      if not FInitialized
      then Initialize;
      if aValue <> FSelectedOutput
      then begin
        with ComboBoxWaveOut
        do begin
          Found := -1;
          for i := 0 to Items.Count - 1
          do begin
            if SameText( aValue, Items[ i])
            then begin
              Found := i;
              Break;
            end;
          end;
          if Found >= 0
          then begin
            ItemIndex         := Found;
            Text              := Items[ Found];
            FSelectedOutput   := Text;
            FSelectedOutputId := Found;
          end;
        end;
      end;
    end;

    procedure   TFormWaveDeviceSelect.SetSelectedInputId( aValue: DWORD);
    begin
      if not FInitialized
      then Initialize;
      if aValue <> FSelectedInputId
      then begin
        if ( aValue <> DWORD( -1)) and ( Integer( aValue) < ComboBoxWaveIn.Items.Count)
        then SelectedInput := ComboBoxWaveIn.Items[ aValue]
        else begin
          SelectedInput   := '';
          SelectedInputId := DWORD( -1);
        end;
      end;
    end;

    procedure   TFormWaveDeviceSelect.SetSelectedOutputId( aValue: DWORD);
    begin
      if not FInitialized
      then Initialize;
      if aValue <> FSelectedOutputId
      then begin
        if ( aValue <> DWORD( -1)) and ( Integer( aValue) < ComboBoxWaveOut.Items.Count)
        then SelectedOutput := ComboBoxWaveOut.Items[ aValue]
        else begin
          SelectedOutput   := '';
          SelectedOutputId := DWORD( -1);
        end;
      end;
    end;

//  public

    procedure   TFormWaveDeviceSelect.Initialize;
    var
      i         : Integer;
      NumDevs   : Integer;
      InCaps    : WaveInCaps;
      OutCaps   : WaveOutCaps;
      Res       : MMResult;
      Cnt       : Integer;
    begin
      FSelectedInputId  := DWORD( -1);
      FSelectedOutputId := DWORD( -1);
      NumDevs := waveInGetNumDevs;
      Cnt     := 0;
      ComboBoxWaveIn.Items.Clear;
      ComboBoxWaveIn.Text := '';
      for i := 0 to NumDevs
      do begin
        Res := waveInGetDevCaps( i, @ InCaps, SizeOf( WaveInCaps));
        if Res = MMSYSERR_NOERROR
        then begin
          Inc( Cnt);
          with InCaps
          do ComboBoxWaveIn.Items.Add( Format( '%s', [ szPname], AppLocale));
        end;
      end;
      if Cnt > 0
      then begin
        ComboBoxWaveIn.Text      := ComboBoxWaveIn.Items[ 0];
        ComboBoxWaveIn.ItemIndex := 1;
      end;

      NumDevs := waveOutGetNumDevs;
      Cnt     := 0;
      ComboBoxWaveOut.Items.Clear;
      ComboBoxWaveOut.Text := '';
      for i := 0 to NumDevs
      do begin
        Res := waveOutGetDevCaps( i, @ OutCaps, SizeOf( WaveOutCaps));
        if Res = MMSYSERR_NOERROR
        then begin
          Inc( Cnt);
          with OutCaps
          do ComboBoxWaveOut.Items.Add( Format( '%s', [ szPname], AppLocale));
        end;
      end;
      if Cnt > 0
      then begin
        ComboBoxWaveOut.Text      := ComboBoxWaveOut.Items[ 0];
        ComboBoxWaveOut.ItemIndex := 1;
      end;
      FInitialized := True;
    end;


    function    TFormWaveDeviceSelect.Execute: Boolean;
    begin
      if not FInitialized
      then Initialize;
      Result := ShowModal = mrOk;
    end;


// Delphi area


procedure TFormWaveDeviceSelect.ComboBoxWaveInChange(Sender: TObject);
begin
  SelectedInputId := ComboBoxWaveIn.ItemIndex;
end;

procedure TFormWaveDeviceSelect.ComboBoxWaveOutChange(Sender: TObject);
begin
  SelectedOutputId := ComboBoxWaveOut.ItemIndex;
end;


end.

