unit knobs2013;

{

  (C) COPYRIGHT 1999 .. 2014 Blue Hell / Jan Punter

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License version 2 as
  published by the Free Software Foundation;

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

  For all listed email addresses :

    _dot. to be substituted by a dot      '.'
    2@t2  to be substituted by an at sign '@'


  Blue Hell is a trade mark owned by

    Jan Punter
    Oogstplein 6
    7545 HP Enschede
    the Netherlands
    http://www.iaf.nl/Users/BlueHell/
    http://bluehell.electro-music.com/
    j_dot.punter2@t2iaf_dot.nl

  All rights attributed to Blue Hell are owned by Jan Punter.
}

interface

uses

  WinApi.Windows, WinApi.Messages, System.SysUtils, System.Types, System.StrUtils, System.Math, System.Classes,
  System.Contnrs, Vcl.Graphics, Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.ExtCtrls, Vcl.Buttons, Vcl.StdCtrls,
  Vcl.ComCtrls, WinApi.MMSystem,

  KnobsUtils, KnobsConversions;

const

  DO_RESTACK                = True;
  NO_RESTACK                = not DO_RESTACK;
  DO_DRAG                   = True;
  NO_DRAG                   = not DO_DRAG;
  DO_REDRAW_WIRES           = True;
  NO_REDRAW_WIRES           = not DO_REDRAW_WIRES;
  DO_NOTIFY                 = True;
  NO_NOTIFY                 = not DO_NOTIFY;

  KNOBS_UM_CONNECTIONADD    = WM_USER + 0;
  KNOBS_UM_CONNECTIONREMOVE = WM_USER + 1;
  KNOBS_UM_COLORCHANGE      = WM_USER + 2;
  KNOBS_UM_HIGHLIGHT        = WM_USER + 3;

  CL_NO_OUTPUT              = clAqua;
  CL_HIGHLIGHT              = clWhite;

const

  KnobsColorDisplay = clGray;    // Default BackgroundColor for TKnobsDisplay.
  KnobsPatchVersion = 2;

type

  TKnobsLedShape = (
    lsRound,
    lsSquare
  );

  TKnobsModuleFlag = (
    mfSelected,
    mfCapturing
  );
  TKnobsModuleFlags = set of TKnobsModuleFlag;

  TKnobsConnectOp = (
    coConnect,
    coDisconnect
  );

  TKnobsReadMode = (
    rmAppend,                    // Adds the read mudules to an editor patch
    rmReplace                    // Removes the existing modules and replaces them with the read modules
  );

  TKnobsWriteMode = (
    wmAll,
    wmSelected
  );

  TKnobsDisplayType = (
    dtSingleFromLow,
    dtSingleFromHigh,
    dtFilledFromLow,
    dtFilledFromHigh
  );

  TKnobsDisplayOrientation = (
    doHorizontal,
    doVertical
  );

  TKnobsButtonsDisplayMode = (
    bdmHorizontal,
    bdmVertical
  );

  TKnobsDVStyle = (        // DataViewer design time styles
    dvsBlank,              // Paint nothing
    dvsSine,               // Paint a sine
    dvsTri,                // triangle
    dvsSaw,                // saw
    dvsSquare,             // pulse / square
    dvsNoise,              // noise / random
    dvsEnvAr,              // AR   type envelope
    dvsEnvAhd,             // AHD  type envelope
    dvsEnvAdsr             // ADSR type envelope
  );

  TKnobsPairingMode = (    // Pairing mode for paired knobs
    pmOff,                 // Pairing is off
    pmNormal,              // Pairing is normal, both knobs have the same position
    pmMirrored             // Pairing is anti normal, knobs have mirrored positions
  );

  EKnobs             = class( Exception);
  EKnobsCorruptPatch = class( EKnobs);
  EKnobsNoDuplicates = class( EKnobs);
  EKnobsShortCircuit = class( EKnobs);

  TKnobsBezierPoints = array[ 0 .. 3] of TPoint;
  TKnobsBezierParams = record
    Angle1    : Integer;
    Angle2    : Integer;
    Strength1 : Integer;
    Strength2 : Integer;
  end;


  TKnobsModuleType           = Integer;

  TKnobsConnector            = class;
  TKnobsValuedControl        = class;
  TKnobsDisplay              = class;
  TKnobsEditLabel            = class;
  TKnobsCustomModule         = class;
  TKnobsWirePanel            = class;
  TKnobsModuleSelector       = class;
  TKnobsModuleButton         = class;

  TKnobsModuleClass          = class of TKnobsCustomModule;

  TKnobsOnValueChange        = procedure( aSender: TObject; const aPath, aControlType: string; aValue: TSignal; IsFinal: Boolean) of object;
  TKnobsOnModuleButtonClick  = procedure( aSender: TObject; aModuleclass: TKnobsModuleType)                                 of object;
  TKnobsOnGetGlyph           = function ( aSender: TObject; const aName: string): TBitmap                                   of object;
  TKnobsOnLog                = procedure( aSender: TObject; const aMsg: string)                                             of object;
  TKnobsOnHistoryChange      = procedure( aSender: TObject; anUndoCount, aRedoCount: Integer)                               of object;
  TKnobsOnPatchChanged       = procedure( aSender: TKnobsWirePanel)                                                         of object;
  TKnobsOnEditLabelChanged   = procedure( aSender: TObject; const aValue: string)                                           of object;
  TKnobsOnDisplayChanged     = procedure( aSender: TObject; const aPath, aValue: string)                                    of object;
  TKnobsOnFileChanged        = procedure( aSender: TObject; const aPath, aValue: string)                                    of object;
  TKnobsOnVisit              = procedure( aSender: TObject)                                                                 of object;
  TOnCreateModuleBitmap      = function ( aModuleType: TKnobsModuleType): TBitmap                                           of object;
  TOnReadModuleComment       = function ( aModuleType: TKnobsModuleType): string                                            of object;
  TOnShowPopup               = procedure( const aSender: TKnobsWirePanel; const aControl: TControl)                         of object;


  TKnobsUMConnectionChange = record
    Msg         : Cardinal;
    Source      : TKnobsConnector;
    Destination : TKnobsConnector;
    Result      : LRESULT;
  end;


  TKnobsUMColorChange = record
    Msg    : Cardinal;
    Source : TKnobsConnector;
    Color  : TColor;
    Result : LRESULT;
  end;
  TKnobsUMConnectionAdd    = TKnobsUMConnectionChange;
  TKnobUMConnectionRemove = TKnobsUMConnectionChange;


  TKnobsUMHighlight = record
    Msg         : Cardinal;
    Source      : TKnobsConnector;
    NotUsed2    : LPARAM;
    Result      : LRESULT;
  end;


  TKnobsUMButtonUp = record
    Msg         : Cardinal;
    Source      : TKnobsModuleButton;
    NotUsed     : LPARAM;
    Result      : LRESULT;
  end;


  IKnobsPatchReader = interface
    procedure   ReadString(
      const aData      : string;
      const aWirePanel : TKnobsWirePanel;
      aDistance        : TPoint;
      aReadMode        : TKnobsReadMode = rmReplace;
      const aFileName: string = ''
    );

    procedure   ReadFile(
      const aFileName  : string;
      const aWirePanel : TKnobsWirePanel;
      aReadMode        : TKnobsReadMode = rmReplace
    );

    procedure   ReadFromClipboard(
      const aWirePanel : TKnobsWirePanel;
      aReadMode        : TKnobsReadMode = rmReplace
    );
  end;


  IKnobsPatchWriter = interface
    function    WriteString(
      const aWirePanel : TKnobsWirePanel;
      const aFileName  : string = '';
      aWriteMode       : TKnobsWriteMode = wmAll
    ): string;

    procedure   WriteFile(
      const aWirePanel : TKnobsWirePanel;
      const aFileName  : string;
      aWriteMode       : TKnobsWriteMode = wmAll
    );

    procedure   WriteToClipboard(
      const aWirePanel : TKnobsWirePanel;
      aWriteMode       : TKnobsWriteMode = wmAll
    );
  end;


  TKnobsLed = class( TGraphicControl)
  private
    FTime       : Cardinal;
    FTimer      : TTimer;
    FColorOff   : TColor;
    FColorOn    : TColor;
    FHighLight  : Boolean;
    FActive     : Boolean;
    FShape      : TKnobsLedShape;
  private
    procedure   TimerFired( aSender: TObject);
    procedure   StartTimer( aValue: Cardinal);
    procedure   StopTimer;
    procedure   SetTime     ( aValue: Cardinal      );
    procedure   SetActive   ( aValue: Boolean       );
    procedure   SetColorOn  ( aValue: TColor        );
    procedure   SetColorOff ( aValue: TColor        );
    procedure   SetHighLight( aValue: Boolean       );
    procedure   SetShape    ( aValue: TKnobsLedShape);
  protected
    procedure   Paint;                       override;
  public
    constructor Create( aOwner: TComponent); override;
    destructor  Destroy;                     override;
    procedure   Toggle;
  published
    property    Time       : Cardinal       read FTime       write SetTime       default 0;
    property    Active     : Boolean        read FActive     write SetActive     default True;
    property    ColorOn    : TColor         read FColorOn    write SetColorOn    default clLime;
    property    ColorOff   : TColor         read FColorOff   write SetColorOff   default clGreen;
    property    HighLight  : Boolean        read FHighLight  write SetHighLight  default True;
    property    Shape      : TKnobsLedShape read FShape      write SetShape      default lsRound;
    property    Height                                                           default 15;
    property    Width                                                            default 15;
    property    Hint;
    property    ParentShowHint                                                   default True;
    property    Dragcursor;
    property    Dragmode;
    property    OnDragDrop;
    property    OnDragOver;
    property    OnEndDrag;
    property    OnMouseDown;
    property    OnMouseMove;
    property    OnMouseUp;
  end;


  TKnobsClicker = class( TSpeedButton)
  // Repeating speed button, semi flat ...
  private
    FRepeatTimer : TTimer;
    FRepeatCount : Integer;
    FCanRepeat   : Boolean;
    FMouseDown   : Boolean;
    FShowFocus   : Boolean;
    FBitmap      : TBitmap;
  private
    procedure   TimerExpired( aSender: TObject);
    procedure   SetShowFocus( aValue: Boolean);
  private
    function    FindWirePanel: TKnobsWirePanel;
  protected
    procedure   MouseDown( Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure   MouseUp  ( Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure   AssignBitmap( aBitmap: TBitmap);
  public
    constructor Create( anOwner: TComponent);                                        override;
    destructor  Destroy;                                                             override;
    procedure   Click;                                                               override;
  private
    property    ShowFocus: Boolean read FShowFocus write SetShowFocus;
  public
    property    CanRepeat: Boolean read FCanRepeat write FCanRepeat default True;
  end;


  TKnobsValuedControl = class( TCustomControl)
  // Base type for valued controls, ones that can have an effect on an external model
  private
    FStepCount       : Integer;             // Number of steps in the total knob travel
    FLocked          : Boolean;
    FKnobPosition    : Integer;             // Current knob position
    FDefaultPosition : Integer;             // default knob position;
    FControlType     : string;              // The control's behaviour
    FOnValueChanged  : TKnobsOnValueChange; // Called with the new value
  private
    procedure   CommonKnobPosition;
    procedure   FixKnobPosition;
    procedure   FinalizeKnobPosition;
  private
    procedure   SetKnobPosition   ( aValue: Integer);                   virtual;
    procedure   SetDefaultPosition( aValue: Integer);                   virtual;
    procedure   SetStepCount      ( aValue: Integer);                   virtual;
    procedure   SetLocked         ( aValue: Boolean);
    procedure   SetControlType    ( const aValue: string);
    function    GetAsHint         : string;
    function    GetAsDisplay      : string;
    function    GetAsValue        : TSignal;
    procedure   SetOnValueChanged ( aValue: TKnobsOnValueChange);
    procedure   NotifyPeers;                                            virtual;
    procedure   ValueChanged( IsFinal: Boolean);                        virtual; // Value had changed
  protected
    function    FindModule   : TKnobsCustomModule;
    function    FindWirePanel: TKnobsWirePanel;
    procedure   BeginStateChange;
    procedure   EndStateChange;
  public
    constructor Create( anOwner: TComponent);                          override;
    procedure   SetDefault;
    procedure   FixDisplay;
    procedure   Dump( var aFile: TextFile; anInd: Integer);
  public
    property    AsDisplay       : string          read GetAsDisplay;
    property    AsHint          : string          read GetAsHint;
    property    AsValue         : TSignal         read GetAsValue;
  published
    property    Action;
    // NOTE: declaration order is important here to avoid loss of resolution on a load action ...
    property    StepCount       : Integer         read FStepCount       write SetStepCount       default 1;
    property    Locked          : Boolean         read FLocked          write SetLocked;
    property    KnobPosition    : Integer         read FKnobPosition    write SetKnobPosition    default 0;
    property    DefaultPosition : Integer         read FDefaultPosition write SetDefaultPosition default 0;
    property    ControlType     : string          read FControlType     write SetControlType;
    // END NOTE.
    property    TabOrder;
    property    TabStop;
    property    Visible;
    property    OnValueChanged  : TKnobsOnValueChange  read FOnValueChanged  write SetOnValueChanged;
  end;


  TKnobsSelector = class( TKnobsValuedControl)
  private
    FCaptions    : TStringList;
    FBorderColor : TColor;
  private
    procedure   SetCaptions   ( const aValue: TStringList);
    procedure   SetBorderColor( aValue: TColor);
  private
    procedure   WMSetFocus  ( var aMessage: TWMSetFocus  );  message WM_SETFOCUS;
    procedure   WMKillFocus ( var aMessage: TWMKillFocus );  message WM_KILLFOCUS;
    procedure   WMGetDlgCode( var aMessage: TWMGetDlgCode);  message WM_GETDLGCODE;
  protected
    procedure   DoClick( aSender: TObject);
    procedure   Paint;                                       override;
  public
    constructor Create( anOwner: TComponent);                override;
    destructor  Destroy;                                     override;
  published
    property    Captions    : TStringList read FCaptions    write SetCaptions;
    property    Color                                                             default clGray;
    property    BorderColor : TColor      read FBorderColor write SetBorderColor  default clWhite;
    property    Font;
  end;


  TKnobsKnob = class( TKnobsValuedControl)
  // A clavia style rotary knob, only circular motion supported.
  private
    FDownClicker       : TKnobsClicker;
    FUpClicker         : TKnobsClicker;
    FMouseDown         : Boolean;
    FBitmap            : TBitmap;
    FMouseInControl    : Boolean;
    FTransparent       : Boolean;
    FDisplay           : TKnobsDisplay;
    FPairedWith        : TKnobsKnob;
    FPairingMode       : TKnobsPairingMode;
    FPairingLock       : Integer;
    FControlMode       : TDistanceMode;
    FFinalizationTimer : TTimer;
    FTresholdPast      : Boolean;
  private
    procedure   InitializeFinalizationTimer;
    procedure   FinalizeFinalizationTimer;
    procedure   StartFinalizationTimer;
    procedure   StopFinalizationTimer;
    procedure   FinalizationTimerFired( aSender: TObject);
  private
    procedure   WMSetFocus  ( var aMessage: TWMSetFocus  );  message WM_SETFOCUS;
    procedure   WMKillFocus ( var aMessage: TWMKillFocus );  message WM_KILLFOCUS;
    procedure   WMGetDlgCode( var aMessage: TWMGetDlgCode);  message WM_GETDLGCODE;
    procedure   CMMouseEnter( var aMessage: TMessage);       message CM_MOUSEENTER;
    procedure   CMMouseLeave( var aMessage: TMessage);       message CM_MOUSELEAVE;
    function    CreateClicker( aLeft, aTop: Integer; aNotifyEvent: TNotifyEvent; aBitmap: TBitmap): TKnobsClicker;
    procedure   OnDownClick( aSender: TObject);
    procedure   OnUpClick  ( aSender: TObject);
    procedure   NotifyPeers;                                           override;
    procedure   LockPairing;
    procedure   UnlockPairing;
    function    PairingLocked: Boolean;
    procedure   FixPairing;
  private
    procedure   SetTransparent    ( aValue: Boolean          );
    procedure   SetDisplay        ( aValue: TKnobsDisplay    );
    procedure   SetKnobPosition   ( aValue: Integer          );        override;
    procedure   SetPairedWith     ( const aValue: TKnobsKnob );
    procedure   SetPairingMode    ( aValue: TKnobsPairingMode);
  private
    function    CenterPoint: TPoint;
    function    CreateRgn: HRGN;
    procedure   BeginMove ( aPoint: TPoint);                           virtual;
    procedure   EndMove   ( aPoint: TPoint);                           virtual;
    procedure   UpdateMove( aPoint: TPoint);                           virtual;
  protected
    procedure   Paint;                                                 override;
    procedure   MouseDown( Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure   MouseUp  ( Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure   MouseMove( Shift: TShiftState; X, Y: Integer);         override;
    procedure   KeyDown( var Key: Word; Shift: TShiftState);           override;
    procedure   AssignBitmap;                                          virtual;
    procedure   Loaded;                                                override;
  public
    constructor Create( anOwner: TComponent);                          override;
    destructor  Destroy;                                               override;
  published
    property    Action;
    property    Transparent  : Boolean           read FTransparent   write SetTransparent;
    property    Color;
    property    TabOrder;
    property    TabStop;
    property    ParentColor                                                             default True;
    property    Display      : TKnobsDisplay     read FDisplay       write SetDisplay;
    property    PairedWith   : TKnobsKnob        read FPairedWith    write SetPairedWith;
    property    PairingMode  : TKnobsPairingMode read FPairingMode   write SetPairingMode;
    property    ControlMode  : TDistanceMode     read FControlMode   write FControlMode;
  end;


  TKnobsSmallKnob = class( TKnobsKnob)
  // just a bit smaller than TKnob
  protected
    procedure   AssignBitmap;                                          override;
  end;


  TKnobsNoKnob = class( TKnobsKnob)
  // No knob at all, just the clickers
  protected
    procedure   AssignBitmap;                                          override;
    procedure   Paint;                                                 override;
  public
    constructor Create( anOwner: TComponent);                          override;
  end;


  TKnobsValuedButton = class( TKnobsValuedControl)
  private
    FButton     : TSpeedButton;
    FColorFocus : TColor;
    FUserColor  : TColor;
    FUserFlat   : Boolean;
  private
    procedure   SetKnobPosition    ( aValue: Integer);                 override;
    function    GetCaption         : string;
    procedure   SetCaption         ( const aValue: string);
    function    GetGlyph           : TBitmap;
    procedure   SetGlyph           ( const aValue: TBitmap);
    function    GetHint            : string;
    procedure   SetHint            ( const aValue: string);
    function    GetGroupIndex      : Integer;
    procedure   SetGroupIndex      ( aValue: Integer);
    function    GetAllowAllUp      : Boolean;
    procedure   SetAllowAllUp      ( aValue: Boolean);
  private
    function    GetAlignWithMargins: Boolean;
    procedure   SetAlignWithMargins( aValue: Boolean);
    function    GetFlat            : Boolean;
    procedure   SetFlat            ( aValue: Boolean);
    function    GetFont            : TFont;
    procedure   SetFont            ( const aValue: TFont);
    function    GetLayout          : TButtonLayout;
    procedure   SetLayout          ( aValue: TButtonLayout);
    function    GetMargin          : Integer;
    procedure   SetMargin          ( aValue: Integer);
    function    GetMargins         : TMargins;
    procedure   SetMargins         ( aValue: TMargins);
    function    GetNumGlyphs       : TNumGlyphs;
    procedure   SetNumGlyphs       ( aValue: TNumGlyphs);
    function    GetParentFont      : Boolean;
    procedure   SetParentFont      ( aValue: Boolean);
    function    GetParentShowHint  : Boolean;
    procedure   SetParentShowHint  ( aValue: Boolean);
  private
    function    GetShowHint        : Boolean;
    procedure   SetShowHint        ( aValue: Boolean);
    function    Gettransparent     : Boolean;
    procedure   SetTransparent     ( aValue: Boolean);
    function    GetVisible         : Boolean;
    procedure   SetVisible         ( aValue: Boolean);
  private
    procedure   OnButtonClick( aSender: TObject);
  protected
    procedure   WMSetFocus  ( var aMessage: TWMSetFocus  );            message WM_SETFOCUS;
    procedure   WMKillFocus ( var aMessage: TWMKillFocus );            message WM_KILLFOCUS;
    procedure   WMGetDlgCode( var aMessage: TWMGetDlgCode);            message WM_GETDLGCODE;
    procedure   KeyDown     ( var Key: Word; Shift: TShiftState);      override;
    procedure   SetBiDiMode        ( aValue: TBiDiMode);               override;
    procedure   SetParentBiDiMode  ( aValue: Boolean);                 override;
    function    GetEnabled         : Boolean;                          override;
    procedure   SetEnabled         ( aValue: Boolean);                 override;
  protected
    procedure   Resize;                                                override;
    procedure   Loaded;                                                override;
    procedure   SetDefaults;                                            virtual;
  public
    constructor Create( anOwner: TComponent);                          override;
    destructor  Destroy;                                               override;
  published
    property    Caption          : string        read GetCaption          write SetCaption                               ;
    property    Glyph            : TBitmap       read GetGlyph            write SetGlyph                                 ;
    property    Hint             : string        read GetHint             write SetHint                                  ;
    property    GroupIndex       : Integer       read GetGroupIndex       write SetGroupIndex       default             1;
    property    AllowAllUp       : Boolean       read GetAllowAllUp       write SetAllowAllUp       default         false;
    property    Height                                                                              default            18;
    property    Width                                                                               default            18;
    property    AlignWithMargins : Boolean       read GetAlignWithMargins write SetAlignWithMargins default         False;
    property    Color                                                                               default      clSilver;
    property    ColorFocus       : TColor        read FColorFocus         write FColorFocus         default     clSkyBlue;
    property    Flat             : Boolean       read GetFlat             write SetFlat             default         False;
    property    Font             : TFont         read GetFont             write SetFont                                  ;
    property    Layout           : TButtonLayout read GetLayout           write SetLayout           default   blGlyphLeft;
    property    Margin           : Integer       read GetMargin           write SetMargin           default            -1;
    property    Margins          : TMargins      read GetMargins          write SetMargins                               ;
    property    NumGlyphs        : TNumGlyphs    read GetNumGlyphs        write SetNumGlyphs        default             1;
    property    ParentFont       : Boolean       read GetParentFont       write SetParentFont       default          True;
    property    ParentShowHint   : Boolean       read GetParentShowHint   write SetParentShowHint   default          True;
    property    ShowHint         : Boolean       read GetShowHint         write SetShowHint         default         False;
    property    Transparent      : Boolean       read GetTransparent      write SetTransparent      default          True;
    property    Visible          : Boolean       read GetVisible          write SetVisible          default          True;
  end;


  TKnobsValuedButtons = class( TKnobsValuedControl)
  private
    FButtons          : array of TSpeedButton;
    FCaptions         : TStrings;
    FImageIndices     : TStrings;
    FGlyphs           : TImageList;
    FHints            : TStrings;
    FGroupIndex       : Integer;
    FAlignWithMargins : Boolean;
    FAllowAllUp       : Boolean;
    FFlat             : Boolean;
    FFont             : TFont;
    FLayout           : TButtonLayout;
    FDisplayMode      : TKnobsButtonsDisplayMode;
    FMargin           : Integer;
    FMargins          : TMargins;
    FNumGlyphs        : TNumGlyphs;
    FParentFont       : Boolean;
    FParentShowHint   : Boolean;
    FShowHint         : Boolean;
    FTransparent      : Boolean;
    FVisible          : Boolean;
    FColorFocus       : TColor;
    FUserColor        : TColor;
    FUserFlat         : Boolean;
  private
    procedure   FixDimensions;
    procedure   FixGlyph( anIndex: Integer; aBitmap: TBitmap);
    function    FindGlyphIndex( anIndex: Integer): Integer;
  private
    procedure   FixAlignWithMargins;
    procedure   FixBiDiMode;
    procedure   FixCaptions;
    procedure   FixEnabled;
    procedure   FixFlat;
    procedure   FixFont;
    procedure   FixGlyphs;
    procedure   FixHints;
    procedure   FixLayout;
    procedure   FixMargin;
    procedure   FixMargins;
    procedure   FixNumGlyphs;
    procedure   FixParentFont;
    procedure   FixParentShowHint;
    procedure   FixParentBiDiMode;
    procedure   FixShowHint;
    procedure   FixTransparent;
    procedure   FixVisible;
    procedure   FixMode( aNewStepCount: Integer);
  private
    procedure   SetKnobPosition    ( aValue: Integer);                 override;
    procedure   SetStepCount       ( aValue: Integer);                 override;
    function    GetCaptions        : TStrings;
    procedure   SetCaptions        ( const aValue: TStrings);
    function    GetImageIndices    : TStrings;
    procedure   SetImageIndices    ( const aValue: TStrings);
    function    GetGlyphs          : TImageList;
    procedure   SetGlyphs          ( const aValue: TImageList);
    function    GetHints           : TStrings;
    procedure   SetHints           ( const aValue: TStrings);
    procedure   SetGroupIndex      ( aValue: Integer);
    procedure   SetAllowAllUp      ( aValue: Boolean);
  private
    procedure   SetAlignWithMargins( aValue: Boolean);
    procedure   SetFlat            ( aValue: Boolean);
    procedure   SetFont            ( const aValue: TFont);
    procedure   SetLayout          ( aValue: TButtonLayout);
    procedure   SetDisplayMode     ( aValue: TKnobsButtonsDisplayMode);
    procedure   SetMargin          ( aValue: Integer);
    procedure   SetMargins         ( aValue: TMargins);
    procedure   SetNumGlyphs       ( aValue: TNumGlyphs);
    procedure   SetParentFont      ( aValue: Boolean);
    procedure   SetParentShowHint  ( aValue: Boolean);
  private
    procedure   SetShowHint        ( aValue: Boolean);
    procedure   SetTransparent     ( aValue: Boolean);
    procedure   SetVisible         ( aValue: Boolean);
  private
    procedure   OnButtonClick( aSender: TObject);
    procedure   CreateButton;
  protected
    procedure   WMSetFocus  ( var aMessage: TWMSetFocus  );            message WM_SETFOCUS;
    procedure   WMKillFocus ( var aMessage: TWMKillFocus );            message WM_KILLFOCUS;
    procedure   WMGetDlgCode( var aMessage: TWMGetDlgCode);            message WM_GETDLGCODE;
    procedure   KeyDown     ( var Key: Word; Shift: TShiftState);      override;
    procedure   SetBiDiMode        ( aValue: TBiDiMode);               override;
    procedure   SetParentBiDiMode  ( aValue: Boolean);                 override;
    procedure   SetEnabled         ( aValue: Boolean);                 override;
  protected
    procedure   Resize;                                                override;
    procedure   Loaded;                                                override;
    procedure   FixButtons;                                             virtual;
    procedure   SetDefaults;                                            virtual;
  public
    constructor Create( anOwner: TComponent);                          override;
    destructor  Destroy;                                               override;
  published
    property    StepCount                                                                                    default             2;
    property    Captions         : TStrings                 read GetCaptions       write SetCaptions                              ;
    property    ImageIndices     : TStrings                 read GetImageIndices   write SetImageIndices                          ;
    property    Glyphs           : TImageList               read GetGlyphs         write SetGlyphs                                ;
    property    Hints            : TStrings                 read GetHints          write SetHints                                 ;
    property    GroupIndex       : Integer                  read FGroupIndex       write SetGroupIndex       default             1;
    property    AllowAllUp       : Boolean                  read FAllowAllUp       write SetAllowAllUp       default         false;
    property    Height                                                                                       default            25;
    property    Width                                                                                        default            51;
    property    AlignWithMargins : Boolean                  read FAlignWithMargins write SetAlignWithMargins default         False;
    property    Color                                                                                        default      clSilver;
    property    ColorFocus       : TColor                   read FColorFocus       write FColorFocus         default     clSkyBlue;
    property    Flat             : Boolean                  read FFlat             write SetFlat             default         False;
    property    Font             : TFont                    read FFont             write SetFont                                  ;
    property    Layout           : TButtonLayout            read FLayout           write SetLayout           default   blGlyphLeft;
    property    DisplayMode      : TKnobsButtonsDisplayMode read FDisplayMode      write SetDisplayMode      default bdmHorizontal;
    property    Margin           : Integer                  read FMargin           write SetMargin           default            -1;
    property    Margins          : TMargins                 read FMargins          write SetMargins                               ;
    property    NumGlyphs        : TNumGlyphs               read FNumGlyphs        write SetNumGlyphs        default             1;
    property    ParentFont       : Boolean                  read FParentFont       write SetParentFont       default          True;
    property    ParentShowHint   : Boolean                  read FParentShowHint   write SetParentShowHint   default          True;
    property    ShowHint         : Boolean                  read FShowHint         write SetShowHint         default         False;
    property    Transparent      : Boolean                  read FTransparent      write SetTransparent      default          True;
    property    Visible          : Boolean                  read FVisible          write SetVisible          default          True;
  end;


  TKnobsDisplayEditor = class( TMemo)
  // An multi line editor for TDisplay
  private
    FDisplay : TKnobsDisplay;
  private
    procedure   WMKillFocus( var aMessage: TWMKillFocus );  message WM_KILLFOCUS;
    procedure   Do_Exit   ( aSender: TObject);
    procedure   Do_KeyDown( aSender: TObject; var aKey: Word; aShift: TShiftState);
  protected
    procedure   SetDisplay( aDisplay: TKnobsDisplay);
  public
    constructor Create( anOwner: TComponent);                          override;
    destructor  Destroy;                                               override;
  end;


  TKnobsDisplay = class( TStaticText)
  // Can be linked to aTKnob for auto update
  // Can have an editor to manually edit the value
  private
    FEditor          : TKnobsDisplayEditor;
    FController      : TKnobsValuedControl;
    FHasEditor       : Boolean;
    FHasParentEditor : Boolean;
    FHasScrollbar    : Boolean;
    FMustSave        : Boolean;
    FMustNotify      : Boolean;
    FOnChanged       : TKnobsOnDisplayChanged;
    FSavedColor      : TColor;
  private
    function    GetModule: TKnobsCustomModule;
    procedure   Do_KeyDown( aSender: TObject; var aKey: Word; aShift: TShiftState);
  private
    function    FindWirePanel: TKnobsWirePanel;
  protected
    procedure   DisplayChanged( const aNewValue: string; DoNotify: Boolean);
    procedure   SetText( const aValue: string);
    procedure   WMNCHitTest( var aMessage: TWMNCHitTest);                            message WM_NCHITTEST;
    procedure   WMSetFocus ( var aMessage: TWMSetFocus  );                           message WM_SETFOCUS;
    procedure   WMKillFocus( var aMessage: TWMKillFocus );                           message WM_KILLFOCUS;
    procedure   MouseDown( Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure   LockKnob;
  public
    constructor Create( anOwner: TComponent);                                        override;
    destructor  Destroy;                                                             override;
  public
    property    Module: TKnobsCustomModule  read GetModule;
    property    Editor: TKnobsDisplayEditor read FEditor   write FEditor;
  published
    property    Ctl3D                                                                                 default False;
    property    ParentColor                                                                           default False;
    property    Color                                                                                 default KnobsColorDisplay;
    property    Width                                                                                 default 33;
    property    Height                                                                                default 15;
    property    AutoSize                                                                              default False;
    property    BorderStyle                                                                           default sbsSingle;
    property    TabStop                                                                               default True;
    property    HasEditor       : Boolean                read FHasEditor       write FHasEditor       default False;
    property    HasParentEditor : Boolean                read FHasParentEditor write FHasParentEditor default False;
    property    HasScrollbar    : Boolean                read FHasScrollbar    write FHasScrollbar    default False;
    property    MustSave        : Boolean                read FMustSave        write FMustSave        default False;
    property    MustNotify      : Boolean                read FMustNotify      write FMustNotify      default False;
    property    OnChanged       : TKnobsOnDisplayChanged read FOnChanged       write FOnChanged;
  end;


  TKnobsFileSelector = class( TStaticText)
  private
    FMustSave     : Boolean;
    FMustNotify   : Boolean;
    FOnChanged    : TKnobsOnFileChanged;
    FSavedColor   : TColor;
    FFileSelector : TOpenDialog;
    FWaveName     : string;
  private
    procedure   SetWaveName( const aValue: string);
    procedure   FixCaption;
    function    GetModule: TKnobsCustomModule;
    procedure   Do_KeyDown( aSender: TObject; var aKey: Word; aShift: TShiftState);
    procedure   SelectFile;
  private
    function    FindWirePanel: TKnobsWirePanel;
  protected
    procedure   FileChanged( const aNewValue: string; DoNotify: Boolean);
    procedure   SetText( const aValue: string);
    procedure   WMSetFocus ( var aMessage: TWMSetFocus  );                           message WM_SETFOCUS;
    procedure   WMKillFocus( var aMessage: TWMKillFocus );                           message WM_KILLFOCUS;
    procedure   MouseDown( Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
  public
    constructor Create( anOwner: TComponent);                                        override;
    destructor  Destroy;                                                             override;
  public
    property    Module   : TKnobsCustomModule  read GetModule;
  published
    property    Ctl3D                                                                    default False;
    property    ParentColor                                                              default False;
    property    Color                                                                    default KnobsColorDisplay;
    property    Width                                                                    default 33;
    property    Height                                                                   default 15;
    property    AutoSize                                                                 default False;
    property    BorderStyle                                                              default sbsSingle;
    property    TabStop                                                                  default True;
    property    MustSave        : Boolean             read FMustSave   write FMustSave   default False;
    property    MustNotify      : Boolean             read FMustNotify write FMustNotify default False;
    property    WaveName        : string              read FWaveName   write SetWaveName;
    property    OnChanged       : TKnobsOnFileChanged read FOnChanged  write FOnChanged;
  end;


  TKnobsIndicator = class( TGraphicControl)
  // an LED
  private
    FActive     : Boolean;
    FActiveBm   : TBitmap;
    FInActiveBm : TBitmap;
  private
    procedure   SetActive        ( aValue: Boolean);
    procedure   SetBitmapActive  ( aValue: TBitmap);
    procedure   SetBitmapInactive( aValue: TBitmap);
  protected
    procedure   AssignBitmaps;                                          virtual;
    procedure   Paint;                                                 override;
  public
    constructor Create( anOwner: TComponent);                          override;
  public
    property    BitmapActive   : TBitmap read FActiveBm   write SetBitmapActive;
    property    BitmapInactive : TBitmap read FInactiveBm write SetBitmapInactive;
  published
    property    Active         : Boolean read FActive     write SetActive;
  end;


  TKnobsIndicatorBar = class( TGraphicControl)
  // A bar of indicators, horizontal or vertical
  private
    FIndicatorCount : Integer;
    FValue          : TSignal;
    FValueLowMid    : TSignal;
    FValueMidHigh   : TSignal;
    FDisplayType    : TKnobsDisplayType;
    FOrientation    : TKnobsDisplayOrientation;
    FSpacing        : Integer;
    FActiveLowBm    : TBitmap;
    FInActiveLowBm  : TBitmap;
    FActiveMidBm    : TBitmap;
    FInActiveMidBm  : TBitmap;
    FActiveHighBm   : TBitmap;
    FInActiveHighBm : TBitmap;
  private
    function    GetMaxBmWidth : Integer;
    function    GetMaxBmHeight: Integer;
    procedure   FixDimensions;
  private
    procedure   SetIndicatorCount( aValue: Integer            );
    procedure   SetValue         ( aValue: TSignal            );
    procedure   SetValueLowMid   ( aValue: TSignal            );
    procedure   SetValueMidHigh  ( aValue: TSignal            );
    procedure   SetDisplayType   ( aValue: TKnobsDisplayType       );
    procedure   SetOrientation   ( aValue: TKnobsDisplayOrientation);
    procedure   SetSpacing       ( aValue: Integer            );
    procedure   SetActiveLowBm   ( aValue: TBitmap            );
    procedure   SetInActiveLowBm ( aValue: TBitmap            );
    procedure   SetActiveMidBm   ( aValue: TBitmap            );
    procedure   SetInActiveMidBm ( aValue: TBitmap            );
    procedure   SetActiveHighBm  ( aValue: TBitmap            );
    procedure   SetInActiveHighBm( aValue: TBitmap            );
  protected
    procedure   AssignBitmaps;                                          virtual;
    function    PaintActive( anIndex: Integer): Boolean;                virtual;
    procedure   Paint;                                                 override;
  public
    constructor Create( anOwner: TComponent);                          override;
  public
    property    ActiveLowBm    : TBitmap                  read FActiveLowBm    write SetActiveLowBm;
    property    InActiveLowBm  : TBitmap                  read FInActiveLowBm  write SetInActiveLowBm;
    property    ActiveMidBm    : TBitmap                  read FActiveMidBm    write SetActiveMidBm;
    property    InActiveMidBm  : TBitmap                  read FInActiveMidBm  write SetInActiveMidBm;
    property    ActiveHighBm   : TBitmap                  read FActiveHighBm   write SetActiveHighBm;
    property    InActiveHighBm : TBitmap                  read FInActiveHighBm write SetInActiveHighBm;
  published
    property    IndicatorCount : Integer                  read FIndicatorCount write SetIndicatorCount default 100;
    property    Value          : TSignal                  read FValue          write SetValue          nodefault;
    property    ValueLowMid    : TSignal                  read FValueLowMid    write SetValueLowMid    nodefault;
    property    ValueMidHigh   : TSignal                  read FValueMidHigh   write SetValueMidHigh   nodefault;
    property    DisplayType    : TKnobsDisplayType        read FDisplayType    write SetDisplayType    default dtFilledFromLow;
    property    Orientation    : TKnobsDisplayOrientation read FOrientation    write SetOrientation    default doHorizontal;
    property    Spacing        : Integer                  read FSpacing        write SetSpacing        default 2;
  end;


  TKnobsIndicatorText = class( TCustomLabel)
  private
    FValue : TSignal;
  protected
    procedure   SetValue( aValue: TSignal);
    procedure   DoDrawText( var aRect: TRect; aFlags: Longint);        override;
  public
    constructor Create( anOwner: TComponent);                          override;
  published
    property Value : TSignal read FValue write SetValue nodefault;
    property Width                                      default 10;
    property Height                                     default 10;
    property AutoSize                                   default false;
    property Align                                      default alNone;
    property Alignment                                  default taCenter;
    property Color                                      default KnobsColorDisplay;
    property Transparent                                default True;
    property Visible                                    default True;
    property WordWrap                                   default False;
    property ParentColor                                default False;
    property ParentFont                                 default False;
    property ShowHint                                   default False;
    property ParentShowHint                             default False;
    property ShowAccelChar                              default False;
    property Layout;
    property Anchors;
    property BiDiMode;
    property Caption;
    property Constraints;
    property Enabled;
    property FocusControl;
    property Font;
    property ParentBiDiMode;
    property PopupMenu;
  end;


  TKnobsDataViewer = class( TGraphicControl)
  // a viewer
  private
    FBackGround  : TBitmap;
    FData        : TSignalArray;
    FXOffset     : TSignal;
    FYOffset     : TSignal;
    FMaxValue    : TSignal;
    FMinValue    : TSignal;
    FXZoom       : TSignal;
    FYZoom       : TSignal;
    FBorderColor : TColor;
    FLineColor   : TColor;
    FFillColor   : TColor;
    FStyle       : TKnobsDVStyle;
  private
    function    GetModule     : TKnobsCustomModule;
    procedure   SetBackGround ( aValue: TBitmap);
    procedure   SetData       ( const aValue: TSignalArray);
    procedure   SetXOffset    ( aValue: TSignal);
    procedure   SetYOffset    ( aValue: TSignal);
    procedure   SetMaxValue   ( aValue: TSignal);
    procedure   SetMinValue   ( aValue: TSignal);
    procedure   SetXZoom      ( aValue: TSignal);
    procedure   SetYZoom      ( aValue: TSignal);
    procedure   SetLineColor  ( aValue: TColor );
    procedure   SetFillColor  ( aValue: TColor );
    procedure   SetBorderColor( aValue: TColor );
    procedure   SetStyle      ( aValue: TKnobsDVStyle);
  private
    procedure   FillBlank;
    procedure   FillSine;
    procedure   FillTri;
    procedure   FillSaw;
    procedure   FillSquare;
    procedure   FillNoise;
    procedure   FillEnvAr;
    procedure   FillEnvAhd;
    procedure   FillEnvAdsr;
  protected
    procedure   WMNCHitTest( var aMessage: TWMNCHitTest);              message WM_NCHITTEST;
    procedure   WMEraseBackground( var aMsg: TWMEraseBkgnd);           message WM_ERASEBKGND;
    procedure   MouseDown( Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    function    ScaleData( anIndex: Integer): Integer;
    function    ScaleX   ( anIndex: Integer): Integer;
    procedure   Paint;                                                 override;
  public
    constructor Create( anOwner: TComponent);                          override;
  public
    property    Data        : TSignalArray       read FData     write SetData;
    property    Module      : TKnobsCustomModule read GetModule;
  published
    property    Background  : TBitmap       read FBackGround  write SetBackGround;
    property    XOffset     : TSignal       read FXOffset     write SetXOffset;    // default  0;
    property    YOffset     : TSignal       read FYOffset     write SetYOffset;    // default 75;
    property    MaxValue    : TSignal       read FMaxValue    write SetMaxValue;   // default High( SmallInt);
    property    MinValue    : TSignal       read FMinValue    write SetMinValue;   // default Low ( SmallInt);
    property    XZoom       : TSignal       read FXZoom       write SetXZoom;      // default 1;
    property    YZoom       : TSignal       read FYZoom       write SetYZoom;      // default 1;
    property    LineColor   : TColor        read FLineColor   write SetLineColor   default clWhite;
    property    FillColor   : TColor        read FFillColor   write SetFillColor   default clSilver;
    property    BorderColor : TColor        read FBorderColor write SetBorderColor default KnobsColorDisplay;
    property    Style       : TKnobsDVStyle read FStyle       write SetStyle       default dvsNoise;
    property    Width                                                              default 150;
    property    Height                                                             default 60;
    property    Color                                                              default KnobsColorDisplay;
  end;


  TKnobsConnector = class( TGraphicControl)
  // to connect wires to
  private
    FBitmap    : TBitmap;
    FWireColor : TColor;
    FLinks     : TList;   // Connections, not wires
    FMouseDown : Boolean;
    FConnected : Boolean; // True when at least one connection exists
    FHighLight : Boolean; // Highlighted connections
    FVisited   : Boolean;
  private
    procedure   SetConnected( aValue: Boolean);
    function    GetModule: TKnobsCustomModule;
    function    GetModuleName: string;
    procedure   SetHighLight( aValue: Boolean);
    procedure   SetWireColor( aValue: TColor );
  private
    function    FindWirePanel: TKnobsWirePanel;
    procedure   AddLink   ( aConnector: TKnobsConnector);
    procedure   DeleteLink( aConnector: TKnobsConnector);
    function    HasDirectlinkTo( aConnector: TKnobsConnector): Boolean;
    function    FindConnector( aScreenPoint: TPoint): TKnobsConnector;
    function    CenterPoint: TPoint;
    procedure   DrawLine( X, Y: Integer);
    procedure   SendWireNotification( Msg: Cardinal; aConnector: TKnobsConnector);
    procedure   SendColorNotification( aColor: TColor);
    procedure   SendHighlightNotification;
    procedure   ScrollMouseInView( anX, anY: LongInt);
  protected
    procedure   MouseDown( Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure   MouseUp  ( Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure   MouseMove( Shift: TShiftState; X, Y: Integer);         override;
    function    FindOutput: TKnobsConnector;
    function    IsConnectedToOutput: Boolean;
    procedure   FindDistanceToOutput( var aDistance: Integer);
    function    IsConnectedTo( aConnector: TKnobsConnector): Boolean;
    function    OutputColor: TColor;
    function    CanAccept( aSource: TKnobsConnector; isMove: Boolean): Boolean;
    procedure   AssignBitmap;                                 virtual; Abstract;
    procedure   MoveLinksTo   ( aConnector: TKnobsConnector);
    procedure   ConnectTo     ( aConnector: TKnobsConnector);
    procedure   DisconnectFrom( aConnector: TKnobsConnector);
    function    DisConnectAll: Boolean;
    procedure   DisConnectDownStream;
    procedure   DisConnectDownStreamRecursive;
    procedure   Paint;                                                 override;
  public
    constructor Create( anOwner: TComponent);                          override;
    destructor  Destroy;                                               override;
    procedure   Dump     ( var aFile: TextFile; anInd: Integer);
    procedure   DumpFlat ( var aFile: TextFile; anInd: Integer);
    procedure   DumpLinks( var aFile: TextFile; anInd: Integer);
    procedure   Log( const aMsg: string);
    function    DistanceToOutput: Integer;
  protected
    property    HighLight: Boolean read FHighLight write SetHighLight;
  public
    property    Connected : Boolean       read FConnected write SetConnected;
    property    Module    : TKnobsCustomModule read GetModule;
    property    ModuleName: string        read GetModuleName;
  published
    property    WireColor : TColor        read FWireColor write SetWireColor default clBlue;
  end;

  TKnobsConnectors = array of TKnobsConnector;


  TKnobsInput = class( TKnobsConnector)
  // Specialized connector with it's own bitmap
  protected
    procedure   AssignBitmap;                                          override;
  end;


  TKnobsOutput = class( TKnobsConnector)
  // Specialized connector with it's own bitmap
  protected
    procedure   AssignBitmap;                                          override;
  end;


  TKnobsTextLabel = class( TCustomLabel)
  // Just a small print TLabel
  protected
    procedure   DoDrawText( var aRect: TRect; aFlags: Longint);        override;
  public
    constructor Create( anOwner: TComponent);                          override;
  published
    property Align;
    property Alignment;
    property Anchors;
    property AutoSize;
    property BiDiMode;
    property Caption;
    property Color;
    property Constraints;
    property DragCursor;
    property DragKind;
    property DragMode;
    property Enabled;
    property FocusControl;
    property Font;
    property ParentBiDiMode;
    property ParentColor;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowAccelChar;
    property ShowHint;
    property Transparent;
    property Layout;
    property Visible;
    property WordWrap;
  end;


  TKnobsLabelEditor = class( TEdit)
  // An editor for TEditLabel
  private
    FLabel : TKnobsEditLabel;
  private
    procedure   WMKillFocus( var aMessage: TWMKillFocus );  message WM_KILLFOCUS;
    procedure   Do_Exit   ( aSender: TObject);
    procedure   Do_KeyDown( aSender: TObject; var aKey: Word; aShift: TShiftState);
  protected
    procedure   SetLabel( aLabel: TKnobsEditLabel);
  public
    constructor Create( anOwner: TComponent);                          override;
    destructor  Destroy;                                               override;
  end;


  TKnobsEditLabel = class( TKnobsTextLabel)
  // a label with a popup editor so it's caption can be edited.
  private
    FOnChanged : TKnobsOnEditLabelChanged;
  private
    function    GetModule: TKnobsCustomModule;
  protected
    procedure   SetText( const aValue: string);
    procedure   MouseDown( Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
  public
    constructor Create( anOwner: TComponent);                          override;
  public
    property    Module: TKnobsCustomModule read GetModule;
  published
    property    OnChanged: TKnobsOnEditLabelChanged read FOnChanged write FOnChanged;
  end;


  TKnobsBox = class( TBevel)
  // A disablable bevel
  public
    constructor Create( anOwner: TComponent);                          override;
  published
    property    Enabled default False;
  end;


  TKnobsWire = class
  // Wire, visual aspect of a connection
  private
    FSource       : TKnobsConnector;
    FDestination  : TKnobsConnector;
    FHighlight    : Boolean;
    FBezierParams : TKnobsBezierParams;
  private
    function    GetSourceName: string;
    function    GetDestinationName: string;
    function    GetFullName: string;
  public
    constructor Create( aSource, aDestination: TKnobsConnector);
    procedure   Wiggle;
    procedure   Dump( var aFile: TextFile; anInd: Integer);
    procedure   FixWire;
    procedure   SwapConnectors;
  public
    property    Source          : TKnobsConnector    read FSource;
    property    Destination     : TKnobsConnector    read FDestination;
    property    SourceName      : string        read GetSourceName;
    property    DestinationName : string        read GetDestinationName;
    property    BezierParams    : TKnobsBezierParams read FBezierParams;
    property    FullName        : string        read GetFullName;
  end;


//  TKnobsWireOverlay = class( TGraphicControl)
  TKnobsWireOverlay = class( TCustomControl)
  //
  // Responsibilty to maintain connections is assumed by the TConnectors
  // themselves. Wires are maintained here, but it's a cache really to ease
  // wire painting. The wires are maintained by the connectors theselves that
  // send their info to us through the WirePanel (by means of UM_CONNEXTIONxx
  // and UM_HIGHLIGHT messages). The WirePanel imforms us by calling the
  // AddWire, DelWire and Highlight methodes.
  //
  // All Wires are owned by FWires, so our destructor will eventually destroy
  // all dangling wires. Nevertheless, TConnector still must, on destruction,
  // clean up the wires it is responsible for .. it wouldn't look good other
  // wise.
  //
  private
    FWires            : TObjectList;
    FCurvedLines      : Boolean;
    FWireThickness    : Integer;
    FdefaultWireColor : TColor;
    FBlockCount       : Integer;
    FPaintCount       : Integer;
    FPaintClocks      : Int64;
  private
    procedure   WMNCHitTest( var aMessage: TWMNCHitTest);  message WM_NCHITTEST;
  private
    function    GetWire         ( anIndex: Integer): TKnobsWire;
    function    GetSource       ( anIndex: Integer): TKnobsConnector;
    function    GetDestination  ( anIndex: Integer): TKnobsConnector;
    procedure   SetCurvedLines  ( aValue: Boolean );
    procedure   SetWireThickNess( aValue: Integer );
  protected
    procedure   Paint;                                                 override;
    procedure   PaintWire( aDC: HDC; aWire: TKnobsWire);
    function    FindWire ( aSource, aDestination: TKnobsConnector): Integer;
    procedure   AddWire  ( aSource, aDestination: TKnobsConnector);
    procedure   DelWire  ( aSource, aDestination: TKnobsConnector);
    procedure   Connect  ( aSource, aDestination: TKnobsConnector);
    procedure   HandleHighlight;
    procedure   HandleColorChange( aSource: TKnobsConnector; aColor: TColor);
    function    WireCount: Integer;
    procedure   WiggleWires;
  public
    constructor Create( anOwner: TComponent);                          override;
    destructor  Destroy;                                               override;
    procedure   InvalidateWires;
    function    Disconnect( aConnector: TKnobsConnector): Boolean;
    procedure   Dump( var aFile: TextFile; anInd: Integer);
    procedure   Log( const aMsg: string);
    procedure   BlockScreenUpdates;
    procedure   UnBlockScreenUpdates;
    procedure   FixAllWires;
  public
    property    Wire       [ anIndex: Integer]: TKnobsWire      read GetWire;
    property    Source     [ anIndex: Integer]: TKnobsConnector read GetSource;
    property    Destination[ anIndex: Integer]: TKnobsConnector read GetDestination;
  public
    property    CurvedLines      : Boolean read FCurvedLines      write SetCurvedLines    default True;
    property    WireThickness    : Integer read FWireThickness    write SetWireThickness  nodefault;
    property    defaultWireColor : TColor  read FdefaultWireColor write FdefaultWireColor default clBlue;
    property    BlockCount       : Integer read FBlockCount;
    property    PaintCount       : Integer read FPaintCount;
    property    PaintClocks      : Int64   read FPaintClocks;
  published
    property    Left   stored False;
    property    Top    stored False;
    property    Width  stored False;
    property    Height stored False;
  end;


  TKnobsModuleList = class( TList)
  // List of TCustomModule - some comon ops defined here
  private
    FonLog       : TKnobsOnLog;
  private
    function    GetModule  ( anIndex: Integer): TKnobsCustomModule;
    function    GetSelected( anIndex: Integer): Boolean;
    procedure   SetSelected( anIndex: Integer; aValue: Boolean);
  public
    procedure   FreeModule( anIndex: Integer);
    function    FreeModules: Boolean;
    function    FreeSelectedModules: Boolean;
    procedure   SelectAll;
    procedure   UnselectAll;
    procedure   SelectUnique( aModule: TKnobsCustomModule);
    procedure   InvertSelection;
  public
    //* Some helpers for selector population and component registration
    function    FindType( aModuleType: TKnobsModuleType): TKnobsCustomModule;
    procedure   PopulateSelector( aModuleSelector: TKnobsModuleSelector);
    function    CreateModule( aWirePanel: TKnobsWirePanel; aModuleType: TKnobsModuleType; MustDrag: Boolean): TKnobsCustomModule;
    procedure   RegisterComponents( doRegister: Boolean);
    procedure   Dump( var aFile: TextFile; anInd: Integer);
    procedure   Log( const aMsg: string);
    function    ModuleTypeName( aModuleType: Integer): string;
  public
    property    Module  [ anIndex: Integer]: TKnobsCustomModule read GetModule;                     default;
    property    Selected[ anIndex: Integer]: Boolean            read GetSelected  write SetSelected;
    property    OnLog                      : TKnobsOnLog        read FOnLog       write FOnLog;
  end;


  TKnobsHistoryStack = class( TStringList)
  private
    FMaxHistory : Integer;
  private
    procedure   SetMaxHistory( aValue: Integer);
  protected
    procedure   PushString( const S: string);
    function    PopString: string;
  public
    procedure   Push( const aWirePanel: TKnobsWirePanel);
    procedure   Pop ( const aWirePanel: TKnobsWirePanel);
  public
    property    MaxHistory : Integer read FMaxHistory write SetMaxHistory;
  end;


  TKnobsUndoRedoHistory = class( TObject)
  private
    FOnHistoryChange : TKnobsOnHistoryChange;
    FMaxHistory      : Integer;
    FLockCount       : Integer;
    FUndos           : TKnobsHistoryStack;
    FRedos           : TKnobsHistoryStack;
  private
    function    GetUndoCount: Integer;
    function    GetRedoCount: Integer;
    procedure   SetmaxHistory( aValue: Integer);
  public
    constructor Create;
    destructor  Destroy;                                               override;
    procedure   HistoryChanged;
    procedure   SaveState( const aWirePanel: TKnobsWirePanel);
    procedure   Undo     ( const aWirePanel: TKnobsWirePanel);
    procedure   Redo     ( const aWirePanel: TKnobsWirePanel);
    procedure   Lock;
    procedure   UnLock;
    procedure   Clear;
  public
    property    UndoCount       : Integer          read GetUndoCount;
    property    RedoCount       : Integer          read GetRedoCount;
    property    MaxHistory      : Integer          read FMaxHistory  write SetmaxHistory;
  public
    property    OnHistoryChange : TKnobsOnHistoryChange read FOnHistoryChange write FOnHistoryChange;
  end;


  TKnobsWirePanel = class( TScrollingWinControl)
  // Contains the modules and the wire overlay - scrollable.
  private
    FOffsetLeft        : Integer;
    FOffsetTop         : Integer;
    FTitle             : string;
    FVersion           : Integer;
    FMouseDown         : Boolean;
    FWireOverlay       : TKnobsWireOverlay;
    FModules           : TKnobsModuleList;
    FFilename          : string;
    FSnapX             : Integer;
    FSnapY             : Integer;
    FControlMode       : TDistanceMode;
    FSnapActive        : Boolean;
    FPatchReader       : IKnobsPatchReader;
    FPatchWriter       : IKnobsPatchWriter;
    FOnLog             : TKnobsOnLog;
    FOnHistoryChange   : TKnobsOnHistoryChange;
    FHistory           : TKnobsUndoRedoHistory;
    FOnValueChanged    : TKnobsOnValueChange;
    FOnDisplayChanged  : TKnobsOnDisplayChanged;
    FOnFileChanged     : TKnobsOnFileChanged;
    FOnPatchChanged    : TKnobsOnPatchChanged;
    FOnShowPopup       : TOnShowPopup;
  private
    procedure   UMConnectionAdd   ( var aMessage: TKnobsUMConnectionAdd  ); message KNOBS_UM_CONNECTIONADD;
    procedure   UMConnectionRemove( var aMessage: TKnobUMConnectionRemove); message KNOBS_UM_CONNECTIONREMOVE;
    procedure   UMColorChange     ( var aMessage: TKnobsUMColorChange    ); message KNOBS_UM_COLORCHANGE;
    procedure   UMHighlight       ( var aMessage: TKnobsUMHighlight      ); message KNOBS_UM_HIGHLIGHT;
  private
    function    GetModule     ( anIndex: Integer): TKnobsCustomModule;
    function    GetWire       ( anIndex: Integer): TKnobsWire;
    function    GetSource     ( anIndex: Integer): TKnobsConnector;
    function    GetDestination( anIndex: Integer): TKnobsConnector;
  private
    procedure   SetOffsetLeft      ( aValue: Integer);
    procedure   SetOffsetTop       ( aValue: Integer);
    function    GetCurvedLines     : Boolean;
    procedure   SetCurvedLines     ( aValue: Boolean);
    function    GetWireThickness   : Integer;
    procedure   SetWireThickness   ( aValue: Integer);
    function    GetdefaultWireColor: TColor;
    procedure   SetdefaultWireColor( aValue: TColor);
    function    GetModuleCount     : Integer;
    function    GetWireCount       : Integer;
    function    GetMaxHistory      : Integer;
    procedure   SetMaxHistory      ( aValue: Integer);
    function    GetBlockCount      : Integer;
    function    GetPaintCount      : Integer;
    function    GetPaintClocks     : Int64;
    function    GetScrollOffset    : TPoint;
    procedure   SetControlMode     ( aValue: TDistanceMode);
  private
    procedure   StrToConnection( const S: string);
  protected
    procedure   HandleRightClick( const aSender: TControl);
    procedure   CreateParams(var Params: TCreateParams);                             override;
    procedure   MouseDown( Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure   MouseUp  ( Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure   MouseMove( Shift: TShiftState; X, Y: Integer);                       override;
    procedure   DragStart( aPoint: TPoint);                                           virtual;
    procedure   DragEnd  ( aPoint: TPoint);                                           virtual;
    procedure   DragMove ( aPoint: TPoint);                                           virtual;
    procedure   SelectModulesInRect( aRect: TRect);
    function    IndexOfModule( aModule: TKnobsCustomModule): Integer;
    procedure   Notification( aComponent: TComponent; anOperation: TOperation);      override;
    procedure   Connect( aSource, aDestination: TKnobsConnector);
    procedure   Restack( DoRestack, RedrawWires: Boolean);
    procedure   Resnap;
    procedure   FixNames;
    procedure   SolveNamingConflictsWith( aWirePanel: TKnobsWirePanel);
    function    Snap( const aPoint: TPoint): TPoint;
    procedure   DoHistoryChanged( aSender: TObject; anUndoCount, aRedoCount: Integer);
    procedure   ValueChanged  ( aSender: TObject; const aPath, aControlType: string; aValue: TSignal; IsFinal: Boolean); virtual;
    procedure   DisplayChanged( aSender: TObject; const aPath, aValue: string);
    procedure   FileChanged   ( aSender: TObject; const aPath, aValue: string);
    procedure   PatchChanged;
    procedure   ScrollPointInView( aPoint: TPoint);                                  overload;
    procedure   ScrollPointInView( anX, anY: LongInt);                               overload;
  public
    constructor Create( anOwner: TComponent);                                        override;
    destructor  Destroy;                                                             override;
    procedure   FixAllSynthParams;
    procedure   BeginStateChange;
    procedure   EndStateChange( PatchWasChanged: Boolean);
    procedure   Undo;
    procedure   Redo;
    procedure   ClearUndoRedo;
    procedure   LockUndoRedo;
    procedure   UnLockUndoRedo;
    procedure   Dump( const aFileName: string; DoAppend: Boolean);                   overload;
    procedure   Dump( var aFile: TextFile);                                          overload;
    procedure   DumpModules( var aFile: TextFile; anInd: Integer);
    procedure   DumpWires( var aFile: TextFile; anInd: Integer);
    procedure   Log( const aMsg: string);
    function    ModuleTypeName( aModuleType: Integer): string;
    function    FindModule( const aName: string): TKnobsCustomModule;
    function    FindFirstSelectedModule: TKnobsCustomModule;
    procedure   ChangeWireNames( const aModuleName: string; anOldType, aNewType: TKnobsModuleType);
    procedure   RetypeSelectedModules( aNewType: TKnobsModuleType);
  public
    procedure   InvalidateWires;
    procedure   Disconnect( aConnector: TKnobsConnector);
    procedure   FreeModules;
    function    HasEditPopup: Boolean;
    procedure   FreeSelectedModules;
    procedure   SelectAllModules;
    procedure   UnSelectAllModules;
    procedure   SelectUnique( aModule: TKnobsCustomModule);
    procedure   InvertModuleSelection;
    procedure   AddModules( aSrc: TKnobsWirePanel; aDistance: TPoint; DoRestack, RedrawWires, MustDrag: Boolean); virtual;
    procedure   AddWire( const aSrc, aDst: string);
    procedure   WiggleWires;
    procedure   FixAllWires;
    procedure   FixControlMode;
 // procedure   Compile;
    procedure   BlockScreenUpdates;
    procedure   UnblockScreenUpdates;
    procedure   Search( const aValue: string);
  public
    procedure   ConnectorDelete    ( const aConnector: TKnobsConnector);
    procedure   ConnectorBreak     ( const aConnector: TKnobsConnector);
    procedure   ConnectorDisconnect( const aConnector: TKnobsConnector);
  public
    property    PatchReader  : IKnobsPatchReader   read FPatchReader   write FPatchReader;
    property    PatchWriter  : IKnobsPatchWriter   read FPatchWriter   write FPatchWriter;
  public
    property    Module     [ anIndex: Integer]: TKnobsCustomModule  read GetModule;
    property    Wire       [ anIndex: Integer]: TKnobsWire          read GetWire;
    property    Source     [ anIndex: Integer]: TKnobsConnector     read GetSource;
    property    Destination[ anIndex: Integer]: TKnobsConnector     read GetDestination;
  public
    property    OnHistoryChange  : TKnobsOnHistoryChange  read FOnHistoryChange    write FOnHistoryChange;
    property    OnLog            : TKnobsOnLog            read FOnLog              write FonLog;
    property    Filename         : string                 read FFilename           write FFilename;
    property    ModuleCount      : Integer                read GetModuleCount;
    property    WireCount        : Integer                read GetWireCount;
    property    BlockCount       : Integer                read GetBlockCount;
    property    PaintCount       : Integer                read GetPaintCount;
    property    PaintClocks      : Int64                  read GetPaintClocks;
    property    ScrollOffset     : TPoint                 read GetScrollOffset;
  published
    property    MaxHistory       : Integer                read GetMaxHistory       write SetMaxHistory       default 100;
    property    OnValueChanged   : TKnobsOnValueChange    read FOnValueChanged     write FOnValueChanged;
    property    OnDisplayChanged : TKnobsOnDisplayChanged read FOnDisplayChanged   write FOnDisplayChanged;
    property    OnFileChanged    : TKnobsOnFileChanged    read FOnFileChanged      write FOnFileChanged;
    property    OnPatchChanged   : TKnobsOnPatchChanged   read FOnPatchChanged     write FOnPatchChanged;
    property    OnShowPoup       : TOnShowPopup           read FOnShowPopup        write FOnShowPopup;
    property    OffsetLeft       : Integer                read FOffsetLeft         write SetOffsetLeft       default 4;
    property    OffsetTop        : Integer                read FOffsetTop          write SetOffsetTop        default 4;
    property    CurvedLines      : Boolean                read GetCurvedLines      write SetCurvedLines      default True;
    property    WireThickness    : Integer                read GetWireThickness    write SetWireThickness    default 1;
    property    defaultWireColor : TColor                 read GetdefaultWireColor write SetdefaultWireColor default clBlue;
    property    Title            : string                 read FTitle              write FTitle;
    property    Version          : Integer                read FVersion            write FVersion;
    property    SnapX            : Integer                read FSnapX              write FSnapX              default 250;
    property    SnapY            : Integer                read FSnapY              write FSnapY              default 15;
    property    SnapActive       : Boolean                read FSnapActive         write FSnapActive         default True;
    property    ControlMode      : TDistanceMode          read FControlMode        write SetControlMode      default dmCircular;
    property    Align;
    property    Anchors;
    property    Constraints;
    property    ParentColor;
    property    ParentShowHint;
    property    PopupMenu;
    property    ShowHint;
    property    TabOrder;
    property    TabStop;
    property    Visible;
    property    Color;
    property    Top;
    property    Left;
    property    Width;
    property    Height;
    property    HorzScrollBar;
    property    VertScrollBar;
  end;


  TKnobsCustomModule = class( TCustomPanel)
  // Base type for TModule, it has a bitmap and a title mainly
  private
    FAnchor           : TPoint;
    FDistance         : TPoint;
    FTresholdPast     : Boolean;
    FFlags            : TKnobsModuleFlags;
    FTitleLabel       : TKnobsEditLabel;
    FTitle            : string;
    FComment          : string;
    FModuleType       : TKnobsModuleType;
    FPageName         : string;
    FBitmap           : TBitmap;
    FControlMode      : TDistanceMode;
    FOnValueChanged   : TKnobsOnValueChange;    // Called with the new value
    FOnDisplayChanged : TKnobsOnDisplayChanged;
    FOnFileChanged    : TKnobsOnFileChanged;
    FOnLog            : TKnobsOnLog;
  private
    procedure   IncludeFlags( aFlags: TKnobsModuleFlags);
    procedure   ExcludeFlags( aFlags: TKnobsModuleFlags);
    function    HasFlags( aFlags: TKnobsModuleFlags): Boolean;
  private
    function    GetSelected   : Boolean;
    procedure   SetSelected   ( aValue: Boolean);
    function    GetCapturing  : Boolean;
    procedure   SetCapturing  ( aValue: Boolean);
  private
    procedure   SetModuleType ( aValue : TKnobsModuleType);
    procedure   SetTitle      ( const aValue: string);
    procedure   SetTitleLabel ( aValue: TKnobsEditLabel);
    procedure   SetControlMode( aValue: TDistanceMode);
  protected
    procedure   WMEraseBackground( var aMsg: TWMEraseBkgnd);                         message WM_ERASEBKGND;
    procedure   MouseDown( Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure   MouseMove( Shift: TShiftState; X, Y: Integer);                       override;
    procedure   MouseUp  ( Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
  protected
    procedure   ValueChanged  ( aSender: TObject; const aPath, aControlType: string; aValue: TSignal; IsFinal: Boolean); virtual;
    procedure   DisplayChanged( aSender: TKnobsDisplay;      const aPath, aValue: string);
    procedure   FileChanged   ( aSender: TKnobsFileSelector; const aPath, aValue: string);
    function    FindWirePanel: TKnobsWirePanel;
    function    Snap( const aPoint: TPoint): TPoint;
    procedure   Paint;                                                 override;
    procedure   Loaded;                                                override;
    procedure   AssignBitmap;                                           virtual;
    procedure   ScrollSelfInView;
    procedure   HandleSelection( CtrlKeyActive: Boolean);
    procedure   BeginMove        ( anAnchor : TPoint);
    procedure   Move             ( aDistance: TPoint);
    procedure   MoveSelected     ( aDistance: TPoint);
    procedure   FinalMoveSelected( aDistance: TPoint; CtrlKeyActive: Boolean);
    procedure   ReleasePeerCapture;
    procedure   Notification( aComponent: TComponent; anOperation: TOperation); override;
    procedure   FixOwnership( aRoot, aControl: TWinControl);
    procedure   FixNames;
    function    FindValuedControl( const aName: string): TKnobsValuedControl;
    function    FindDisplay      ( const aName: string): TKnobsDisplay;
    function    FindFileSelector ( const aName: string): TKnobsFileSelector;
  public
    constructor Create( anOwner: TComponent);                          override;
    destructor  Destroy;                                               override;
    procedure   FixAllSynthParams;
    function    Clone( anOwner: TComponent; aParent: TWinControl; anOffset: TPoint; MustDrag: Boolean): TKnobsCustomModule;
    procedure   SetKnobValue  ( const aType, aName: string; aValue: Integer);
    procedure   SetDetailValue( const aType, aName: string; const aValue: string);
    procedure   SetLight( const aName: string; aValue: TSignal);
    procedure   SetData ( const aName: string; const aData: TSignalArray);
    procedure   SetInfo ( const aName: string; const aValue: string);
    procedure   Dump( var aFile: TextFile; anInd: Integer);
    procedure   Log( const aMsg: string);
    procedure   LogFmt( const aFmt: string; const anArgs: array of const);
    function    AsBitmap: TBitmap;
  public
    procedure   Restack( DoRestack, RedrawWires: Boolean);
    procedure   FixupInsertion;
    procedure   FixDisplays;
    procedure   Disconnect;
    procedure   SelectUnique;
    procedure   CollectInputsWithBaseName ( var anInputs : TKnobsConnectors; const aBaseName: string);
    procedure   CollectOutputsWithBaseName( var anOutputs: TKnobsConnectors; const aBaseName: string);
    procedure   ChangeType( aNewType: TKnobsModuleType);
  public
    property    Selected         : Boolean                read GetSelected       write SetSelected;
    property    Capturing        : Boolean                read GetCapturing      write SetCapturing;
    property    Title            : string                 read FTitle            write SetTitle;
    property    Comment          : string                 read FComment          write FComment;
    property    PageName         : string                 read FPageName         write FPageName;
    property    ModuleType       : TKnobsModuleType       read FModuleType       write SetModuleType;
    property    OnLog            : TKnobsOnLog            read FOnLog            write FOnLog;
  published
    property    TitleLabel       : TKnobsEditLabel        read FTitleLabel       write SetTitleLabel;
    property    ControlMode      : TDistanceMode          read FControlMode      write SetControlMode;
    property    OnValueChanged   : TKnobsOnValueChange    read FOnValueChanged   write FOnValueChanged;
    property    OnDisplayChanged : TKnobsOnDisplayChanged read FOnDisplayChanged write FOnDisplayChanged;
    property    OnFileChanged    : TKnobsOnFileChanged    read FOnFileChanged    write FOnFileChanged;
  end;


  TKnobsModule = class( TKnobsCustomModule)
  // exports stuff of TCustomModule
  published
    property    BiDiMode;
    property    Constraints;
    property    Enabled;
    property    Color;
    property    Ctl3D;
    property    Font;
    property    ParentBiDiMode;
    property    ParentColor;
    property    ParentCtl3D;
    property    ParentFont;
    property    ParentShowHint;
    property    PopupMenu;
    property    ShowHint;
    property    TabOrder;
    property    TabStop;
    property    Visible;
    property    OnContextPopup;
    property    OnEnter;
    property    OnExit;
    property    OnMouseDown;
    property    OnMouseMove;
    property    OnMouseUp;
    property    OnMouseWheel;
    property    OnMouseWheelDown;
    property    OnMouseWheelUp;
    property    OnResize;
    property    Width;
    property    Height;
    property    Tag;
    property    Title;
    property    Comment;
    property    PageName;
    property    ModuleType;
    property    BevelInner;
    property    BevelOuter;
    property    BevelWidth;
  end;


  TKnobsModuleButton = class( TSpeedButton)
  // Button as used in module selector, on click it creates a module
  // of the type that got registered with it.
  private
    FPageName   : string;
    FModuleType : TKnobsModuleType;
    FModuleName : string;
  public
    procedure   LoadGlyph( const aGlyph: TBitmap);
    procedure   Initialize( const aPageName: string; aModuleType: TKnobsModuleType; const aModuleName: string; const aGlyph: TBitmap);
  public
    property    PageName   : string      read FPageName;
    property    ModuleType : TKnobsModuleType read FModuleType;
    property    ModuleName : string      read FModuleName;
  end;

  TKnobsModuleSeparator = class( TKnobsModuleButton);


  TKnobsModuleSelector = class( TPageControl)
  // A tabbed thingy having TModuleButtons on it's pages.
  private
    FRegisteredModules   : TKnobsModuleList;
    FOnModuleButtonClick : TKnobsOnModuleButtonClick;
    FOnGetGlyph          : TKnobsOnGetGlyph;
    FButtonsEnabled      : Boolean;
    FAllowDuplicates     : Boolean;
    FTimer               : TTimer;
  private
    procedure   StartTimeout;
    procedure   StopTimeout;
    procedure   TimeOutFired( aSender: TObject);
    procedure   AllButtonsUp;
  private
    procedure   SetButtonsEnabled( aValue: Boolean);
    function    AddPage          ( const aName: string): TTabSheet;
    function    PageButtonCount  ( const aName: string): Integer;
    function    PageExists       ( const aName: string): Boolean;
    function    PageByName       ( const aName: string): TTabSheet;
    function    FindModulePage   ( aModuleType: TKnobsModuleType): TTabSheet;
    function    FindModuleButton ( aModuleType: TKnobsModuleType): TKnobsModuleButton;
    procedure   DoModuleMouseDown( aSender: TObject; aButton: TMouseButton; aShift: TShiftState; X, Y: Integer);
    procedure   FormatPage( aPage: TTabSheet);
  protected
    procedure   Notification( aComponent: TComponent; anOperation: TOperation); override;
  public
    constructor Create( anOwner: TComponent);                          override;
    destructor  Destroy;                                               override;
    procedure   RegisterModuleType  ( const aPageName: string; aModuleType: TKnobsModuleType; const aModuleName: string);
    procedure   UnregisterModuleType( aModuleType: TKnobsModuleType);
  published
    property    OnModuleButtonClick : TKnobsOnModuleButtonClick read FonModuleButtonClick write FOnModuleButtonClick;
    property    OnGetGlyph          : TKnobsOnGetGlyph          read FOnGetGlyph          write FonGetGlyph;
    property    ButtonsEnabled      : Boolean                   read FButtonsEnabled      write SetButtonsEnabled;
    property    AllowDuplicates     : Boolean                   read FAllowDuplicates     write FAllowDuplicates     default True;
  end;


  TKnobsHintWindow = class( THintWindow)
  private
    FModuleType : string;
    FBitmap     : TBitmap;
    FComment    : string;
  protected
    function    MakeBitmap( const aModuleType: string): TBitmap;
    procedure   ParseHint( var aHintStr: string);
    procedure   Paint;                                            override;
  public
    constructor Create( anOwner: TComponent);                     override;
    destructor  Destroy;                                          override;
    procedure   ActivateHint( aRect: TRect; const aHint: string); override;
  published
    property    Caption;
  end;


  procedure Register;

var

  GOnCreateModuleBitmap : TOnCreateModuleBitmap;
  GOnReadModuleComment  : TOnReadModuleComment;


implementation



{$R KnobsBitmaps.res}


const

  InitRepeatPause     = 200;          // Initial pause before repeat kicks in
  SlowRepeatPause     = 100;          // RepeatPause slow repeat
  FastRepeatPause     =  30;          // RepeatPause fast repeat;
  SlowRepeats         =  15;          // After SlowRepeats repeats we start the fast repeat
  ClickerSize         =  12;          // Height for clickers

  // Bitmap Id's as used in the linked resource 'Bitmaps.res'

  idbmDn              =   1;
  idbmUp              =   2;
  idbmConIn           =   3;
  idbmConOut          =   4;
  idbmIndActive       =   5;
  idbmIndInactive     =   6;
  idbmKnob            =   7;
  idbmSmallKnob       =   8;
  idbmSkin            =   9;
  idbmIndLowActive    =  10;
  idbmIndLowInactive  =  11;
  idbmIndMidActive    =  12;
  idbmIndMidInactive  =  13;
  idbmIndHighActive   =  14;
  idbmIndHighInactive =  15;
  idbmNoKnob          =  16;
  idbmSkin2           =  17;
  idbmKnobNew         =  18;
  idbmSmallKnobNew    =  19;


var

  GAnchor             : TPoint         = ( x: 0; y: 0);
  GStartPoint         : TPoint         = ( x: 0; y: 0);
  GDistance           : TPoint         = ( x: 0; y: 0);
  GLabelEditor        : TKnobsLabelEditor   = nil;
  GDisplayEditor      : TKnobsDisplayEditor = nil;

   // Shared bitmaps, loaded from 'Bitmaps.res' at startup

  bmDn              : TBitmap = nil;
  bmUp              : TBitmap = nil;
  bmConIn           : TBitmap = nil;
  bmConOut          : TBitmap = nil;
  bmIndActive       : TBitmap = nil;
  bmIndInactive     : TBitmap = nil;
  bmIndLowActive    : TBitmap = nil;
  bmIndLowInactive  : TBitmap = nil;
  bmIndMidActive    : TBitmap = nil;
  bmIndMidInactive  : TBitmap = nil;
  bmIndHighActive   : TBitmap = nil;
  bmIndHighInactive : TBitmap = nil;
  bmKnob            : TBitmap = nil;
  bmSmallKnob       : TBitmap = nil;
  bmNoKnob          : TBitmap = nil;
  bmSkin            : TBitmap = nil;


  procedure NeedsLabelEditor;
  begin
    if GLabelEditor = nil
    then TKnobsLabelEditor.Create( Application);
  end;

  procedure NeedsDisplayEditor;
  begin
    if GDisplayEditor = nil
    then TKnobsDisplayEditor.Create( Application);
  end;

  procedure Register;
  begin
    RegisterComponents(
      'Knobs2013',
      [
        TKnobsLed,
        TKnobsSelector,
        TKnobsKnob,
        TKnobsSmallKnob,
        TKnobsNoKnob,
        TKnobsValuedButton,
        TKnobsValuedButtons,
        TKnobsDisplay,
        TKnobsFileSelector,
        TKnobsIndicator,
        TKnobsIndicatorBar,
        TKnobsIndicatorText,
        TKnobsDataViewer,
        TKnobsInput,
        TKnobsOutput,
        TKnobsTextLabel,
        TKnobsEditLabel,
        TKnobsBox,
        TKnobsWirePanel,
        TKnobsModule,
        TKnobsModuleSelector
      ]
    );
  end;

  function  CreateModuleBitmap( aModuleType: TKnobsModuleType): TBitmap;
  begin
    if Assigned( GOnCreateModuleBitmap)
    then Result := GOnCreateModuleBitmap( aModuleType)
    else Result := nil;
  end;

  function  ReadModuleComment( aModuleType: TKnobsModuleType): string;
  begin
    if Assigned( GOnReadModuleComment)
    then Result := GOnReadModuleComment( aModuleType)
    else Result := '';
  end;

  procedure   LoadBitmap( var aBitmap: TBitmap; anIndex: Integer);
  // Loads one bitmap from 'Bitmaps.res'
  begin
    if not Assigned( aBitmap)
    then aBitmap := TBitmap.Create;
    with aBitmap
    do begin
      LoadFromResourceID( HInstance, anIndex);
      TransparentMode := tmAuto;
      Transparent     := True;
    end;
  end;

  procedure CreateBitmaps;
  // Loads all bitmaps from 'Bitmaps.res'
  begin
    LoadBitmap( bmDn             , idbmDn             );
    LoadBitmap( bmUp             , idbmUp             );
    LoadBitmap( bmConIn          , idbmConIn          );
    LoadBitmap( bmConOut         , idbmConOut         );
    LoadBitmap( bmIndActive      , idbmIndActive      );
    LoadBitmap( bmIndInactive    , idbmIndInactive    );
    LoadBitmap( bmIndLowActive   , idbmIndLowActive   );
    LoadBitmap( bmIndLowInactive , idbmIndLowInactive );
    LoadBitmap( bmIndMidActive   , idbmIndMidActive   );
    LoadBitmap( bmIndMidInactive , idbmIndMidInactive );
    LoadBitmap( bmIndHighActive  , idbmIndHighActive  );
    LoadBitmap( bmIndHighInactive, idbmIndHighInactive);
    LoadBitmap( bmKnob           , idbmKnobNew        );
    LoadBitmap( bmSmallKnob      , idbmSmallKnobNew   );
    LoadBitmap( bmNoKnob         , idbmNoKnob         );
    LoadBitmap( bmSkin           , idbmSkin2          );
  end;

  procedure FreeBitmaps;
  // Free the bitmaps loaded at program startup
  begin
    FreeAndnil( bmKnob           );
    FreeAndnil( bmSmallKnob      );
    FreeAndnil( bmNoKnob         );
    FreeAndnil( bmIndActive      );
    FreeAndnil( bmIndInactive    );
    FreeAndnil( bmIndLowActive   );
    FreeAndnil( bmIndLowInactive );
    FreeAndnil( bmIndMidActive   );
    FreeAndnil( bmIndMidInactive );
    FreeAndnil( bmIndHighActive  );
    FreeAndnil( bmIndHighInactive);
    FreeAndnil( bmConIn          );
    FreeAndnil( bmConOut         );
    FreeAndnil( bmDn             );
    FreeAndnil( bmUp             );
    FreeAndnil( bmSkin           );
  end;

  procedure   CalcBezierCurve( var Points: TKnobsBezierPoints; const Params: TKnobsBezierParams);
  // Used to calculate wire bending - for rotate etc. see 'knobutils.pas'.
  var
    Dir1 : TPoint;
    Dir2 : TPoint;
  begin
    Dir1       := Rotate( SubPoints( Points[ 0], Points[ 3]), Params.Angle1);
    Dir2       := Rotate( SubPoints( Points[ 0], Points[ 3]), Params.Angle2);
    Points[ 1] := SubPoints( Points[ 0], DivPoint( MultPoint( Dir1, Params.Strength1), 100));
    Points[ 2] := AddPoints( Points[ 3], DivPoint( MultPoint( Dir2, Params.Strength2), 100));
  end;

  function  ConnectorName( aConnector: TKnobsConnector): string;
  begin
    if Assigned( aConnector)
    then begin
      if aConnector.HasParent
      then Result := Format( '%s.%s', [ aConnector.Parent.Name, aConnector.Name])
      else Result := Format( '<nil>.%s', [ aConnector.Name]);
    end
    else Result := '<nil>.<nil>';
  end;

  function  WireName( aSource, aDestination: TKnobsConnector): string;
  begin
    Result := Format( '%s -> %s', [ ConnectorName( aSource), ConnectorName( aDestination)]);
  end;


{ ========
  TKnobsLed = class( TGraphicControl)
  private
    FTime       : Cardinal;
    FTimer      : TTimer;
    FColorOff   : TColor;
    FColorOn    : TColor;
    FHighLight  : Boolean;
  published
    property    Time       : Cardinal       read FTime       write SetTime       default 0;
    property    Active     : Boolean        read FActive     write SetActive     default True;
    property    ColorOn    : TColor         read FColorOn    write SetColorOn    default clLime;
    property    ColorOff   : TColor         read FColorOff   write SetColorOff   default clGreen;
    property    HighLight  : Boolean        read FHighLight  write SetHighLight  default True;
    property    Shape      : TKnobsLedShape read FShape      write SetShape;
    property    Height                                                           default 15;
    property    Width                                                            default 15;
    property    Hint;
    property    ParentShowHint                                                   default True;
    property    Dragcursor;
    property    Dragmode;
    property    OnDragDrop;
    property    OnDragOver;
    property    OnEndDrag;
    property    OnMouseDown;
    property    OnMouseMove;
    property    OnMouseUp;
  private
}

    procedure   TKnobsLed.TimerFired( aSender: TObject);
    begin
      if FTime <> 0
      then Active := False;
    end;

    procedure   TKnobsLed.StartTimer( aValue: Cardinal);
    begin
      if not ( csDesigning in ComponentState)
      then begin
        if not Assigned( FTimer)
        then FTimer := TTimer.Create( nil);
        if Assigned( FTimer)
        then begin
          with FTimer
          do begin
            Enabled  := False;
            OnTimer  := TimerFired;
            Interval := FTime;
            Enabled  := True;
          end;
        end;
      end;
    end;

    procedure   TKnobsLed.StopTimer;
    begin
      if Assigned( FTimer)
      then begin
        with FTimer
        do begin
          Enabled := False;
          OnTimer := nil;
        end;
        FreeAndNil( FTimer);
      end;
    end;

    procedure   TKnobsLed.SetTime( aValue: Cardinal);
    begin
      if aValue <> FTime
      then begin
        FTime := aValue;
        if ( FTime > 0) and Active
        then Starttimer( FTime)
      end;
    end;

    procedure   TKnobsLed.SetActive( aValue: Boolean);
    begin
      if csDestroying in ComponentState
      then Exit;

      if aValue <> FActive
      then begin
        FActive := aValue;
        Invalidate;
        if FActive and ( FTime > 0)
        then StartTimer( FTime)
        else StopTimer;
      end;
    end;

    procedure   TKnobsLed.SetColorOn( aValue: TColor);
    begin
      if csDestroying in ComponentState
      then Exit;

      if aValue <> FColorOn
      then begin
        FColorOn := aValue;
        Invalidate;
      end;
    end;

    procedure   TKnobsLed.SetColorOff( aValue: TColor);
    begin
      if csDestroying in ComponentState
      then Exit;

      if aValue <> FColorOff
      then begin
        FColorOff := aValue;
        Invalidate;
      end;
    end;

    procedure   TKnobsLed.SetHighLight( aValue: Boolean);
    begin
      if csDestroying in ComponentState
      then Exit;

      if aValue <> FHighLight
      then begin
        FHighLight := aValue;
        Invalidate;
      end;
    end;

    procedure   TKnobsLed.SetShape( aValue: TKnobsLedShape);
    begin
      if csDestroying in ComponentState
      then Exit;

      if aValue <> FShape
      then begin
        FShape := aValue;
        Invalidate;
      end;
    end;

//  protected

    procedure   TKnobsLed.Paint; // override;
    begin
      with Canvas
      do begin
        Pen.Width := 1;
        Pen.Color := clBlack;

        if Active
        then Brush.Color := FColorOn
        else Brush.Color := FColorOff;

        case Shape of

          lsRound :
            begin
              Ellipse( 0, 0, Width, Height);

              if FHighLight
              then begin
                Pen.Color := clWhite;
                Pen.Width := 2;
                Arc(
                  Width  * 3 div 4         ,
                  Height * 3 div 4         ,
                  Width  - Width  * 3 div 4,
                  Height - Height * 3 div 4,
                  Width  div 2             ,
                  0                        ,
                  0                        ,
                  Height div 2
                );
              end;
            end;

          lsSquare :
            begin
              Rectangle( 0, 0, Width, Height);
            end;

        end;
      end;
    end;

//  public

    constructor TKnobsLed.Create( aOwner: TComponent); // override;
    begin
      inherited;
      Height    := 15;
      Width     := 15;
      Shape     := lsRound;
      ColorOn   := clLime;
      ColorOff  := clGreen;
      HighLight := True;
      Active    := True;
    end;

    destructor  TKnobsLed.Destroy; // override;
    begin
      StopTimer;
      inherited;
    end;

    procedure   TKnobsLed.Toggle;
    begin
      if csDestroying in ComponentState
      then Exit;

      Active := not Active;
    end;



{ ========
  TClicker = class( TGraphicControl)
  private
      FRepeatTimer : TTimer;
      FRepeatCount : Integer;
      FCanRepeat   : Boolean;
      FMouseDown   : Boolean;
      FShowFocus   : Boolean;
      FBitmap      : TBitmap;
  private
    property    ShowFocus: Boolean read FShowFocus write SetShowFocus;
  public
    property    CanRepeat: Boolean read FCanRepeat write FCanRepeat   default True;
  private
}

    procedure   TKnobsClicker.TimerExpired( aSender: TObject);
    begin
      with FRepeatTimer
      do begin
        if FRepeatCount > SlowRepeats
        then Interval := FastRepeatPause
        else Interval := SlowRepeatPause;
      end;

      if FMouseDown and MouseCapture
      then begin
        try
          Click;
        except
          FRepeatTimer.Enabled := False;
          raise;
        end;
      end;

      Inc( FRepeatCount);
    end;

    procedure   TKnobsClicker.SetShowFocus( aValue: Boolean);
    begin
      if Parent is TKnobsNoKnob
      then begin
        Flat   := False;
        Glyph  := FBitmap;
        Invalidate;
      end
      else begin
        if aValue <> FShowFocus
        then begin
          FShowFocus  := aValue;
          Flat        := not FShowFocus;
          if ShowFocus
          then Glyph := FBitmap
          else Glyph := nil;
          Invalidate;
        end;
      end;
    end;

//  private

    function    TKnobsClicker.FindWirePanel: TKnobsWirePanel;
    var
      aParent : TWinControl;
    begin
      Result  := nil;
      aParent := Parent;

      while Assigned( aParent) and not ( aParent is TKnobsWirePanel)
      do aParent :=  aParent.Parent;

      if aParent is TKnobsWirePanel
      then Result := TKnobsWirePanel( aParent);
    end;

//  protected

    procedure   TKnobsClicker.MouseDown( Button: TMouseButton; Shift: TShiftState; X, Y: Integer); // override;
    var
      aWP : TKnobsWirePanel;
    begin
      inherited;

      if FMouseDown
      then Exit;

      if CanRepeat and ( Button = mbLeft)
      then begin
        FMouseDown   := True;
        FRepeatCount := 0;

        if not Assigned( FRepeatTimer)
        then FRepeatTimer := TTimer.Create( Self);

        with FRepeatTimer
        do begin
          OnTimer  := TimerExpired;
          Interval := InitRepeatPause;
          Enabled  := True;
        end;
      end
      else if Button = mbRight
      then begin
        aWP := FindWirePanel;
        if Assigned( aWP)
        then aWP.HandleRightClick( Self);
      end;
    end;

    procedure   TKnobsClicker.MouseUp( Button: TMouseButton; Shift: TShiftState; X, Y: Integer); // override;
    begin
      inherited;

      if Assigned( FRepeatTimer)
      then FRepeatTimer.Enabled  := False;

      FMouseDown := False;
    end;

    procedure   TKnobsClicker.AssignBitmap( aBitmap: TBitmap);
    begin
      FBitmap := aBitmap;
    end;

//  public

    constructor TKnobsClicker.Create( anOwner: TComponent); // override;
    begin
      inherited;

      CanRepeat := True;
      if anOwner is TKnobsNoKnob
      then begin
        Transparent := False;
        Flat        := False;
      end
      else begin
        Transparent := True;
        Flat        := True;
      end;
    end;

    destructor  TKnobsClicker.Destroy; // override;
    begin
      if Assigned( FRepeatTimer)
      then FRepeatTimer.Free;

      inherited;
    end;

    procedure   TKnobsClicker.Click; // override;
    begin
      inherited;

      if HasParent
      then Parent.SetFocus;
    end;


{ ========
  TValuedControl = class( TCustomControl)
  private
    FStepCount       : Integer;             // Number of steps in the total knob travel
    FLocked          : Boolean;
    FKnobPosition    : Integer;             // Current knob position
    FDefaultPosition : Integer;             // default knob position;
    FControlType     : string;              // The control's behaviour
    FOnValueChanged  : TKnobsOnValueChange; // Called with the new value
  public
    property    AsDisplay       : string          read GetAsDisplay;
    property    AsHint          : string          read GetAsHint;
    property    AsValue         : TSignal         read GetAsValue;
  published
    property    Action;
    // NOTE: declaration order is important here to avoid loss of resolution on a load action ...
    property    StepCount       : Integer         read FStepCount       write SetStepCount       default 1;
    property    Locked          : Boolean         read FLocked          write SetLocked;
    property    KnobPosition    : Integer         read FKnobPosition    write SetKnobPosition    default 0;
    property    DefaultPosition : Integer         read FDefaultPosition write SetDefaultPosition default 0;
    property    ControlType     : string          read FControlType     write SetControlType;
    // END NOTE.
    property    TabOrder;
    property    TabStop;
    property    Visible;
    property    OnValueChanged  : TKnobsOnValueChange  read FOnValueChanged  write SetOnValueChanged;
  private
}

    procedure   TKnobsValuedControl.CommonKnobPosition;
    begin
      KnobPosition    := KnobPosition;
      DefaultPosition := DefaultPosition;
      Invalidate;
      Hint := AsHint;
    end;

    procedure   TKnobsValuedControl.FixKnobPosition;
    begin
      CommonKnobPosition;
      ValueChanged( False);
    end;

    procedure   TKnobsValuedControl.FinalizeKnobPosition;
    begin
      CommonKnobPosition;
      ValueChanged( True);
    end;

//  private

    procedure   TKnobsValuedControl.SetDefaultPosition( aValue: Integer); // virtual;
    begin
      aValue := Clip( aValue, 0, StepCount);
      if aValue <> FDefaultPosition
      then FDefaultPosition := aValue;
      Locked := False;
    end;

    procedure   TKnobsValuedControl.SetStepCount( aValue: Integer); // virtual;
    begin
      if aValue <= 0
      then aValue := 1;
      if aValue <> FStepCount
      then begin
        FStepCount   := aValue;
        FinalizeKnobPosition;
      end;
    end;

    procedure   TKnobsValuedControl.SetKnobPosition( aValue: Integer); // virtual;
    begin
      if not Locked
      then begin
        aValue := Clip( aValue, 0, StepCount - 1);
        if aValue <> FKnobPosition
        then begin
          FKnobPosition := aValue;
          if csDesigning in ComponentState
          then DefaultPosition := FKnobPosition;
          FixKnobPosition;
        end;
      end;
    end;

    procedure   TKnobsValuedControl.SetLocked( aValue: Boolean);
    begin
      if aValue <> FLocked
      then begin
        FLocked := aValue;
        Invalidate;
      end;
    end;

    procedure   TKnobsValuedControl.SetControlType( const aValue: string);
    begin
      if aValue <> FControlType
      then begin
        FControlType := aValue;
        FinalizeKnobPosition;
      end;
    end;

    function    TKnobsValuedControl.GetAsHint: string;
    begin
      if csDesigning in ComponentState
      then Result := Converters.PosToDisplay( ControlType, KnobPosition, StepCount)
      else Result := Converters.PosToHint   ( ControlType, KnobPosition, StepCount);
    end;

    function    TKnobsValuedControl.GetAsDisplay: string;
    begin
      Result := Converters.PosToDisplay( ControlType, KnobPosition, StepCount);
    end;

    function    TKnobsValuedControl.GetAsValue: TSignal;
    begin
      Result := Converters.PosToValue( ControlType, KnobPosition, StepCount);
    end;

    procedure   TKnobsValuedControl.SetOnValueChanged( aValue: TKnobsOnValueChange); // virtual;
    begin
      FOnValueChanged := aValue;
      ValueChanged( True);
    end;

    procedure   TKnobsValuedControl.NotifyPeers; // virtual;
    begin
      // For inherited
    end;

    procedure   TKnobsValuedControl.ValueChanged( IsFinal: Boolean);
    var
      aModule : TKnobsCustomModule;
    begin
      NotifyPeers;

      if Assigned( FOnValueChanged)
      then FOnValueChanged( Self, Name, ControlType, AsValue, IsFinal)
      else begin
        aModule := FindModule;
        if Assigned( aModule)
        then aModule.ValueChanged( Self, MakePath( [ aModule.Name, Name]), ControlType, AsValue, IsFinal);
      end;
    end;

//  protected

    function    TKnobsValuedControl.FindModule: TKnobsCustomModule;
    var
      aParent : TWinControl;
    begin
      Result  := nil;
      aParent := Parent;

      while Assigned( aParent) and not ( aParent is TKnobsCustomModule)
      do aParent :=  aParent.Parent;

      if aParent is TKnobsCustomModule
      then Result := TKnobsCustomModule( aParent);
    end;

    function    TKnobsValuedControl.FindWirePanel: TKnobsWirePanel;
    var
      aParent : TWinControl;
    begin
      Result  := nil;
      aParent := Parent;

      while Assigned( aParent) and not ( aParent is TKnobsWirePanel)
      do aParent :=  aParent.Parent;

      if aParent is TKnobsWirePanel
      then Result := TKnobsWirePanel( aParent);
    end;

    procedure   TKnobsValuedControl.BeginStateChange;
    var
      aWp : TKnobsWirePanel;
    begin
      aWp := FindWirePanel;
      if Assigned( aWp)
      then aWp.BeginStateChange;
    end;

    procedure   TKnobsValuedControl.EndStateChange;
    var
      aWp : TKnobsWirePanel;
    begin
      aWp := FindWirePanel;
      if Assigned( aWp)
      then aWp.EndStateChange( False);
    end;

//  public

    constructor TKnobsValuedControl.Create( anOwner: TComponent); // override;
    begin
      inherited;
      StepCount       := 1;
      KnobPosition    := 0;
      DefaultPosition := KnobPosition;
    end;

    procedure   TKnobsValuedControl.SetDefault;
    begin
      Locked       := False;
      KnobPosition := DefaultPosition;
    end;

    procedure   TKnobsValuedControl.FixDisplay;
    var
      Old : Integer;
    begin
      Old := KnobPosition;
      if KnobPosition > 0
      then KnobPosition := KnobPosition - 1
      else KnobPosition := KnobPosition + 1;
      KnobPosition := Old;
    end;

    procedure   TKnobsValuedControl.Dump( var aFile: TextFile; anInd: Integer);
    begin
      WriteLnInd( aFile, anInd, Format( 'ValuedControl "%s" = %f', [ Name, AsValue]));
    end;


{ ========
  TKnobsSelector = class( TKobsValuedControl)
  private
    FCaptions : TStringList;
  published
    property    Captions: TStringList read FCaptions write SetCaptions;
  private
}

    procedure   TKnobsSelector.SetCaptions( const aValue: TStringList);
    begin
      if Assigned( FCaptions)
      then begin
        FCaptions.Assign( aValue);
        StepCount := FCaptions.Count;
        Invalidate;
      end;
    end;

    procedure   TKnobsSelector.SetBorderColor( aValue: TColor);
    begin
      if aValue <> FBorderColor
      then begin
        FBorderColor := aValue;
        Invalidate;
      end;
    end;

//  private

    procedure   TKnobsSelector.WMSetFocus( var aMessage: TWMSetFocus); // message WM_SETFOCUS;
    begin
      inherited;
    end;

    procedure   TKnobsSelector.WMKillFocus( var aMessage: TWMKillFocus); // message WM_KILLFOCUS;
    begin
      inherited;
    end;

    procedure   TKnobsSelector.WMGetDlgCode( var aMessage: TWMGetDlgCode); // message WM_GETDLGCODE;
    begin
      aMessage.Result := DLGC_WANTARROWS;
    end;

//  protected

    procedure   TKnobsSelector.DoClick( aSender: TObject);
    begin
      if KnobPosition < StepCount - 1
      then KnobPosition := KnobPosition + 1
      else KnobPosition := 0;
      FinalizeKnobPosition;
      Invalidate;
    end;

    procedure   TKnobsSelector.Paint; // override;
    var
      p         : Integer;
      aTextSize : TSize;
      aLeft     : Integer;
      aTop      : Integer;
    begin
      p := Round( KnobPosition + 0.1);
      if ( p >= 0) and ( p < Captions.Count)
      then Caption := Captions[ p]
      else Caption := 'invalid';

      with Canvas
      do begin
        Pen.Color   := BorderColor;
        Pen.Width   := 1;
        Brush.Color := Color;
        Font        := Self.Font;
        Rectangle( 0, 0, Width, Height);
        aTextSize := TextExtent( Caption);
        aLeft     := ( Self.Width  - aTextSize.cx) div 2;
        aTop      := ( Self.Height - aTextSize.cy) div 2;
        TextOut( aLeft, aTop, Caption);
      end;
    end;

//  public

    constructor TKnobsSelector.Create( anOwner: TComponent); // override;
    begin
      inherited;
      ControlStyle    := ControlStyle - [ csOpaque] + [ csSetCaption];
      FCaptions       := TStringList.Create;
      KnobPosition    :=  0;
      Height          := 16;
      Width           := 35;
      Color           := clGray;
      Font.Color      := clWhite;
      BorderColor     := clWhite;
      TabStop         := True;
      OnClick         := DoClick;
    end;

    destructor  TKnobsSelector.Destroy; // override;
    begin
      FreeAndNil( FCaptions);
      inherited;
    end;


{ ========
  TKnobsKnob = class( TCustomControl)
  private
    FDownClicker       : TKnobsClicker;
    FUpClicker         : TKnobsClicker;
    FMouseDown         : Boolean;
    FBitmap            : TBitmap;
    FMouseInControl    : Boolean;
    FTransparent       : Boolean;
    FDisplay           : TKnobsDisplay;
    FPairedWith        : TKnobsKnob;
    FPairingMode       : TKnobsPairingMode;
    FPairingLock       : Integer;
    FControlMode       : TDistanceMode;
    FFinalizationTimer : TTimer;
    FTresholdPast      : Boolean;
    FLocked            : Boolean;
  published
    property    Action;
    property    Transparent  : Boolean           read FTransparent   write SetTransparent;
    property    Color;
    property    TabOrder;
    property    TabStop;
    property    ParentColor                                                             default True;
    property    Display      : TKnobsDisplay     read FDisplay       write SetDisplay;
    property    PairedWith   : TKnobsKnob        read FPairedWith    write SetPairedWith;
    property    PairingMode  : TKnobsPairingMode read FPairingMode   write SetPairingMode;
    property    ControlMode  : TDistanceMode     read FControlMode   write FControlMode;
    property    Locked       : Boolean           read FLocked        write SetLocked;
  private
}

    procedure   TKnobsKnob.InitializeFinalizationTimer;
    begin
      FFinalizationTimer := TTimer.Create( nil);
      StopFinalizationTimer;
      FFinalizationTimer.OnTimer := FinalizationTimerFired;
    end;

    procedure   TKnobsKnob.FinalizeFinalizationTimer;
    begin
      FFinalizationTimer.OnTimer := nil;
      StopFinalizationTimer;
      FreeAndNil( FFinalizationTimer);
    end;

    procedure   TKnobsKnob.StartFinalizationTimer;
    begin
      StopFinalizationTimer;
      FFinalizationTimer.Interval := 739;
      FFinalizationTimer.Enabled  := True;
    end;

    procedure   TKnobsKnob.StopFinalizationTimer;
    begin
      FFinalizationTimer.Enabled := False;
    end;

    procedure   TKnobsKnob.FinalizationTimerFired( aSender: TObject);
    begin
      StopFinalizationTimer;
      FinalizeKnobPosition;
    end;

//  private

    procedure   TKnobsKnob.WMSetFocus( var aMessage: TWMSetFocus ); // message WM_SETFOCUS;
    begin
      FUpClicker  .ShowFocus := True;
      FDownClicker.ShowFocus := True;
      Invalidate;

      inherited;
    end;

    procedure   TKnobsKnob.WMKillFocus( var aMessage: TWMKillFocus); // message WM_KILLFOCUS;
    begin
      FUpClicker  .ShowFocus := False;
      FDownClicker.ShowFocus := False;
      Invalidate;

      inherited;
    end;

    procedure   TKnobsKnob.WMGetDlgCode( var aMessage: TWMGetDlgCode); // message WM_GETDLGCODE;
    begin
      aMessage.Result := DLGC_WANTARROWS;
    end;

    procedure TKnobsKnob.CMMouseEnter( var aMessage: TMessage);
    begin
      inherited;

      if not FMouseInControl and Enabled and ( GetCapture = 0)
      then begin
        FMouseInControl := True;
        Invalidate;
      end;
    end;

    procedure TKnobsKnob.CMMouseLeave( var aMessage: TMessage);
    begin
      inherited;

      if FMouseInControl and Enabled and not Dragging
      then begin
        FMouseInControl := False;
        Invalidate;
      end;
    end;

    function    TKnobsKnob.CreateClicker( aLeft, aTop: Integer; aNotifyEvent: TNotifyEvent; aBitmap: TBitmap): TKnobsClicker;
    begin
      Result := TKnobsClicker.Create( Self);

      with Result
      do begin
        Left    := aLeft;
        Top     := aTop;
        Width   := Self.Width div 2 + 1;
        Height  := ClickerSize;
        OnClick := aNotifyEvent;
        Parent  := Self;
        AssignBitmap( aBitmap);
        SetShowFocus( ClassType = TKnobsNoKnob);
      end;
    end;

    procedure   TKnobsKnob.OnDownClick( aSender: TObject);
    begin
      if not Locked
      then begin
        StartFinalizationTimer;
        try
          KnobPosition := KnobPosition - 1;
        except
          on E: Exception
          do KilledException( E);
        end;
      end;
    end;

    procedure   TKnobsKnob.OnUpClick( aSender: TObject);
    begin
      if not Locked
      then begin
        StartFinalizationTimer;
        try
          KnobPosition := KnobPosition + 1;
        except
          on E: Exception
          do KilledException( E);
        end;
      end;
    end;

    procedure   TKnobsKnob.NotifyPeers; // Override;
    begin
      if Assigned( FDisplay)
      then FDisplay.Caption := AsDisplay;
    end;

    procedure   TKnobsKnob.LockPairing;
    begin
      Inc( FPairingLock);
    end;

    procedure   TKnobsKnob.UnlockPairing;
    begin
      if PairingLocked
      then Dec( FPairingLock);
    end;

    function    TKnobsKnob.PairingLocked: Boolean;
    begin
      Result := FPairingLock > 0;
    end;

    procedure   TKnobsKnob.FixPairing;
    begin
      if not PairingLocked and Assigned( PairedWith)
      then begin
        LockPairing;
        try
          case PairingMode of
            pmOff      : ;
            pmNormal   : PairedWith.KnobPosition := KnobPosition;
            pmMirrored : PairedWith.KnobPosition := PairedWith.StepCount - KnobPosition - 1;
          end;
        finally
          UnlockPairing;
        end;
      end;
    end;

    procedure   TKnobsKnob.SetTransparent( aValue: Boolean);
    begin
      if aValue <> FTransparent
      then begin
        FTransparent := aValue;
        Invalidate;
      end;
    end;

    procedure   TKnobsKnob.SetDisplay( aValue: TKnobsDisplay);
    begin
      FDisplay := aValue;

      if Assigned( FDisplay)
      then begin
        FDisplay.Caption     := AsDisplay;
        FDisplay.FController := Self;
      end;
    end;

    procedure   TKnobsKnob.SetKnobPosition( aValue: Integer); // override;
    var
      OldVal : TSignal;
    begin
      OldVal := KnobPosition;
      inherited;
      if OldVal <> KnobPosition
      then FixPairing;
    end;

    procedure   TKnobsKnob.SetPairedWith( const aValue: TKnobsKnob);
    begin
      if aValue <> FPairedWith
      then begin
        if not PairingLocked
        then begin
          LockPairing;
          try
            if Assigned( aValue)
            then aValue.PairedWith := Self
            else if Assigned( FPairedWith)
            then begin
              FPairedWith.PairingMode := pmOff;
              FPairedWith.PairedWith  := nil;
            end;
            FPairedWith := aValue;
          finally
            UnlockPairing;
          end;
          FixPairing;
        end;
      end;
    end;

    procedure   TKnobsKnob.SetPairingMode( aValue: TKnobsPairingMode);
    begin
      if aValue <> FPairingMode
      then begin
        if not PairingLocked
        then begin
          LockPairing;
          try
            FPairingMode := aValue;
            if Assigned( FPairedWith)
            then FPairedWith.PairingMode := aValue;
          finally
            UnlockPairing;
          end;
          FixPairing;
        end;
      end;
    end;

    function    TKnobsKnob.CenterPoint: TPoint;
    begin
      with Result
      do begin
        X := Width         div 2;
        Y := ( Height - 1) div 2;
      end;
    end;

    function    TKnobsKnob.CreateRgn: HRGN;
    var
      HR : HRGN;
      OP : HPEN;
      DC : HDC;
    begin
      if FTransparent
      then begin
        Result := CreateEllipticRgn( 1, 1, Width - 1, Height - 2);
        if Focused
        then begin
          HR := CreateRectRgn( 1                           , Height - FDownClicker.Height + 2, FDownClicker.Width - 1, Height);
          CombineRgn( Result, HR, Result, RGN_OR);
          DeleteObject( HR);
          HR := CreateRectRgn( Width - FUpClicker.Width + 2, Height - FUpClicker.Height   + 2, Width                 , Height);
          CombineRgn( Result, HR, Result, RGN_OR);
          DeleteObject( HR);
        end
        else begin
          with Canvas
          do begin
            Pen.Width := 2;
            DC := CreateCompatibleDC( Handle);
          end;
          OP := SelectObject( DC, Canvas.Pen.Handle);
          try
            BeginPath( DC);
            with CenterPoint
            do MoveToEx( DC, X, Y, nil);
            LineTo( DC, 3, Height - 4);
            EndPath( DC);
            WidenPath( DC);
            HR := PathToRegion( DC);
            CombineRgn( Result, Result, HR, RGN_OR);
            DeleteObject( HR);

            BeginPath( DC);
            with CenterPoint
            do MoveToEx( DC, X, Y, nil);
            LineTo( DC, Width - 3, Height - 4);
            EndPath( DC);
            WidenPath( DC);
            HR := PathToRegion( DC);
            CombineRgn( Result, Result, HR, RGN_OR);
            DeleteObject( HR);
          finally
            SelectObject( DC, OP);
            DeleteDC( DC);
          end;
        end;
      end
      else Result := CreateRectRgn( 0, 0, Width, Height);
    end;

    procedure   TKnobsKnob.BeginMove( aPoint: TPoint); // override;
    var
      P : TPoint;
      A : Extended;
    begin
      BeginStateChange;               // This will lock UndoRedo as well

      case ControlMode of

        dmCircular :

          begin
            P := PolarToPoint( CenterPoint, ValuesToAngle( 0, KnobPosition, StepCount - 1), Width / 2);
            with ClientToScreen( P)
            do SetCursorPos( X, Y);
          end;

        dmHorizontal :

          begin
            if StepCount > 0
            then begin
              A := Clip( KnobPosition / StepCount, 0, 1);
              P := CenterPoint + Point( Round( -100 + A * 200), GAnchor.Y);
            end
            else P := CenterPoint;
            with ClientToScreen( P)
            do SetCursorPos( X, Y);
          end;

        dmVertical :

          begin
            if StepCount > 0
            then begin
              A := Clip( KnobPosition / StepCount, 0, 1);
              P := CenterPoint + Point( GAnchor.X, Round( 100 -  A * 200));
            end
            else P := CenterPoint;
            with ClientToScreen( P)
            do SetCursorPos( X, Y);
          end;

      end;

      GDistance   := SubPoints( CenterPoint, P);
      GStartPoint := GAnchor;
      GAnchor     := CenterPoint;
      Application.ActivateHint( ClientToScreen( CenterPoint));
    end;

    procedure   TKnobsKnob.EndMove( aPoint: TPoint); // override;
    var
      aVal : TSignal;
    begin
      with ClientToScreen( GStartPoint)
      do SetCursorPos( X, Y);
      Application.HideHint;
      aVal := DistanceToValue( 0, StepCount - 1, aPoint, ControlMode);
      KnobPosition := Round( aVal);
      FinalizeKnobPosition;
      EndStateChange;                 // This will unlock UndoRedo as well
    end;

    procedure   TKnobsKnob.UpdateMove( aPoint: TPoint); // override;
    var
      aVal : TSignal;
    begin
      aVal := DistanceToValue( 0, StepCount - 1, aPoint, ControlMode);
      KnobPosition := Round( aVal);
    end;

//  protected

    procedure   TKnobsKnob.Paint; // override;
    const
      LockSpotSize = 4;
    var
      S : TPoint;
      E : TPoint;
      A : TSignal;
    begin
      if not ( csDesigning in ComponentState)
      then SetWindowRgn( Handle, CreateRgn, True);

      with Canvas
      do begin
        FBitmap.Transparent := not FTransparent;
        Draw( 1, 1, FBitmap);

        if Focused and not Locked
        then begin
          Pen.Color := clRed;
          Pen.Width := 3;
        end
        else begin
          Pen.Color := clBlack;
          Pen.Width := 1;
        end;

        A := ValuesToAngle( 0, KnobPosition, StepCount - 1);
        S := PolarToPoint( CenterPoint, A, 1);
        E := PolarToPoint( CenterPoint, A, Width div 2 - 2);
        MoveTo( S.x, S.y);
        LineTo( E.x, E.y);

        if Locked
        then begin
          Pen.Color   := clRed;
          Pen.Width   := 1;
          Brush.Color := clRed;
          Ellipse(
            CenterPoint.X - LockSpotSize,
            CenterPoint.Y - LockSpotSize,
            CenterPoint.X + LockSpotSize,
            CenterPoint.Y + LockSpotSize
          );
        end;
      end;
    end;

    procedure   TKnobsKnob.MouseDown( Button: TMouseButton; Shift: TShiftState; X, Y: Integer); // override;
    var
      aWP: TKnobsWirePanel;
    begin
      inherited;

      if FMouseDown
      then Exit;

      if ( Button = mbLeft) and not Locked
      then begin
        if TabStop and ( GetFocus <> Handle) and CanFocus
        then SetFocus;

        GAnchor       := Point( X, Y);
        FTresholdPast := False;
        BeginMove( GAnchor);
        FMouseDown := True;
      end
      else if Button = mbRight
      then begin
        aWP := FindWirePanel;
        if Assigned( aWP)
        then aWP.HandleRightClick( Self);
      end;
    end;

    procedure   TKnobsKnob.MouseUp( Button: TMouseButton; Shift: TShiftState; X, Y: Integer); // override;
    begin
      inherited;

      if FMouseDown
      then begin
        FMouseDown  := False;
        if FTresholdPast
        then EndMove( Point( X - GAnchor.x, Y - GAnchor.y))
        else EndMove( GAnchor);
      end;
    end;

    procedure   TKnobsKnob.MouseMove( Shift: TShiftState; X, Y: Integer); // override;
    var
      aDistance : TPoint;
    begin
      inherited;

      if FMouseDown
      then begin
        aDistance := SubPoints( Point( X, Y), GAnchor);

        if not FTresholdPast
        then FTresholdPast := Distance( aDistance) > 10;

        if FTresholdPast
        then UpdateMove( Point( X - GAnchor.x, Y - GAnchor.y));
      end;
    end;

    procedure   TKnobsKnob.KeyDown( var Key: Word; Shift: TShiftState); // override;
    begin
      if not Locked
      then begin
        case Key of
          VK_UP   : begin FUpClicker  .Click; Key := 0; end;
          VK_DOWN : begin FDownClicker.Click; Key := 0; end;
          else      inherited;
        end;
      end;
    end;

    procedure   TKnobsKnob.AssignBitmap; // virtual;
    begin
      FBitmap := bmKnob;
    end;

    procedure   TKnobsKnob.Loaded; // override;
    begin
      inherited;

      NotifyPeers;
      Hint := AsHint;
    end;

//  public

    constructor TKnobsKnob.Create( anOwner: TComponent); // override;
    begin
      inherited;

      InitializeFinalizationTimer;

      ControlStyle    := ControlStyle - [ csSetCaption, csOpaque];
      AssignBitmap;
      Width           := FBitmap.Width  + 2;
      Height          := FBitmap.Height + 3;
      FDownClicker    := CreateClicker( 0              , Height - ClickerSize + 1, OnDownClick, bmDn);
      FUpClicker      := CreateClicker( Width div 2 + 1, Height - ClickerSize + 1, OnUpClick  , bmUp);
      DoubleBuffered  := True;
      TabStop         := True;
      ShowHint        := True;
      Color           := clWhite;
      ParentColor     := True;
    end;

    destructor  TKnobsKnob.Destroy; // override;
    begin
      FinalizeFinalizationTimer;
      if Assigned( FDisplay)
      then FDisplay.FController := nil;
      inherited;
    end;


{ ========
  TSmallKnob = class( TKnob)
  public
}

    procedure   TKnobsSmallKnob.AssignBitmap; // Overide;
    begin
      FBitmap := bmSmallKnob;
    end;



{ ========
  TKnobsNoKnob = class( TKnobsKnob)
  public
}

    procedure   TKnobsNoKnob.AssignBitmap; // Overide;
    begin
      FBitmap := bmNoKnob;
    end;

    procedure   TKnobsNoKnob.Paint; // override;
    begin
      if not ( csDesigning in ComponentState)
      then SetWindowRgn( Handle, CreateRgn, True);

//      with Canvas
//      do begin
//        FBitmap.Transparent := True;
//        Draw( 0, 0, FBitmap);
//      end;
    end;

//  public

    constructor TKnobsNoKnob.Create( anOwner: TComponent); // override;
    begin
      inherited;

      FDownClicker.Top  := -1;        // todo : this sucks a bit ...
      FUpClicker  .Top  := -1;
      FUpClicker  .Left := FUpClicker.Left - 1;
    end;



{ ========
  TKnobsValuedButton = class( TKnobsValuedControl)
  private
    FButton     : TSpeedButton;
    FColorFocus : TColor;
    FUserColor  : TColor;
    FUserFlat   : Boolean;
  published
    property    Caption          : string        read GetCaption          write SetCaption                               ;
    property    Glyph            : TBitmap       read GetGlyph            write SetGlyph                                 ;
    property    Hint             : string        read GetHint             write SetHint                                  ;
    property    GroupIndex       : Integer       read GetGroupIndex       write SetGroupIndex       default             1;
    property    AllowAllUp       : Boolean       read GetAllowAllUp       write SetAllowAllUp       default         false;
    property    Height                                                                              default            18;
    property    Width                                                                               default            18;
    property    AlignWithMargins : Boolean       read GetAlignWithMargins write SetAlignWithMargins default         False;
    property    Color                                                                               default      clSilver;
    property    ColorFocus       : TColor        read FColorFocus         write FColorFocus         default     clSkyBlue;
    property    Flat             : Boolean       read GetFlat             write SetFlat             default         False;
    property    Font             : TFont         read GetFont             write SetFont                                  ;
    property    Layout           : TButtonLayout read GetLayout           write SetLayout           default   blGlyphLeft;
    property    Margin           : Integer       read GetMargin           write SetMargin           default            -1;
    property    Margins          : TMargins      read GetMargins          write SetMargins                               ;
    property    NumGlyphs        : TNumGlyphs    read GetNumGlyphs        write SetNumGlyphs        default             1;
    property    ParentFont       : Boolean       read GetParentFont       write SetParentFont       default          True;
    property    ParentShowHint   : Boolean       read GetParentShowHint   write SetParentShowHint   default          True;
    property    ShowHint         : Boolean       read GetShowHint         write SetShowHint         default         False;
    property    Transparent      : Boolean       read GetTransparent      write SetTransparent      default          True;
    property    Visible          : Boolean       read GetVisible          write SetVisible          default          True;
  private
}

    procedure   TKnobsValuedButton.SetKnobPosition( aValue: Integer); // override;
    begin
      if Assigned( FButton)
      then FButton.Down := SignalToLogic( aValue);
      inherited;
    end;

    function    TKnobsValuedButton.GetCaption: string;
    begin
      if Assigned( FButton)
      then Result := FButton.Caption
      else Result := '';
    end;

    procedure   TKnobsValuedButton.SetCaption( const aValue: string);
    begin
      if Assigned( FButton)
      then FButton.Caption := aValue;
    end;

    function    TKnobsValuedButton.GetGlyph: TBitmap;
    begin
      if Assigned( FButton)
      then Result := FButton.Glyph
      else Result := nil;
    end;

    procedure   TKnobsValuedButton.SetGlyph( const aValue: TBitmap);
    begin
      if Assigned( FButton)
      then FButton.Glyph.Assign( aValue);
    end;

    function    TKnobsValuedButton.GetHint: string;
    begin
      if Assigned( FButton)
      then Result := FButton.Hint
      else Result := '';
    end;

    procedure   TKnobsValuedButton.SetHint( const aValue: string);
    begin
      if Assigned( FButton)
      then FButton.Hint := aValue;
    end;

    function    TKnobsValuedButton.GetGroupIndex: Integer;
    begin
      if Assigned( FButton)
      then Result := FButton.GroupIndex
      else Result := 0;
    end;

    procedure   TKnobsValuedButton.SetGroupIndex( aValue: Integer);
    begin
      if Assigned( FButton)
      then FButton.GroupIndex := aValue;
    end;

    function    TKnobsValuedButton.GetAllowAllUp: Boolean;
    begin
      if Assigned( FButton)
      then Result := FButton.AllowAllUp
      else Result := False;
    end;

    procedure   TKnobsValuedButton.SetAllowAllUp( aValue: Boolean);
    begin
      if Assigned( FButton)
      then FButton.AllowAllUp := aValue;
    end;

//  private

    function    TKnobsValuedButton.GetAlignWithMargins: Boolean;
    begin
      if Assigned( FButton)
      then Result := FButton.AlignWithMargins
      else Result := False;
    end;

    procedure   TKnobsValuedButton.SetAlignWithMargins( aValue: Boolean);
    begin
      if Assigned( FButton)
      then FButton.AlignWithMargins := aValue;
    end;

    function    TKnobsValuedButton.GetFlat: Boolean;
    begin
      if Assigned( FButton)
      then Result := FButton.Flat
      else Result := False;
    end;

    procedure   TKnobsValuedButton.SetFlat( aValue: Boolean);
    begin
      if Assigned( FButton)
      then FButton.Flat := aValue;
    end;

    function    TKnobsValuedButton.GetFont: TFont;
    begin
      if Assigned( FButton)
      then Result := FButton.Font
      else Result := nil;
    end;

    procedure   TKnobsValuedButton.SetFont( const aValue: TFont);
    begin
      if Assigned( FButton)
      then FButton.Font.Assign( aValue);
    end;

    function    TKnobsValuedButton.GetLayout: TButtonLayout;
    begin
      if Assigned( FButton)
      then Result := FButton.Layout
      else Result := blGlyphLeft;
    end;

    procedure   TKnobsValuedButton.SetLayout( aValue: TButtonLayout);
    begin
      if Assigned( FButton)
      then FButton.Layout := aValue;
    end;

    function    TKnobsValuedButton.GetMargin: Integer;
    begin
      if Assigned( FButton)
      then Result := FButton.Margin
      else Result := -1;
    end;

    procedure   TKnobsValuedButton.SetMargin( aValue: Integer);
    begin
      if Assigned( FButton)
      then FButton.Margin := aValue;
    end;

    function    TKnobsValuedButton.GetMargins: TMargins;
    begin
      if Assigned( FButton)
      then Result := FButton.Margins
      else Result := nil;
    end;

    procedure   TKnobsValuedButton.SetMargins( aValue: TMargins);
    begin
      if Assigned( FButton)
      then FButton.Margins := aValue;
    end;

    function    TKnobsValuedButton.GetNumGlyphs: TNumGlyphs;
    begin
      if Assigned( FButton)
      then Result := FButton.NumGlyphs
      else Result := 1;
    end;

    procedure   TKnobsValuedButton.SetNumGlyphs( aValue: TNumGlyphs);
    begin
      if Assigned( FButton)
      then FButton.NumGlyphs := aValue;
    end;

    function    TKnobsValuedButton.GetParentFont: Boolean;
    begin
      if Assigned( FButton)
      then Result := FButton.ParentFont
      else Result := False;
    end;

    procedure   TKnobsValuedButton.SetParentFont( aValue: Boolean);
    begin
      if Assigned( FButton)
      then FButton.ParentFont := aValue;
    end;

    function    TKnobsValuedButton.GetParentShowHint: Boolean;
    begin
      if Assigned( FButton)
      then Result := FButton.ParentShowHint
      else Result := False;
    end;

    procedure   TKnobsValuedButton.SetParentShowHint( aValue: Boolean);
    begin
      if Assigned( FButton)
      then FButton.ParentShowHint := aValue;
    end;

//  private

    function    TKnobsValuedButton.GetShowHint: Boolean;
    begin
      if Assigned( FButton)
      then Result := FButton.ShowHint
      else Result := False;
    end;

    procedure   TKnobsValuedButton.SetShowHint( aValue: Boolean);
    begin
      if Assigned( FButton)
      then FButton.ShowHint := aValue;
    end;

    function    TKnobsValuedButton.Gettransparent: Boolean;
    begin
      if Assigned( FButton)
      then Result := FButton.Transparent
      else Result := False;
    end;

    procedure   TKnobsValuedButton.SetTransparent( aValue: Boolean);
    begin
      if Assigned( FButton)
      then FButton.Transparent := aValue;
    end;

    function    TKnobsValuedButton.GetVisible: Boolean;
    begin
      if Assigned( FButton)
      then Result := FButton.Visible
      else Result := True;
    end;

    procedure   TKnobsValuedButton.SetVisible( aValue: Boolean);
    begin
      if Assigned( FButton)
      then FButton.Visible := aValue;
    end;

//  private

    procedure   TKnobsValuedButton.OnButtonClick( aSender: TObject);
    begin
      if Assigned( FButton)
      then KnobPosition := Round( LogicToSignal( FButton.Down));
      FinalizeKnobPosition;
    end;

//  protected

    procedure   TKnobsValuedButton.WMSetFocus( var aMessage: TWMSetFocus); // message WM_SETFOCUS;
    begin
      inherited;
      FUserColor := Color;
      FUserFlat  := Flat;
      Color      := ColorFocus;
      Flat       := not Flat;
    end;

    procedure   TKnobsValuedButton.WMKillFocus( var aMessage: TWMKillFocus); // message WM_KILLFOCUS;
    begin
      inherited;
      Color := FUserColor;
      Flat  := FUserFlat;
    end;

    procedure   TKnobsValuedButton.WMGetDlgCode( var aMessage: TWMGetDlgCode); // message WM_GETDLGCODE;
    begin
      aMessage.Result := DLGC_WANTARROWS;
    end;

    procedure   TKnobsValuedButton.KeyDown( var Key: Word; Shift: TShiftState); // override;
    begin
      if not Assigned( FButton)
      then Exit;

      case Key of
        VK_UP  , VK_RIGHT : begin FButton.Down := False; Key := 0; end;
        VK_DOWN, VK_LEFT  : begin FButton.Down := True ; Key := 0; end;
        else                inherited;
      end;
    end;

    procedure   TKnobsValuedButton.SetBiDiMode( aValue: TBiDiMode); // override;
    begin
      if Assigned( FButton)
      then FButton.BiDiMode := aValue;
    end;

    procedure   TKnobsValuedButton.SetParentBiDiMode( aValue: Boolean); // override;
    begin
      if Assigned( FButton)
      then FButton.ParentBiDiMode := aValue;
    end;

    function    TKnobsValuedButton.GetEnabled: Boolean; // override;
    begin
      if Assigned( FButton)
      then Result := FButton.Enabled
      else Result := False;
    end;

    procedure   TKnobsValuedButton.SetEnabled( aValue: Boolean); // override;
    begin
      if Assigned( FButton)
      then FButton.Enabled := aValue;
    end;

//  protected

    procedure   TKnobsValuedButton.Resize; // override;
    begin
      inherited;
      if Assigned( FButton)
      then FButton.SetBounds( 0, 0, Width, Height);
    end;

    procedure   TKnobsValuedButton.Loaded; // override;
    begin
      inherited;
      Width := Width - 1; // A resize is needed to make the button visible ... weird
      Width := Width + 1;
    end;

    procedure   TKnobsValuedButton.SetDefaults; // virtual;
    begin
      StepCount       := 1;
      KnobPosition    := 0;
      GroupIndex      := 1;
      AllowAllUp      := false;
      Height          := 18;
      Width           := 18;
      Color           := clSilver;
      Flat            := False;
      Layout          := blGlyphLeft;
      Margin          := -1;
      NumGlyphs       := 1;
      ParentFont      := True;
      ParentShowHint  := True;
      ShowHint        := False;
      Transparent     := True;
      Visible         := True;
    end;

//  public

    constructor TKnobsValuedButton.Create( anOwner: TComponent); // override;
    begin
      inherited;
      ControlStyle := ControlStyle - [ csSetCaption] + [ csOpaque];

      FButton := TSpeedButton.Create( Self);
      FButton.Parent := Self;
      FButton.SetBounds( 0, 0, Width, Height);
      FButton.OnClick := OnButtonClick;
      SetDefaults;
      DoubleBuffered := True;
    end;

    destructor  TKnobsValuedButton.Destroy; // override;
    begin
      FButton.OnClick := nil;
      FreeAndNil( FButton);
      inherited;
    end;



{ ========
  TSValuedButtons = class( TValuedControl)
  private
    FButtons          : array of TSpeedButton;
    FCaptions         : TStrings;
    FImageIndices     : TStrings;
    FGlyphs           : TImageList;
    FHints            : TStrings;
    FGroupIndex       : Integer;
    FAlignWithMargins : Boolean;
    FAllowAllUp       : Boolean;
    FFlat             : Boolean;
    FFont             : TFont;
    FLayout           : TButtonLayout;
    FDisplayMode      : TKnobsButtonsDisplayMode;
    FMargin           : Integer;
    FMargins          : TMargins;
    FNumGlyphs        : TNumGlyphs;
    FParentFont       : Boolean;
    FParentShowHint   : Boolean;
    FShowHint         : Boolean;
    FTransparent      : Boolean;
    FVisible          : Boolean;
    FColorFocus       : TColor;
    FUserColor        : TColor;
    FUserFlat         : Boolean;
  published
    property    StepCount                                                                                    default             2;
    property    Captions         : TStrings                 read GetCaptions       write SetCaptions                              ;
    property    ImageIndices     : TStrings                 read GetImageIndices   write SetImageIndices                          ;
    property    Glyphs           : TImageList               read GetGlyphs         write SetGlyphs                                ;
    property    Hints            : TStrings                 read GetHints          write SetHints                                 ;
    property    GroupIndex       : Integer                  read FGroupIndex       write SetGroupIndex       default             1;
    property    AllowAllUp       : Boolean                  read FAllowAllUp       write SetAllowAllUp       default         false;
    property    Height                                                                                       default            25;
    property    Width                                                                                        default            51;
    property    AlignWithMargins : Boolean                  read FAlignWithMargins write SetAlignWithMargins default         False;
    property    Color                                                                                        default      clSilver;
    property    ColorFocus       : TColor                   read FColorFocus       write FColorFocus         default     clSkyBlue;
    property    Flat             : Boolean                  read FFlat             write SetFlat             default         False;
    property    Font             : TFont                    read FFont             write SetFont                                  ;
    property    Layout           : TButtonLayout            read FLayout           write SetLayout           default   blGlyphLeft;
    property    DisplayMode      : TKnobsButtonsDisplayMode read FDisplayMode      write SetDisplayMode      default bdmHorizontal;
    property    Margin           : Integer                  read FMargin           write SetMargin           default            -1;
    property    Margins          : TMargins                 read FMargins          write SetMargins                               ;
    property    NumGlyphs        : TNumGlyphs               read FNumGlyphs        write SetNumGlyphs        default             1;
    property    ParentFont       : Boolean                  read FParentFont       write SetParentFont       default          True;
    property    ParentShowHint   : Boolean                  read FParentShowHint   write SetParentShowHint   default          True;
    property    ShowHint         : Boolean                  read FShowHint         write SetShowHint         default         False;
    property    Transparent      : Boolean                  read FTransparent      write SetTransparent      default          True;
    property    Visible          : Boolean                  read FVisible          write SetVisible          default          True;
  private
}

    procedure   TKnobsValuedButtons.FixDimensions;
    var
      i : Integer;
      w : Integer;
      h : Integer;
    begin
      case DisplayMode of

        bdmHorizontal :

          if StepCount > 0
          then begin
            w := Width div StepCount;
            h := Height;
            if AlignWithMargins
            then begin
              w := w - Margins.Left - Margins.Right;
              h := h - Margins.Top  - Margins.Bottom;
            end;
            for i := 0 to StepCount - 1
            do begin
              FButtons[ i].Width  := w;
              FButtons[ i].Height := h;
            end;
          end;

        bdmVertical :

          if StepCount > 0
          then begin
            w := Width;
            h := Height div StepCount;
            if AlignWithMargins
            then begin
              w := w - Margins.Left - Margins.Right;
              h := h - Margins.Top  - Margins.Bottom;
            end;
            for i := 0 to StepCount - 1
            do begin
              FButtons[ i].Width  := w;
              FButtons[ i].Height := h;
            end;
          end;
      end;
    end;

    procedure   TKnobsValuedButtons.FixGlyph( anIndex: Integer; aBitmap: TBitmap);
    begin
      FButtons[ anIndex].Glyph.Assign( aBitmap);
    end;

    function    TKnobsValuedButtons.FindGlyphIndex( anIndex: Integer): Integer;
    begin
      if ( anIndex >= 0) and ( anIndex < FImageIndices.Count)
      then Result := StrToIntDef( FImageIndices[ anIndex], -1)
      else Result := -1;
    end;

//  private

    procedure   TKnobsValuedButtons.FixAlignWithMargins;
    var
      i : Integer;
    begin
      for i := 0 to StepCount - 1
      do FButtons[ i].AlignWithMargins := AlignWithMargins;
      FixDimensions;
    end;

    procedure   TKnobsValuedButtons.FixBiDiMode;
    var
      i : Integer;
    begin
      for i := 0 to StepCount - 1
      do FButtons[ i].BiDiMode := BiDiMode;
    end;

    procedure   TKnobsValuedButtons.FixCaptions;
    var
      i : Integer;
    begin
      if Assigned( FCaptions)
      then begin
        for i := 0 to StepCount - 1
        do begin
          if i < FCaptions.Count
          then FButtons[ i].Caption := FCaptions[ i]
          else FButtons[ i].Caption := '';
        end;
      end;
    end;

    procedure   TKnobsValuedButtons.FixEnabled;
    var
      i : Integer;
    begin
      for i := 0 to StepCount - 1
      do FButtons[ i].Enabled := Enabled;
    end;

    procedure   TKnobsValuedButtons.FixFlat;
    var
      i : Integer;
    begin
      for i := 0 to StepCount - 1
      do FButtons[ i].Flat := Flat;
    end;

    procedure   TKnobsValuedButtons.FixFont;
    var
      i : Integer;
    begin
      for i := 0 to StepCount - 1
      do FButtons[ i].Font.Assign( Font);
    end;

    procedure   TKnobsValuedButtons.FixGlyphs;
    var
      i   : Integer;
      aBm : TBitmap;
      idx : Integer;
    begin
      if Assigned( FGlyphs)
      then begin
        aBm := TBitmap.Create;
        try
          for i := 0 to StepCount - 1
          do begin
            idx := FindGlyphIndex( i);
            if ( idx >= 0) and ( idx < FGlyphs.Count)
            then begin
              FGlyphs.GetBitmap( idx, aBm);
              FixGlyph( i, aBm);
            end
            else FixGlyph( i, nil);
          end;
        finally
          aBm.Free;
        end;
      end;
    end;

    procedure   TKnobsValuedButtons.FixHints;
    var
      i : Integer;
    begin
      if Assigned( FHints)
      then begin
        for i := 0 to StepCount - 1
        do begin
          if i < FHints.Count
          then FButtons[ i].Hint := FHints[ i]
          else FButtons[ i].Hint := '';
        end;
      end;
    end;

    procedure   TKnobsValuedButtons.FixLayout;
    var
      i : Integer;
    begin
      for i := 0 to StepCount - 1
      do FButtons[ i].Layout := Layout;
    end;

    procedure   TKnobsValuedButtons.FixMargin;
    var
      i : Integer;
    begin
      for i := 0 to StepCount - 1
      do FButtons[ i].Margin := Margin;
    end;

    procedure   TKnobsValuedButtons.FixMargins;
    var
      i : Integer;
    begin
      for i := 0 to StepCount - 1
      do FButtons[ i].Margins.Assign( FMargins);
      FixDimensions;
    end;

    procedure   TKnobsValuedButtons.FixNumGlyphs;
    var
      i : Integer;
    begin
      for i := 0 to StepCount - 1
      do FButtons[ i].NumGlyphs := NumGlyphs;
    end;

    procedure   TKnobsValuedButtons.FixParentFont;
    var
      i : Integer;
    begin
      for i := 0 to StepCount - 1
      do FButtons[ i].ParentFont := ParentFont;
    end;

    procedure   TKnobsValuedButtons.FixParentShowHint;
    var
      i : Integer;
    begin
      for i := 0 to StepCount - 1
      do FButtons[ i].ParentShowHint := ParentShowHint;
    end;

    procedure   TKnobsValuedButtons.FixParentBiDiMode;
    var
      i : Integer;
    begin
      for i := 0 to StepCount - 1
      do FButtons[ i].ParentBiDiMode := ParentBiDiMode;
    end;

    procedure   TKnobsValuedButtons.FixShowHint;
    var
      i : Integer;
    begin
      for i := 0 to StepCount - 1
      do FButtons[ i].ShowHint := ShowHint;
    end;

    procedure   TKnobsValuedButtons.FixTransparent;
    var
      i : Integer;
    begin
      for i := 0 to StepCount - 1
      do FButtons[ i].Transparent := Transparent;
    end;

    procedure   TKnobsValuedButtons.FixVisible;
    var
      i : Integer;
    begin
      for i := 0 to StepCount - 1
      do FButtons[ i].Visible := Visible;
    end;

//  private

    procedure   TKnobsValuedButtons.FixMode( aNewStepCount: Integer);
    var
      aComponent: TComponent;
    begin
      while aNewStepCount < Length( FButtons)
      do begin
        aComponent := FButtons[ Length( FButtons) - 1];
        SetLength( FButtons, Length( FButtons) - 1);
        Owner.RemoveComponent( aComponent);
        aComponent.Free;
      end;
      FStepCount := aNewStepCount;
      while aNewStepCount > Length( FButtons)
      do CreateButton;
      FixDimensions;
    end;

    procedure   TKnobsValuedButtons.SetKnobPosition( aValue: Integer); // override;
    var
      anIndex : Integer;
    begin
      if aValue <> FKnobPosition
      then begin
        inherited;
        anIndex := Round( aValue + 0.1);
        if ( anIndex >= 0) and ( anIndex < StepCount)
        then FButtons[ anIndex].Down := True;
        FinalizeKnobPosition;
      end;
    end;

    procedure   TKnobsValuedButtons.SetStepCount( aValue: Integer);
    begin
      if aValue <= 0
      then aValue := 1;
      if aValue <> StepCount
      then begin
        FixMode( aValue);
        FixGlyphs;
        FixCaptions;
        FixHints;
        FixDimensions;
        FinalizeKnobPosition;
      end;
    end;

    function    TKnobsValuedButtons.GetCaptions: TStrings;
    begin
      Result := FCaptions;
    end;

    procedure   TKnobsValuedButtons.SetCaptions( const aValue: TStrings);
    begin
      FCaptions.Assign( aValue);
      FixCaptions;
    end;

    function    TKnobsValuedButtons.GetImageIndices: TStrings;
    begin
      Result := FImageIndices;
    end;

    procedure   TKnobsValuedButtons.SetImageIndices( const aValue: TStrings);
    begin
      FImageIndices.Assign( aValue);
      FixGlyphs;
    end;

    function    TKnobsValuedButtons.GetGlyphs: TImageList;
    begin
      Result := FGlyphs;
    end;

    procedure   TKnobsValuedButtons.SetGlyphs( const aValue: TImageList);
    begin
      FGlyphs := aValue;
      FixGlyphs;
    end;

    function    TKnobsValuedButtons.GetHints: TStrings;
    begin
      Result := FHints;
    end;

    procedure   TKnobsValuedButtons.SetHints( const aValue: TStrings);
    begin
      FHints.Assign( aValue);
      FixHints;
    end;

    procedure   TKnobsValuedButtons.SetGroupIndex( aValue: Integer);
    var
      i : Integer;
    begin
      if aValue <> FGroupIndex
      then begin
        FGroupIndex := aValue;
        for i := 0 to StepCount - 1
        do FButtons[ i].GroupIndex := FGroupIndex;
      end;
    end;

    procedure   TKnobsValuedButtons.SetAllowAllUp( aValue: Boolean);
    var
      i : Integer;
    begin
      if aValue <> FAllowAllUp
      then begin
        FAllowAllUp := aValue;
        for i := 0 to StepCount - 1
        do FButtons[ i].AllowAllUp := FAllowAllUp;
      end;
    end;

    procedure   TKnobsValuedButtons.SetAlignWithMargins( aValue: Boolean);
    begin
      if aValue <> FAlignWithMargins
      then begin
        FAlignWithMargins := aValue;
        FixAlignWithMargins;
      end;
    end;

    procedure   TKnobsValuedButtons.SetFlat( aValue: Boolean);
    begin
      if avalue <> FFlat
      then begin
        FFlat := aValue;
        FixFlat;
      end;
    end;

    procedure   TKnobsValuedButtons.SetFont( const aValue: TFont);
    begin
      FFont.Assign( aValue);
      FixFont;
    end;

    procedure   TKnobsValuedButtons.SetLayout( aValue: TButtonLayout);
    begin
      if aValue <> FLayout
      then begin
        FLayout := aValue;
        FixLayout;
      end;
    end;

    procedure   TKnobsValuedButtons.SetDisplayMode( aValue: TKnobsButtonsDisplayMode);
    begin
      if aValue <> FDisplayMode
      then begin
        FDisplayMode := aValue;
        FixMode( StepCount);
      end;
    end;

    procedure   TKnobsValuedButtons.SetMargin( aValue: Integer);
    begin
      if avalue <> FMargin
      then begin
        FMargin := aValue;
        FixMargin;
      end;
    end;

    procedure   TKnobsValuedButtons.SetMargins( aValue: TMargins);
    begin
      FMargins.Assign( aValue);
      FixMargins;
    end;

    procedure   TKnobsValuedButtons.SetNumGlyphs( aValue: TNumGlyphs);
    begin
      if avalue <> FNumGlyphs
      then begin
        FNumGlyphs := aValue;
        FixNumGlyphs;
      end;
    end;

    procedure   TKnobsValuedButtons.SetParentFont( aValue: Boolean);
    begin
      if aValue <> FParentFont
      then begin
        FParentFont := aValue;
        FixParentFont;
      end;
    end;

    procedure   TKnobsValuedButtons.SetParentShowHint( aValue: Boolean);
    begin
      if aValue <> FParentShowHint
      then begin
        FParentShowHint := aValue;
        FixParentShowHint;
        if ParentShowHint
        then ShowHint := False;
      end;
    end;

    procedure   TKnobsValuedButtons.setShowHint( aValue: Boolean);
    begin
      if aValue <> FShowHint
      then begin
        FShowHint := aValue;
        FixShowHint;
        if ShowHint
        then ParentShowHint := False;
      end;
    end;

    procedure   TKnobsValuedButtons.SetTransparent( aValue: Boolean);
    begin
      if aValue <> FTransparent
      then begin
        FTransparent := aValue;
        FixTransparent;
      end;
    end;

    procedure   TKnobsValuedButtons.SetVisible( aValue: Boolean);
    begin
      if aValue <> FVisible
      then begin
        FVisible := aValue;
        FixVisible;
      end;
    end;

//  private

    procedure   TKnobsValuedButtons.OnButtonClick( aSender: TObject);
    var
      aTag: Integer;
    begin
      aTag := TSpeedButton( aSender).Tag;
      KnobPosition := aTag;
      FinalizeKnobPosition;
    end;

    procedure   TKnobsValuedButtons.CreateButton;
    var
      aButton : TSpeedButton;
    begin
      aButton := TSpeedButton.Create( Self);
      with aButton do
      begin
        Parent     := Self;
        AllowAllUp := Self.AllowAllUp;
        GroupIndex := Self.GroupIndex;
        Name       := '';
        Tag        := Length( FButtons);
        OnClick    := OnButtonClick;
        Align      := alLeft;
        SetLength( FButtons, Length( FButtons) + 1);
        FButtons[ Length( FButtons) - 1] := aButton;
      end;
      StepCount := Length( FButtons);
    end;

//  protected

    procedure   TKnobsValuedButtons.WMSetFocus( var aMessage: TWMSetFocus); // message WM_SETFOCUS;
    begin
      inherited;
      FUserColor := Color;
      FUserFlat  := Flat;
      Color      := ColorFocus;
      Flat       := not Flat;
    end;

    procedure   TKnobsValuedButtons.WMKillFocus( var aMessage: TWMKillFocus); // message WM_KILLFOCUS;
    begin
      inherited;
      Color := FUserColor;
      Flat  := FUserFlat;
    end;

    procedure   TKnobsValuedButtons.WMGetDlgCode( var aMessage: TWMGetDlgCode); // message WM_GETDLGCODE;
    begin
      aMessage.Result := DLGC_WANTARROWS;
    end;

    procedure   TKnobsValuedButtons.KeyDown( var Key: Word; Shift: TShiftState); // override;
    begin
      case Key of
        VK_UP  , VK_RIGHT : begin if KnobPosition < StepCount - 1 then KnobPosition := KnobPosition + 1; Key := 0; end;
        VK_DOWN, VK_LEFT  : begin if KnobPosition > 0             then KnobPosition := KnobPosition - 1; Key := 0; end;
        else                inherited;
      end;
    end;

    procedure   TKnobsValuedButtons.SetBiDiMode( aValue: TBiDiMode); // Override;
    begin
      if aValue <> BiDiMode
      then begin
        inherited;
        FixBiDiMode;
      end;
    end;

    procedure   TKnobsValuedButtons.SetEnabled( aValue: Boolean); // Override;
    begin
      if aValue <> Enabled
      then begin
        inherited;
        FixEnabled;
      end;
    end;

    procedure   TKnobsValuedButtons.SetParentBiDiMode( aValue: Boolean); // Override;
    begin
      if aValue <> ParentBiDiMode
      then begin
        inherited;
        FixParentBiDiMode;
      end;
    end;

//  protected

    procedure   TKnobsValuedButtons.Resize; // override;
    begin
      inherited;
      FixDimensions;
    end;

    procedure   TKnobsValuedButtons.Loaded; // override;
    begin
      // FixCaptions;
      FixButtons;
    end;

    procedure   TKnobsValuedButtons.FixButtons; // virtual;
    begin
      FixAlignWithMargins;
      FixBiDiMode;
      FixCaptions;
      FixEnabled;
      FixFlat;
      FixFont;
      FixGlyphs;
      FixHints;
      FixLayout;
      FixMargin;
      FixMargins;
      FixNumGlyphs;
      FixParentFont;
      FixParentShowHint;
      FixParentBiDiMode;
      FixShowHint;
      FixTransparent;
      FixVisible;
    end;

    procedure   TKnobsValuedButtons.SetDefaults; // virtual;
    begin
      StepCount       := 2;
      GroupIndex      := 1;
      AllowAllUp      := False;
      Height          := 25;
      Width           := 51;
      Color           := clSilver;
      ColorFocus      := clSkyBlue;
      Flat            := False;
      Layout          := blGlyphLeft;
      Margin          := -1;
      NumGlyphs       := 1;
      ParentFont      := True;
      ParentShowHint  := True;
      ShowHint        := False;
      Transparent     := True;
      Visible         := True;
    end;

//  public

    constructor TKnobsValuedButtons.Create( anOwner: TComponent); // override;
    begin
      inherited;
      ControlStyle  := ControlStyle + [ csOpaque];
      FCaptions     := TStringList.Create;
      FHints        := TStringList.Create;
      FImageIndices := TStringList.Create;
      FFont         := TFont      .Create;
      FMargins      := TMargins   .Create( Self);
      SetDefaults;
    end;

    destructor  TKnobsValuedButtons.Destroy; // override;
    begin
      FFont        .Free;
      FImageIndices.Free;
      FHints       .Free;
      FCaptions    .Free;
      inherited;
    end;



{ ========
  TDisplayEditor = class( TMemo)
  // An multi line editor for TEditableDisplay
  private
    FDisplay : TDisplay;
  private
}

    procedure   TKnobsDisplayEditor.WMKillFocus( var aMessage: TWMKillFocus ); // message WM_KILLFOCUS;
    begin
      inherited;

      Visible := False;
    end;

    procedure   TKnobsDisplayEditor.Do_Exit( aSender: TObject);
    begin
      Visible := False;
      if Assigned( FDisplay)
      then FDisplay.SetFocus;
    end;

    procedure   TKnobsDisplayEditor.Do_KeyDown( aSender: TObject; var aKey: Word; aShift: TShiftState);
    begin
      if aShift = []
      then
        case aKey Of

          VK_ESCAPE :
            begin
              Visible := False;
              aKey    := 0;
            end;

        end
      else if aShift = [ ssCtrl]
      then
        case aKey Of

          VK_RETURN:
            begin
              FDisplay.SetText( Text);
              Visible := False;
              aKey    := 0;
              if Assigned( FDisplay)
              then FDisplay.LockKnob;
            end;
        end;
    end;

//  protected

    procedure   TKnobsDisplayEditor.SetDisplay( aDisplay: TKnobsDisplay);
    var
      aLeft   : Integer;
      aTop    : Integer;
      aWidth  : Integer;
      aHeight : Integer;
    begin
      FDisplay := aDisplay;
      Parent   := aDisplay.Parent;

      // todo : make display editing possible again

      // if Assigned( aDisplay.FController)
      // then Text := SignalToStr( aDisplay.FController.AsValue)
      // else Text := aDisplay.Caption;

      // Changed the above into the following line therefore :

      Text := aDisplay.Caption;

      if aDisplay.HasParentEditor
      then SetBounds( 0, 0, Parent.Width, Parent.Height)
      else begin
        aLeft   := 0;
        aTop    := aDisplay.Top -  2;
        aWidth  := Parent.Width;
        aHeight := aDisplay.Height +  4;
        if aTop < 0
        then aTop := 0;
        if aTop + aHeight > Parent.Height
        then aTop := Parent.Height - aHeight;
        SetBounds( aLeft, aTop, aWidth, aHeight);
      end;

      if aDisplay.HasScrollbar
      then ScrollBars := ssVertical
      else ScrollBars := ssNone;

      Font       := aDisplay.Font;
      Color      := clWhite;
      Font.Color := clBlack;
      Visible    := True;
      SetFocus;
    end;

//  public

    constructor TKnobsDisplayEditor.Create( anOwner: TComponent); // override;
    begin
      if Assigned( GDisplayEditor)
      then Exit;

      inherited;

      ControlStyle   := ControlStyle + [ csOpaque];

      Visible        := False;
      Left           := 1;
      Top            := 1;
      OnExit         := Do_Exit;
      OnKeyDown      := Do_KeyDown;
      GDisplayEditor := Self;
    end;

    destructor  TKnobsDisplayEditor.Destroy; // override;
    begin
      inherited;
      if Assigned( Owner)
      then Owner.RemoveComponent( Self);
    end;


{ ========
  TDisplay = class( TStaticText)
  private
    FEditor          : TKnobsDisplayEditor;
    FController      : TKnobsValuedControl;
    FHasEditor       : Boolean;
    FHasParentEditor : Boolean;
    FHasScrollbar    : Boolean;
    FMustSave        : Boolean;
    FOnChanged       : TKnobsOnDisplayChanged;
    FSavedColor      : TColor;
  public
    property    Module: TKnobsCustomModule  read GetModule;
    property    Editor: TKnobsDisplayEditor read FEditor   write FEditor;
  published
    property    Ctl3D                                                                                 default False;
    property    ParentColor                                                                           default False;
    property    Color                                                                                 default KnobsColorDisplay;
    property    Width                                                                                 default 33;
    property    Height                                                                                default 15;
    property    AutoSize                                                                              default False;
    property    BorderStyle                                                                           default sbsSingle;
    property    HasEditor       : Boolean                read FHasEditor       write FHasEditor       default False;
    property    HasParentEditor : Boolean                read FHasParentEditor write FHasParentEditor default False;
    property    HasScrollbar    : Boolean                read FHasScrollbar    write FHasScrollbar    default False;
    property    MustSave        : Boolean                read FMustSave        write FMustSave        default False;
    property    OnChanged       : TKnobsOnDisplayChanged read FOnChanged       write FOnChanged;
  private
}

    function    TKnobsDisplay.GetModule: TKnobsCustomModule;
    begin
      if Owner is TKnobsCustomModule
      then result := TKnobsCustomModule( Owner)
      else result := nil;
    end;

    procedure   TKnobsDisplay.Do_KeyDown( aSender: TObject; var aKey: Word; aShift: TShiftState);
    begin
      if aShift = []
      then begin
        case aKey of
          VK_RETURN:
            begin
              NeedsDisplayEditor;
              GDisplayEditor.SetDisplay( Self);
              aKey := 0;
            end;
        end;
      end;
    end;

//  private

    function    TKnobsDisplay.FindWirePanel: TKnobsWirePanel;
    var
      aParent : TWinControl;
    begin
      Result  := nil;
      aParent := Parent;

      while Assigned( aParent) and not ( aParent is TKnobsWirePanel)
      do aParent :=  aParent.Parent;

      if aParent is TKnobsWirePanel
      then Result := TKnobsWirePanel( aParent);
    end;

//  protected

    procedure   TKnobsDisplay.DisplayChanged(  const aNewValue: string; DoNotify: Boolean);
    var
      aModule : TKnobsCustomModule;
    begin
      if Assigned( FOnChanged)
      then FOnChanged( Self, Name, aNewValue)
      else begin
        aModule := Module;
        if Assigned( aModule) and DoNotify or MustNotify
        then Module.DisplayChanged( Self, MakePath( [ aModule.Name, Name]), aNewValue);
      end;
    end;

    procedure   TKnobsDisplay.SetText( const aValue: string);
    begin
      if Caption <> aValue
      then begin
        try
          DisplayChanged( aValue, DO_NOTIFY);
          Caption := aValue;
        except
          on EAbort
          do nop            // Absorb aborts
          else raise;       // Re-raise errors
        end;
      end;
    end;

    procedure   TKnobsDisplay.WMNCHitTest( var aMessage: TWMNCHitTest); // message WM_NCHITTEST;
    begin
      if ( csDesigning in ComponentState) or HasEditor
      then inherited
      else aMessage.Result := HTTRANSPARENT;
    end;

    procedure   TKnobsDisplay.WMSetFocus( var aMessage: TWMSetFocus); // message WM_SETFOCUS;
    begin
      inherited;
      FSavedColor := Color;
      Color       := clBlack;
    end;

    procedure   TKnobsDisplay.WMKillFocus( var aMessage: TWMKillFocus); // message WM_KILLFOCUS;
    begin
      inherited;
      Color := FSavedColor;
    end;

    procedure   TKnobsDisplay.MouseDown( Button: TMouseButton; Shift: TShiftState; X, Y: Integer); // override;

    // We only need the double click here to start a label editor, all other
    // events should be handled by the containing module.

    var
      aModule : TKnobsCustomModule;
      aWP     : TKnobsWirePanel;
    begin
      SetFocus;
      if csDesigning in ComponentState
      then inherited
      else begin
        if ( ssDouble in Shift) and ( Button = mbLeft) and HasEditor
        then begin
          // Handle the double click
          NeedsDisplayEditor;
          GDisplayEditor.SetDisplay( Self);

        end
        else if Button = mbRight
        then begin
          aWP := FindWirePanel;
          if Assigned( aWP)
          then aWP.HandleRightClick( Self);
        end
        else begin
          // Pass it on
          aModule := Module;

          if Assigned( aModule)
          then begin
            aModule.MouseCapture := True;
            aModule.MouseDown( Button, Shift, X + Left, Y + Top);
          end;
        end;
      end;
    end;

    procedure   TKnobsDisplay.LockKnob;
    begin
      if FController is TKnobsKnob
      then TKnobsKnob( FController).Locked := True;
    end;

//  public

    constructor TKnobsDisplay.Create( anOwner: TComponent); // override;
    begin
      inherited;

      ControlStyle    := ControlStyle - [ csSetCaption] + [ csOpaque];
      OnKeyDown       := Do_KeyDown;
      ParentColor     := False;
      Ctl3D           := False;
      Color           := KnobsColorDisplay;
      AutoSize        := False;
      BorderStyle     := sbsSingle;
      Font.Color      := clWhite;
      Caption         := '0.57';
      Width           := 33;
      Height          := 15;
      DoubleBuffered  := True;
      HasEditor       := False;
      HasParentEditor := False;
      HasScrollbar    := False;
      MustSave        := False;
      FSavedColor     := Color;
      TabStop         := True;
    end;

    destructor  TKnobsDisplay.Destroy; // override;
    begin
      if FController is TKnobsKnob
      then TKnobsKnob( FController).Display := nil;
      inherited;
    end;


{ ========
  TKnobsFileSelector = class( TStaticText)
  private
    FMustSave     : Boolean;
    FMustNotify   : Boolean;
    FOnChanged    : TKnobsOnFileChanged;
    FSavedColor   : TColor;
    FFileSelector : TOpenDialog;
    FFilename     : string;
  public
    property    Module: TKnobsCustomModule  read GetModule;
  published
    property    Ctl3D                                                                    default False;
    property    ParentColor                                                              default False;
    property    Color                                                                    default KnobsColorDisplay;
    property    Width                                                                    default 33;
    property    Height                                                                   default 15;
    property    AutoSize                                                                 default False;
    property    BorderStyle                                                              default sbsSingle;
    property    TabStop                                                                  default True;
    property    MustSave        : Boolean             read FMustSave   write FMustSave   default False;
    property    MustNotify      : Boolean             read FMustNotify write FMustNotify default False;
    property    OnChanged       : TKnobsOnFileChanged read FOnChanged  write FOnChanged;
  private
}

    procedure   TKnobsFileSelector.SetWaveName( const aValue: string);
    begin
      FWaveName := aValue;
      FixCaption;
    end;

    procedure   TKnobsFileSelector.FixCaption;
    begin
      // Caption := FWaveName;
      Caption := ' ' + ExtractFileName( FWaveName);
    end;

    function    TKnobsFileSelector.GetModule: TKnobsCustomModule;
    begin
      if Owner is TKnobsCustomModule
      then result := TKnobsCustomModule( Owner)
      else result := nil;
    end;

    procedure   TKnobsFileSelector.Do_KeyDown( aSender: TObject; var aKey: Word; aShift: TShiftState);
    begin
      if aShift = []
      then begin
        case aKey of
          VK_RETURN:
            begin
              SelectFile;
              aKey := 0;
            end;
        end;
      end;
    end;

    procedure   TKnobsFileSelector.SelectFile;
    begin
      with FFileSelector
      do begin
        FileName := WaveName;
        if Execute
        then SetText( FileName);
//        then WaveName := FileName;
      end;
    end;

//  private

    function    TKnobsFileSelector.FindWirePanel: TKnobsWirePanel;
    var
      aParent : TWinControl;
    begin
      Result  := nil;
      aParent := Parent;

      while Assigned( aParent) and not ( aParent is TKnobsWirePanel)
      do aParent :=  aParent.Parent;

      if aParent is TKnobsWirePanel
      then Result := TKnobsWirePanel( aParent);
    end;

//  protected

    procedure   TKnobsFileSelector.FileChanged( const aNewValue: string; DoNotify: Boolean);
    var
      aModule : TKnobsCustomModule;
    begin
      if Assigned( FOnChanged)
      then FOnChanged( Self, Name, aNewValue)
      else begin
        aModule := Module;
        if Assigned( aModule) and DoNotify or MustNotify
        then Module.FileChanged( Self, MakePath( [ aModule.Name, Name]), aNewValue);
      end;
    end;

    procedure   TKnobsFileSelector.SetText( const aValue: string);
    begin
      if FWaveName <> aValue
      then begin
        try
          FileChanged( aValue, DO_NOTIFY);
          WaveName := aValue;
        except
          on EAbort
          do nop            // Absorb aborts
          else raise;       // Re-raise errors
        end;
      end;
    end;

    procedure   TKnobsFileSelector.WMSetFocus( var aMessage: TWMSetFocus); // message WM_SETFOCUS;
    begin
      inherited;
      FSavedColor := Color;
      Color       := clBlack;
    end;

    procedure   TKnobsFileSelector.WMKillFocus( var aMessage: TWMKillFocus); // message WM_KILLFOCUS;
    begin
      inherited;
      Color := FSavedColor;
    end;

    procedure   TKnobsFileSelector.MouseDown( Button: TMouseButton; Shift: TShiftState; X, Y: Integer); // override;
    // We only need the double click here to start a file dialog
    // events should be handled by the containing module.
    var
      aModule : TKnobsCustomModule;
      aWP     : TKnobsWirePanel;
    begin
      SetFocus;
      if csDesigning in ComponentState
      then inherited
      else begin
        if ( ssDouble in Shift) and ( Button = mbLeft)
        then begin
          // Handle the double click
          SelectFile;
        end
        else if Button = mbRight
        then begin
          aWP := FindWirePanel;
          if Assigned( aWP)
          then aWP.HandleRightClick( Self);
        end
        else begin
          // Pass it on
          aModule := Module;
          if Assigned( aModule)
          then begin
            aModule.MouseCapture := True;
            aModule.MouseDown( Button, Shift, X + Left, Y + Top);
          end;
        end;
      end;
    end;

//  public

    constructor TKnobsFileSelector.Create( anOwner: TComponent); // override;
    begin
      inherited;
      FFileSelector   := TOpenDialog.Create( nil);
      ControlStyle    := ControlStyle - [ csSetCaption] + [ csOpaque];
      OnKeyDown       := Do_KeyDown;
      ParentColor     := False;
      Ctl3D           := False;
      Color           := KnobsColorDisplay;
      AutoSize        := False;
      BorderStyle     := sbsSingle;
      Font.Color      := clWhite;
      Caption         := ' double click to load a wave file';
      FWaveName       := Caption;
      Width           := 33;
      Height          := 15;
      DoubleBuffered  := True;
      MustSave        := False;
      MustNotify      := False;
      FSavedColor     := Color;
      TabStop         := True;
      with FFileSelector
      do begin
        DefaultExt := '.wav';
        Title      := 'select wave file';
        Filter     := 'wave files (*.wav)|*.wav|All files (*.*)|*.*';
      end;
    end;

    destructor  TKnobsFileSelector.Destroy; // override;
    begin
      FreeAndNil( FFileSelector);
      inherited;
    end;


{ ========
  TIndicator = class( TGraphicControl)
  private
    FActive     : Boolean;
    FActiveBm   : TBitmap;
    FInActiveBm : TBitmap;
  public
    property    BitmapActive   : TBitmap read FActiveBm   write SetBitmapActive;
    property    BitmapInactive : TBitmap read FInactiveBm write SetBitmapInactive;
  published
    property    Active         : Boolean read FActive     write SetActive;
  private
}

    procedure   TKnobsIndicator.SetActive( aValue: Boolean);
    begin
      if aValue <> FActive
      then begin
        FActive := aValue;
        Invalidate;
      end;
    end;

    procedure   TKnobsIndicator.SetBitmapActive( aValue: TBitmap);
    begin
      FActiveBm.Assign( aValue);
      if FActive
      then Invalidate;
    end;

    procedure   TKnobsIndicator.SetBitmapInactive( aValue: TBitmap);
    begin
      FInactiveBm.Assign( aValue);
      if not FActive
      then Invalidate;
    end;

//  protected

    procedure   TKnobsIndicator.AssignBitmaps; // virtual;
    begin
      FInactiveBm := bmIndInactive;
      FActiveBm   := bmIndActive  ;
      FInActiveBm.Transparent := False;
      FActiveBm  .Transparent := False;
    end;

    procedure   TKnobsIndicator.Paint; // override;
    begin
      inherited;

      with Canvas
      do begin
        if Active
        then Draw( 0, 0, FActiveBm  )
        else Draw( 0, 0, FInactiveBm);
      end;
    end;

//  public

    constructor TKnobsIndicator.Create( anOwner: TComponent); // override;
    begin
      inherited;

      ControlStyle := ControlStyle + [ csOpaque];
      AssignBitmaps;
      Width       := Max( FActiveBm.Width , FInactiveBm.Width );
      Height      := Max( FActiveBm.Height, FInactiveBm.Height);
      Enabled     := False;
    end;


{ ========
  TDisplayType = (
    dtSingleFromLow,
    dtSingleFromHigh,
    dtFilledFromLow,
    dtFilledFromHigh
  );

  TDisplayOrientation = (
    doHorizontal,
    doVertical
  );

  TIndicatorBar = class( TGraphicControl)
  private
    FIndicatorCount : Integer;
    FValue          : TSValue;
    FValueLowMid    : TSValue;
    FValueMidHigh   : TSValue;
    FDisplayType    : TDisplayType;
    FOrientation    : TDisplayOrientation;
    FSpacing        : Integer;
    FActiveLowBm    : TBitmap;
    FInActiveLowBm  : TBitmap;
    FActiveMidBm    : TBitmap;
    FInActiveMidBm  : TBitmap;
    FActiveHighBm   : TBitmap;
    FInActiveHighBm : TBitmap;
  public
    property    ActiveLowBm    : TBitmap             read FActiveLowBm    write SetActiveLowBm;
    property    InActiveLowBm  : TBitmap             read FInActiveLowBm  write SetInActiveLowBm;
    property    ActiveMidBm    : TBitmap             read FActiveMidBm    write SetActiveMidBm;
    property    InActiveMidBm  : TBitmap             read FInActiveMidBm  write SetInActiveMidBm;
    property    ActiveHighBm   : TBitmap             read FActiveHighBm   write SetActiveHighBm;
    property    InActiveHighBm : TBitmap             read FInActiveHighBm write SetInActiveHighBm;
  published
    property    IndicatorCount : Integer             read FIndicatorCount write SetIndicatorCount default 100;
    property    Value          : TSValue             read FValue          write SetValue          nodefault;
    property    ValueLowMid    : TSValue             read FValueLowMid    write SetValueLowMid    nodefault;
    property    ValueMidHigh   : TSValue             read FValueMidHigh   write SetValueMidHigh   nodefault;
    property    DisplayType    : TDisplayType        read FDisplayType    write SetDisplayType    default dtFilledFromLow;
    property    Orientation    : TDisplayOrientation read FOrientation    write SetOrientation    default doHorizontal;
    property    Spacing        : Integer             read FSpacing        write SetSpacing        default 2;
  private
}

    function    TKnobsIndicatorBar.GetMaxBmWidth: Integer;
    begin
      Result := 0;
      Result := Max( Result,   ActiveLowBm .Width);
      Result := Max( Result, InActiveLowBm .Width);
      Result := Max( Result,   ActiveMidBm .Width);
      Result := Max( Result, InActiveMidBm .Width);
      Result := Max( Result,   ActiveHighBm.Width);
      Result := Max( Result, InActiveHighBm.Width);
    end;

    function    TKnobsIndicatorBar.GetMaxBmHeight: Integer;
    begin
      Result := 0;
      Result := Max( Result,   ActiveLowBm .Height);
      Result := Max( Result, InActiveLowBm .Height);
      Result := Max( Result,   ActiveMidBm .Height);
      Result := Max( Result, InActiveMidBm .Height);
      Result := Max( Result,   ActiveHighBm.Height);
      Result := Max( Result, InActiveHighBm.Height);
    end;

    procedure   TKnobsIndicatorBar.FixDimensions;
    begin
      if Orientation = doHorizontal
      then begin
        Width  := IndicatorCount * ( GetMaxBmWidth + Spacing);
        Height :=                    GetMaxBmHeight;
      end
      else begin
        Width  :=                    GetMaxBmWidth;
        Height := IndicatorCount * ( GetMaxBmHeight + Spacing);
      end;
    end;

//  private

    procedure   TKnobsIndicatorBar.SetIndicatorCount( aValue: Integer);
    begin
      if aValue <> FIndicatorCount
      then begin
        FIndicatorCount := aValue;

        if ( Value < 0) or ( Value > aValue)
        then Value := 3 * aValue div 2;

        FixDimensions;
        Invalidate;
      end;
    end;

    procedure   TKnobsIndicatorBar.SetValue( aValue: TSignal);
    begin
      if aValue <> FValue
      then begin
        FValue := aValue;
        Invalidate;
      end;
    end;

    procedure   TKnobsIndicatorBar.SetValueLowMid( aValue: TSignal);
    begin
      if aValue <> FValueLowMid
      then begin
        FValueLowMid := aValue;
        Invalidate;
      end;
    end;

    procedure   TKnobsIndicatorBar.SetValueMidHigh( aValue: TSignal);
    begin
      if aValue <> FValueMidHigh
      then begin
        FValueMidHigh := aValue;
        Invalidate;
      end;
    end;

    procedure   TKnobsIndicatorBar.SetDisplayType( aValue: TKnobsDisplayType);
    begin
      if aValue <> FDisplayType
      then begin
        FDisplayType := aValue;
        Invalidate;
      end;
    end;

    procedure   TKnobsIndicatorBar.SetOrientation( aValue: TKnobsDisplayOrientation);
    begin
      if aValue <> FOrientation
      then begin
        FOrientation := aValue;
        FixDimensions;
        Invalidate;
      end;
    end;

    procedure   TKnobsIndicatorBar.SetSpacing( aValue: Integer);
    begin
      if aValue <> FSpacing
      then begin
        FSpacing := aValue;
        FixDimensions;
        Invalidate;
      end;
    end;

    procedure   TKnobsIndicatorBar.SetActiveLowBm( aValue: TBitmap);
    begin
      FActiveLowBm.Assign( aValue);
      FixDimensions;
      Invalidate;
    end;

    procedure   TKnobsIndicatorBar.SetInActiveLowBm( aValue: TBitmap);
    begin
      FInActiveLowBm.Assign( aValue);
      FixDimensions;
      Invalidate;
    end;

    procedure   TKnobsIndicatorBar.SetActiveMidBm( aValue: TBitmap);
    begin
      FActiveMidBm.Assign( aValue);
      FixDimensions;
      Invalidate;
    end;

    procedure   TKnobsIndicatorBar.SetInActiveMidBm( aValue: TBitmap);
    begin
      FInActiveMidBm.Assign( aValue);
      FixDimensions;
      Invalidate;
    end;

    procedure   TKnobsIndicatorBar.SetActiveHighBm( aValue: TBitmap);
    begin
      FActiveHighBm.Assign( aValue);
      FixDimensions;
      Invalidate;
    end;

    procedure   TKnobsIndicatorBar.SetInActiveHighBm( aValue: TBitmap);
    begin
      FInActiveHighBm.Assign( aValue);
      FixDimensions;
      Invalidate;
    end;

//  protected

    procedure   TKnobsIndicatorBar.AssignBitmaps; // virtual;
    begin
      FInactiveLowBm  := bmIndLowInactive;
      FActiveLowBm    := bmIndLowActive;
      FInactiveMidBm  := bmIndMidInactive;
      FActiveMidBm    := bmIndMidActive;
      FInactiveHighBm := bmIndHighInactive;
      FActiveHighBm   := bmIndHighActive;
      FInactiveLowBm .Transparent := False;
      FActiveLowBm   .Transparent := False;
      FInactiveMidBm .Transparent := False;
      FActiveMidBm   .Transparent := False;
      FInactiveHighBm.Transparent := False;
      FActiveHighBm  .Transparent := False;
      FixDimensions;
    end;

    function    TKnobsIndicatorBar.PaintActive( anIndex: Integer): Boolean;
    begin
      Result := False;
      case DisplayType of
        dtSingleFromLow, dtSingleFromHigh : Result := anIndex = Value - 1;
        dtFilledFromLow, dtFilledFromHigh : Result := anIndex < Value;
      end;
    end;

    procedure   TKnobsIndicatorBar.Paint; // override;
    var
      i : Integer;
      L : Integer;
      T : Integer;
      W : Integer;
      H : Integer;
    begin
      inherited;

      W := GetMaxBmWidth  + Spacing;
      H := GetMaxBmHeight + Spacing;

      if Orientation = doHorizontal
      then begin
        T := 0;

        if DisplayType in [ dtSingleFromHigh, dtFilledFromHigh]
        then begin
          W := - W;
          L := Width + W;
        end
        else L := 0;
      end
      else begin
        L := 0;

        if DisplayType in [ dtSingleFromLow, dtFilledFromLow]
        then begin
          H := - H;
          T := Height + H;
        end
        else T := 0;
      end;

      with Canvas
      do begin
        Pen.Color := clBlack;
        Brush.Color := clBlack;
        Rectangle( 0, 0, Width, Height);

        for i := 0 to IndicatorCount - 1
        do begin
          if PaintActive( i)
          then begin
            if i < ValueLowMid
            then Draw( L, T, FActiveLowBm  )
            else if i >= ValueMidHigh
            then Draw( L, T, FActiveHighBm)
            else Draw( L, T, FActiveMidBm);
          end
          else begin
            if i < ValueLowMid
            then Draw( L, T, FInActiveLowBm  )
            else if i >= ValueMidHigh
            then Draw( L, T, FInActiveHighBm)
            else Draw( L, T, FInActiveMidBm);
          end;

          if Orientation = doHorizontal
          then L := L + W
          else T := T + H;
        end;
      end;
    end;

//  public

    constructor TKnobsIndicatorBar.Create( anOwner: TComponent); // override;
    begin
      inherited;
      AssignBitmaps;
      IndicatorCount := 10;
      Value          :=  6;
      ValueLowMid    :=  5;
      ValueMidHigh   :=  8;
      DisplayType    := dtFilledFromLow;
      Orientation    := doHorizontal;
      Spacing        := 2;
      Enabled        := False;
      FixDimensions;
    end;


{ ========
  TIndicatorText = class( TGraphicControl)
  private
    FValue : TSValue;
  published
    property Width                   default 10;
    property Height                  default 10;
    property AutoSize                default false;
    property Align                   default alNone;
    property Alignment               default taCenter;
    property Color                   default ColorDisplay;
    property Transparent             default True;
    property Visible                 default True;
    property WordWrap                default False;
    property ParentColor             default False;
    property ParentFont              default False;
    property ShowHint                default False;
    property ParentShowHint          default False;
    property ShowAccelChar           default False;
    property Layout;
    property Anchors;
    property BiDiMode;
    property Caption;
    property Constraints;
    property Enabled;
    property FocusControl;
    property Font;
    property ParentBiDiMode;
    property PopupMenu;
  protected
}

    procedure   TKnobsIndicatorText.SetValue( aValue: TSignal);
    begin
      if aValue <> FValue
      then begin
        FValue := aValue;
        Invalidate;
      end;
    end;

    procedure   TKnobsIndicatorText.DoDrawText( var aRect: TRect; aFlags: Longint); // override;
    var
      Text: string;
    begin
      Text   := IntToStr( Round( FValue));
      aFlags := aFlags or DT_NOPREFIX;
      aFlags := DrawTextBiDiModeFlags( aFlags);
      Canvas.Font := Font;
      DrawText( Canvas.Handle, PChar( Text), Length( Text), aRect, aFlags);
    end;

//  public

    constructor TKnobsIndicatorText.Create( anOwner: TComponent); // override;
    begin
      inherited;

      Width          := 10;
      Height         := 10;
      AutoSize       := false;
      Align          := alNone;
      Alignment      := taCenter;
      Color          := KnobsColorDisplay;
      Transparent    := True;
      Visible        := True;
      WordWrap       := False;
      ParentColor    := False;
      ParentFont     := False;
      ShowHint       := False;
      ParentShowHint := False;
      ShowAccelChar  := False;

      with Font
      do begin
        Name  := 'Small Fonts';
        Size  := 7;
        Color := clWhite;
      end;
    end;



{ ========
  TDataViewer = class( TGraphicControl)
  // a viewer
  private
    FBackGround  : TBitmap;
    FData        : TSValues;
    FXOffset     : TSValue;
    FYOffset     : TSValue;
    FMaxValue    : TSValue;
    FMinValue    : TSValue;
    FXZoom       : TSValue;
    FYZoom       : TSValue;
    FBorderColor : TColor;
    FLineColor   : TColor;
    FFillColor   : TColor;
    FStyle       : TKnobsDVStyle;
  public
    property    Data       : TSValues      read FData       write SetData;
    property    Module     : TCustomModule read GetModule;
  published
    property    Background  : TBitmap       read FBackGround  write SetBackGround;
    property    XOffset     : TSValue       read FXOffset     write SetXOffset;    // default  0;
    property    YOffset     : TSValue       read FYOffset     write SetYOffset;    // default 75;
    property    MaxValue    : TSValue       read FMaxValue    write SetMaxValue;   // default High( SmallInt);
    property    MinValue    : TSValue       read FMinValue    write SetMinValue;   // default Low ( SmallInt);
    property    XZoom       : TSValue       read FXZoom       write SetXZoom;      // default 1;
    property    YZoom       : TSValue       read FYZoom       write SetYZoom;      // default 1;
    property    LineColor   : TColor        read FLineColor   write SetLineColor   default clWhite;
    property    FillColor   : TColor        read FFillColor   write SetFillColor   default clSilver;
    property    BorderColor : TColor        read FBorderColor write SetBorderColor default KnobsColorDisplay;
    property    Style       : TKnobsDVStyle read FStyle       write SetStyle       default dvsNoise;
    property    Width                                                              default 150;
    property    Height                                                             default 60;
    property    Color                                                              default KnobsColorDisplay;
  private
}

    function    TKnobsDataViewer.GetModule: TKnobsCustomModule;
    begin
      if Owner is TKnobsCustomModule
      then result := TKnobsCustomModule( Owner)
      else result := nil;
    end;

    procedure   TKnobsDataViewer.SetBackGround( aValue: TBitmap);
    begin
      FBackGround.Assign( aValue);
      Invalidate;
    end;

    procedure   TKnobsDataViewer.SetData( const aValue: TSignalArray);
    begin
      FData := aValue;
      Invalidate;
    end;

    procedure   TKnobsDataViewer.SetXOffset( aValue: TSignal);
    begin
      if aValue <> FXOffset
      then begin
        FXOffset := aValue;
        Invalidate;
      end;
    end;

    procedure   TKnobsDataViewer.SetYOffset( aValue: TSignal);
    begin
      if aValue <> FYOffset
      then begin
        FYOffset := aValue;
        Invalidate;
      end;
    end;

    procedure   TKnobsDataViewer.SetMaxValue( aValue: TSignal);
    begin
      if aValue <> FMaxValue
      then begin
        FMaxValue := aValue;
        Invalidate;
      end;
    end;

    procedure   TKnobsDataViewer.SetMinValue( aValue: TSignal);
    begin
      if aValue <> FMinValue
      then begin
        FMinValue := aValue;
        Invalidate;
      end;
    end;

    procedure   TKnobsDataViewer.SetXZoom( aValue: TSignal);
    begin
      if aValue <> FXZoom
      then begin
        FXZoom := aValue;
        Invalidate;
      end;
    end;

    procedure   TKnobsDataViewer.SetYZoom( aValue: TSignal);
    begin
      if aValue <> FYZoom
      then begin
        FYZoom := aValue;
        Invalidate;
      end;
    end;

    procedure   TKnobsDataViewer.SetLineColor( aValue: TColor );
    begin
      if aValue <> FLineColor
      then begin
        FLineColor := aValue;
        Invalidate;
      end;
    end;

    procedure   TKnobsDataViewer.SetFillColor( aValue: TColor);
    begin
      if aValue <> FFillColor
      then begin
        FFillColor := aValue;
        Invalidate;
      end;
    end;

    procedure   TKnobsDataViewer.SetBorderColor( aValue: TColor );
    begin
      if aValue <> FBorderColor
      then begin
        FBorderColor := aValue;
        Invalidate;
      end;
    end;

    procedure   TKnobsDataViewer.SetStyle( aValue: TKnobsDVStyle);
    begin
      if aValue <> FStyle
      then begin
        FStyle := aValue;
          case aValue of
            dvsBlank   : FillBlank;
            dvsSine    : FillSine;
            dvsTri     : FillTri;
            dvsSaw     : FillSaw;
            dvsSquare  : FillSquare;
            dvsNoise   : FillNoise;
            dvsEnvAr   : FillEnvAr;
            dvsEnvAhd  : FillEnvAhd;
            dvsEnvAdsr : FillEnvAdsr;
            else         FillBlank;
        end;
        Invalidate;
      end;
    end;

//  private

    procedure   TKnobsDataViewer.FillBlank;
    begin
      SetLength( FData, 0);
    end;

    procedure   TKnobsDataViewer.FillSine;
    var
      i : Integer;
    begin
      SetLength( FData, Width);
      for i := 0 to Width
      do FData[ i] := ( MaxValue - MinValue) * ( Sin( 2 * pi * i / width) + 1) / 2;
    end;

    procedure   TKnobsDataViewer.FillTri;
    var
      i         : Integer;
      PhaseAccu : TSignal;
      HalfValue : TSignal;
    begin
      SetLength( FData, Width);
      HalfValue := ( MaxValue - MinValue) / 2;
      for i := 0 to Width
      do begin
        PhaseAccu := i / Width;
        if PhaseAccu <= 0.5
        then FData[ i] := HalfValue * 4 * PhaseAccu
        else FData[ i] := HalfValue * 4 * ( 1 - PhaseAccu);
      end;
    end;

    procedure   TKnobsDataViewer.FillSaw;
    var
      i         : Integer;
      PhaseAccu : TSignal;
      HalfValue : TSignal;
    begin
      SetLength( FData, Width);
      HalfValue := ( MaxValue - MinValue) / 2;
      for i := 0 to Width
      do begin
        PhaseAccu := i / Width;
        FData[ i] := HalfValue * 2 * PhaseAccu;
      end;
    end;

    procedure   TKnobsDataViewer.FillSquare;
    var
      i         : Integer;
      PhaseAccu : TSignal;
    begin
      SetLength( FData, Width);
      for i := 0 to Width
      do begin
        PhaseAccu := i / Width;
        if PhaseAccu < 0.5
        then FData[ i] := MinValue
        else FData[ i] := MaxValue;
      end;
    end;

    procedure   TKnobsDataViewer.FillNoise;
    var
      i : Integer;
    begin
      SetLength( FData, Width div 5);
      for i := 0 to Length( FData) - 1
      do FData[ i] := MinValue + ( MaxValue - MinValue) * Random;
    end;

    procedure   TKnobsDataViewer.FillEnvAr;
    var
      i    : Integer;
      Knee : Integer;
    begin
      SetLength( FData, Width);
      Knee := Width div 5;
      for i := 0 to Length( FData) - 1
      do begin
        if i < Knee
        then FData[ i] := LinAttack( i, Knee, MinValue, MaxValue)
        else FData[ i] := ExpDecay( i - Knee, Width - Knee, MaxValue, MinValue);
      end;
    end;

    procedure   TKnobsDataViewer.FillEnvAhd;
    var
      i     : Integer;
      Knee1 : Integer;
      Knee2 : Integer;
    begin
      SetLength( FData, Width);
      Knee1 := Width div 6;
      Knee2 := Width div 3;
      for i := 0 to Length( FData) - 1
      do begin
        if i < Knee1
        then FData[ i] := LinAttack( i, Knee1, MinValue, MaxValue)
        else if i < Knee2
        then FData[ i] := MaxValue
        else FData[ i] := ExpDecay( i - Knee2, Width - Knee2, MaxValue, MinValue);
      end;
    end;

    procedure   TKnobsDataViewer.FillEnvAdsr;
    var
      i     : Integer;
      Knee1 : Integer;
      Knee2 : Integer;
      Knee3 : Integer;
      Sust  : TSignal;
    begin
      SetLength( FData, Width);
      Knee1 := Width div 8;
      Knee2 := Width div 2;
      Knee3 := 2 * Width div 3;
      Sust  := MinValue + ( MaxValue - MinValue) / 3;
      for i := 0 to Length( FData) - 1
      do begin
        if i < Knee1
        then FData[ i] := LinAttack( i, Knee1, MinValue, MaxValue)
        else if i < Knee2
        then FData[ i] := ExpDecay( i - Knee1, Knee3 - Knee2, MaxValue, Sust)
        else if i < Knee3
        then FData[ i] := Sust
        else FData[ i] := ExpDecay( i - Knee3, Width - Knee3, Sust, MinValue);
      end;
    end;

//  protected

    procedure   TKnobsDataViewer.WMNCHitTest( var aMessage: TWMNCHitTest);  // message WM_NCHITTEST;
    begin
      if csDesigning in ComponentState
      then inherited
      else aMessage.Result := HTTRANSPARENT;
    end;

    procedure   TKnobsDataViewer.WMEraseBackground( var aMsg: TWMEraseBkgnd); // message WM_ERASEBKGND;
    begin
      aMsg.Result := -1;
    end;

    procedure   TKnobsDataViewer.MouseDown( Button: TMouseButton; Shift: TShiftState; X, Y: Integer); // override;
    var
      aModule: TKnobsCustomModule;
    begin
      if csDesigning in ComponentState
      then inherited
      else begin
        // Pass it on
        aModule := Module;

        if Assigned( aModule)
        then begin
          aModule.MouseCapture := True;
          aModule.MouseDown( Button, Shift, X + Left, Y + Top);
        end;
      end;
    end;

    function    TKnobsDataViewer.ScaleData( anIndex: Integer): Integer;
    var
      aVal: TSignal;
    begin
      aVal   := Clip( FData[ anIndex], MinValue, MaxValue);
      Result := Height - Round( Clip( aVal * YZoom * ( Height / ( MaxValue - MinValue)) + YOffset, 0, Height));
    end;

    function    TKnobsDataViewer.ScaleX( anIndex: Integer): Integer;
    begin
      Result := Round( Clip( anIndex * XZoom * ( Width / Length( FData)) - XOffset, 0, Width));
    end;

    procedure   TKnobsDataViewer.Paint; // override;
    var
      i : Integer;
    begin
      inherited;

      with Canvas
      do begin
        if csDesigning in ComponentState
        then begin
          Pen.Style   := psDot;
          Brush.Style := bsClear;
          Rectangle( 0, 0, Width, Height);
          Pen.Style   := psSolid;
          Brush.Style := bsSolid;
        end;

        if Assigned( FBackGround) and not FBackGround.Empty
        then StretchDraw( Rect( 0, 0, Left, Width), FBackGround)
        else begin
          Pen.Color   := BorderColor;
          Brush.Color := Color;
          Rectangle( 0, 0, Width, Height);
        end;

        if Length( FData) > 0
        then begin
          Pen.Color := FillColor;
          for i := 1 to Length( FData) - 1
          do begin
            MoveTo( ScaleX( i), Height - 1);
            LineTo( ScaleX( i), ScaleData( i));
          end;

          Pen.Color := LineColor;
          MoveTo( ScaleX( 0), ScaleData( 0));
          for i := 1 to Length( FData) - 1
          do LineTo( ScaleX( i), ScaleData( i));
        end;
      end;
    end;

//  public

    constructor TKnobsDataViewer.Create( anOwner: TComponent); // override;
    begin
      inherited;

      ControlStyle := ControlStyle + [ csOpaque];
      FBackGround  := TBitmap.Create;
      XOffset      := 0;
      YOffset      := 30;
      MaxValue     := High( SmallInt);
      MinValue     := Low ( SmallInt);
      XZoom        := 1;
      YZoom        := 1;
      Width        := 150;
      Height       := 60;
      LineColor    := clWhite;
      FillColor    := clSilver;
      BorderColor  := KnobsColorDisplay;
      Color        := KnobsColorDisplay;
      Style        := dvsNoise;
    end;


{ ========
  TConnector = class( TGraphicControl)
  private
    FBitmap    : TBitmap;
    FWireColor : TColor;
    FLinks     : TList;   // Connections, not wires
    FMouseDown : Boolean;
    FConnected : Boolean; // True when at least one connection exists
    FHighLight : Boolean; // Highlighted connections
    FVisited   : Boolean;
  protected
    property    HighLight: Boolean read FHighLight write SetHighLight;
  public
    property    Connected : Boolean       read FConnected write SetConnected;
    property    Module    : TCustomModule read GetModule;
    property    ModuleName: string        read GetModuleName;
  published
    property    WireColor : TColor        read FWireColor write SetWireColor default clBlue;
  private
}

    procedure   TKnobsConnector.SetConnected( aValue: Boolean);
    begin
      if aValue <> FConnected
      then begin
        FConnected := aValue;
        Invalidate;
      end;
    end;

    function    TKnobsConnector.GetModule: TKnobsCustomModule;
    begin
      if Owner is TKnobsCustomModule
      then result := TKnobsCustomModule( Owner)
      else result := nil;
    end;

    function    TKnobsConnector.GetModuleName: string;
    var
      aModule: TKnobsCustomModule;
    begin
      aModule := Module;

      if Assigned( aModule)
      then Result := aModule.Name
      else Result := '';
    end;

    procedure   TKnobsConnector.SetHighLight( aValue: Boolean);
    var
      i : Integer;
    const
      Depth : Integer = 0;
    begin
      if aValue <> FHighLight
      then begin
        if not FVisited
        then begin
          try
            FVisited   := True;
            FHighLight := aValue;
            Invalidate;
            Inc( Depth);

            try
              with FLinks
              do
                for i := 0 to Count - 1
                do TKnobsConnector( Items[ i]).HighLight := aValue;
            finally
              Dec( Depth);
            end;

          finally
            FVisited := False;
          end;
        end;
      end;
      if Depth = 0
      then SendHighlightNotification;
    end;

    procedure   TKnobsConnector.SetWireColor( aValue: TColor);
    begin
      if aValue <> FWireColor
      then begin
        FWireColor := aValue;
        SendColorNotification( FWireColor);
        Invalidate;
      end;
    end;

    function    TKnobsConnector.FindWirePanel: TKnobsWirePanel;
    var
      aParent : TWinControl;
    begin
      Result  := nil;
      aParent := Parent;

      while Assigned( aParent) and not ( aParent is TKnobsWirePanel)
      do aParent :=  aParent.Parent;

      if aParent is TKnobsWirePanel
      then Result := TKnobsWirePanel( aParent);
    end;

    procedure   TKnobsConnector.AddLink( aConnector: TKnobsConnector);
    begin
      if not HasDirectlinkTo( aConnector)
      then begin
        FLinks.Add( aConnector);
        Connected := True;
      end;
    end;

    procedure   TKnobsConnector.DeleteLink( aConnector: TKnobsConnector);
    begin
      Connected := False;

      with FLinks
      do begin
        Remove( aConnector);
        Connected := Count > 0;
      end;
    end;

    function    TKnobsConnector.HasDirectlinkTo( aConnector: TKnobsConnector): Boolean;
    begin
      Result := FLinks.IndexOf( aConnector) >= 0;
    end;

    function    TKnobsConnector.FindConnector( aScreenPoint: TPoint): TKnobsConnector;
    var
      aControl : TControl;
    begin
      Result := nil;
      aControl := FindDragTarget( aScreenPoint, True);

      if Assigned( aControl) and ( aControl is TKnobsConnector)
      then Result := TKnobsConnector( aControl);
    end;

    procedure   TKnobsConnector.DrawLine( X, Y: Integer);
    var
      aDC    : HDC;
      aPen   : HPen;
      OldObj : HGDIOBJ;
      aColor : TColor;
      aWp    : TKnobsWirePanel;
    begin
      ScrollMouseInView( X, Y);
      aColor := clWhite;
      aDC    := GetDC( 0);

      try
        SetRop2( aDC, R2_XORPEN);
        aPen   := CreatePen( PS_SOLID, 2, ColorToRGB( aColor));
        OldObj := SelectObject( aDC, aPen);

        try
          aWp := FindWirePanel;
          if Assigned( aWp)
          then
            with aWp, ClientOrigin
            do IntersectClipRect( aDC, X, Y, X + ClientWidth, Y + ClientHeight);
          with ClientToScreen( CenterPoint)
          do MoveToEx( aDC, X, Y, nil);
          with ClientToScreen( Point( X, Y))
          do LineTo( aDC, X, Y);
        finally
          SelectObject( aDC, OldObj);
          DeleteObject( aPen);
        end;

      finally
        ReleaseDC( 0, aDC);
      end;
    end;

    procedure   TKnobsConnector.SendWireNotification( Msg: Cardinal; aConnector: TKnobsConnector);
    var
      aWp : TKnobsWirePanel;
    begin
      aWp := FindWirePanel;

      if Assigned( aWp)
      then aWp.Perform( Msg, WPARAM( Self), LPARAM( aConnector));
    end;

    procedure   TKnobsConnector.SendColorNotification( aColor: TColor);
    var
      aWp : TKnobsWirePanel;
    begin
      aWp := FindWirePanel;

      if Assigned( aWp)
      then aWp.Perform( KNOBS_UM_COLORCHANGE, WPARAM( Self), LPARAM( aColor));
    end;

    procedure   TKnobsConnector.SendHighlightNotification;
    var
      aWp : TKnobsWirePanel;
    begin
      aWp := FindWirePanel;

      if Assigned( aWp)
      then aWp.Perform( KNOBS_UM_HIGHLIGHT, WPARAM( Self), 0);
    end;

    procedure   TKnobsConnector.ScrollMouseInView( anX, anY: LongInt);
    var
      aWp : TKnobsWirePanel;
    begin
      aWp := FindWirePanel;

      if Assigned( aWp)
      then aWp.ScrollPointInView( aWp.ScreenToClient( ClientToScreen( Point( anX, anY))));
    end;

//  protected

    function    TKnobsConnector.CenterPoint: TPoint;
    begin
      with Result
      do begin
        x := Width  div 2;
        y := Height div 2;
      end;
    end;

    procedure   TKnobsConnector.MouseDown( Button: TMouseButton; Shift: TShiftState; X, Y: Integer); // override;
    var
      aWP : TKnobsWirePanel;
    begin
      inherited;

      if FMouseDown
      then Exit;

      if Button = mbLeft
      then begin
        FMouseDown := True;
        GDistance  := Point( X, Y);
        DrawLine( X, Y);
        HighLight := True;
      end
      else if Button = mbRight
      then begin
        aWP := FindWirePanel;
        if Assigned( aWP)
        then aWP.HandleRightClick( Self);
      end;
    end;

    procedure   TKnobsConnector.MouseUp( Button: TMouseButton; Shift: TShiftState; X, Y: Integer); // override;
    var
      aConnector : TKnobsConnector;
      isMove     : Boolean;
    begin
      inherited;

      if FMouseDown
      then begin
        with GDistance
        do DrawLine( X, Y);

        FMouseDown := False;
        isMove        := ( ssDouble in Shift) or ( ssCtrl in Shift);
        HighLight     := False;
        Screen.Cursor := crdefault;
        aConnector    := FindConnector( ClientToScreen( Point( X, Y)));

        if Assigned( aConnector) and aConnector.CanAccept( Self, isMove)
        then begin
          if isMove
          then MoveLinksTo( aConnector)
          else ConnectTo  ( aConnector);
        end
        else begin
          if not Assigned( aConnector) and isMove
          then DisconnectAll;
        end;
      end;
    end;

    procedure   TKnobsConnector.MouseMove( Shift: TShiftState; X, Y: Integer); // override;
    var
      aConnector : TKnobsConnector;
      isMove     : Boolean;
    begin
      inherited;

      if FMouseDown
      then begin
        with GDistance
        do DrawLine( X, Y);

        GDistance := Point( X, Y);

        with GDistance
        do DrawLine( X, Y);

        isMove     := ( ssDouble in Shift) or ( ssCtrl in Shift);
        aConnector := FindConnector( ClientToScreen( Point( X, Y)));

        if Assigned( aConnector)
        then begin
          if aConnector.CanAccept( Self, isMove)
          then begin
            if isMove
            then Screen.Cursor := crMultiDrag
            else Screen.Cursor := crHandPoint;
          end
          else Screen.Cursor := crDefault;
        end
        else Screen.Cursor := crdefault;
      end;
    end;

    function    TKnobsConnector.FindOutput: TKnobsConnector;
    var
      i : Integer;
    begin
      if Self is TKnobsOutput
      then Result := Self
      else begin
        Result := nil;

        if not FVisited
        then begin
          FVisited := True;

          try
            with FLinks
            do
              for i := 0 to Count - 1
              do begin
                Result := TKnobsConnector( Items[ i]).FindOutput;
                if Assigned( Result)
                then Break;
              end;
          finally
            FVisited := False;
          end;

        end;
      end;
    end;

    function    TKnobsConnector.IsConnectedToOutput: Boolean;
    begin
      Result := FindOutput <> nil;
    end;

    procedure   TKnobsConnector.FindDistanceToOutput( var aDistance: Integer);
    // Should at the top level be called with aDistance set to -1
    // so that when no originating output can be found the final
    // value of aDistance will still be -1.
    // When this module is an output, just set aDistance to 0 and return,
    // otherwise iterate over all links to find one that ends on an output.
    // If so, that recursive call will return 0, which then will be
    // inceremented before this function returns .. i.e it is on further away
    // for every return executed.
    var
      i : Integer;
    begin
      if Self is TKnobsOutput
      then aDistance := 0
      else begin
        if not FVisited
        then begin
          FVisited := True;
          try
            with FLinks
            do
              for i := 0 to Count - 1
              do begin
                TKnobsConnector( Items[ i]).FindDistanceToOutput( aDistance);
                if aDistance >= 0
                then begin
                  Inc( aDistance);
                  Break;
                end;
              end;
          finally
            FVisited := False;
          end;
        end;
      end;
    end;

    function    TKnobsConnector.IsConnectedTo( aConnector: TKnobsConnector): Boolean;
    var
      i : Integer;
    begin
      Result := False;

      if not FVisited
      then begin
        FVisited := True;

        try
          with FLinks
          do
            for i := 0 to Count - 1
            do begin
              Result :=
                HasDirectlinkTo( aConnector) or
                TKnobsConnector( Items[ i]).IsConnectedTo( aConnector);

              if Result
              then Break;
            end;
        finally
          FVisited := False;
        end;
      end;
    end;

    function    TKnobsConnector.OutputColor: TColor;
    var
      aConnector : TKnobsConnector;
    begin
      aConnector := FindOutput;

      if Assigned( aConnector)
      then Result := aConnector.WireColor
      else Result := CL_NO_OUTPUT;
    end;

    function    TKnobsConnector.CanAccept( aSource: TKnobsConnector; isMove: Boolean): Boolean;
    begin
      Result := False;

      if Assigned( aSource) and ( aSource <> Self)
      then begin
        if isMove
        then begin
          Result :=
            ( aSource is TKnobsOutput) or
            not ( IsConnectedToOutput and aSource.IsConnectedToOutput);
        end
        else begin
          if IsConnectedToOutput
          then Result := not aSource.IsConnectedToOutput
          else Result := not IsConnectedTo( aSource);
        end;
      end;
    end;

    procedure   TKnobsConnector.MoveLinksTo( aConnector: TKnobsConnector);
    var
      aMoved  : TKnobsConnector;
      aWp     : TKnobsWirePanel;
      Changed : Boolean;
    begin
      if Assigned( aConnector) and ( aConnector <> Self)
      then begin
        Changed := False;
        aWp     := FindWirePanel;
        if Assigned( aWp)
        then begin
          with FLinks
          do begin
            if Count > 0
            then begin
              aWp.BeginStateChange;
              try
                while Count > 0
                do begin
                  aMoved := TKnobsConnector( Items[ Count - 1]);
                  DisConnectFrom( aMoved);
                  aConnector.ConnectTo( aMoved);
                  Changed := True;
                end;
              finally
                aWp.EndStateChange( Changed);
              end;
            end;
          end;
        end;
      end;
    end;

    procedure   TKnobsConnector.ConnectTo( aConnector: TKnobsConnector);
    var
      aWp     : TKnobsWirePanel;
      Changed : Boolean;
    begin
      if Assigned( aConnector) and aConnector.CanAccept( Self, False)
      then begin
        Changed := False;
        aWp     := FindWirePanel;
        if Assigned( aWp)
        then begin
          aWp.BeginStateChange;
          try
            AddLink( aConnector);
            aConnector.AddLink( Self);
            SendWireNotification( KNOBS_UM_CONNECTIONADD, aConnector);
            Changed := True;
          finally
            aWp.EndStateChange( Changed);
          end;
        end;
      end;
    end;

    procedure   TKnobsConnector.DisconnectFrom( aConnector: TKnobsConnector);
    var
      aWp     : TKnobsWirePanel;
      Changed : Boolean;
    begin
      if HasDirectlinkTo( aConnector)
      then begin
        Changed := False;
        aWp     := FindWirePanel;
        if Assigned( aWp)
        then begin
          aWp.BeginStateChange;
          try
            DeleteLink( aConnector);
            aConnector.DeleteLink( Self);
            SendWireNotification( KNOBS_UM_CONNECTIONREMOVE, aConnector);
            Changed := True;
          finally
            aWp.EndStateChange( Changed);
          end;
        end;
      end;
    end;

    function   TKnobsConnector.DisConnectAll: Boolean;
    var
      i        : Integer;
      OldLinks : TList;
      aWp      : TKnobsWirePanel;
      Changed  : Boolean;
    begin
      Changed := False;
      if FLinks.Count > 0
      then begin
        Changed := False;
        aWp     := FindWirePanel;
        if Assigned( aWp)
        then begin
          aWp.BeginStateChange;
          try
            OldLinks := TList.Create;

            try
              with FLinks do
              begin
                for i := 0 to Count - 1
                do OldLinks.Add( Items[ i]);

                while Count > 0
                do begin
                  DisconnectFrom( FLinks[ Count - 1]);
                  Changed := True;
                end;
              end;

              with OldLinks
              do begin
                if Count > 1
                then begin
                  for i := 1 to Count - 1
                  do begin
                    if TKnobsConnector( Items[ i - 1]).HasParent and TKnobsConnector( Items[ i]).HasParent
                    then begin
                      TKnobsConnector( Items[ i - 1]).ConnectTo( TKnobsConnector( Items[ i]));
                      Changed := True;
                    end;
                  end;
                end;
              end;
            finally
              OldLinks.Free;
            end;
          finally
            aWp.EndStateChange( Changed);
          end;
        end;
      end;
      Result := Changed;
    end;

    procedure   TKnobsConnector.DisConnectDownStream;
    var
      aWp     : TKnobsWirePanel;
      Changed : Boolean;
    begin
      if FLinks.Count > 0
      then begin
        Changed := False;
        aWp     := FindWirePanel;
        if Assigned( aWp)
        then begin
          aWp.BeginStateChange;
          try
            with FLinks do
            begin
              while Count > 0
              do begin
                DisconnectFrom( FLinks[ Count - 1]);
                Changed := True;
              end;
            end;
          finally
            aWp.EndStateChange( Changed);
          end;
        end;
      end;
    end;

    procedure   TKnobsConnector.DisConnectDownStreamRecursive;
    var
      aWp     : TKnobsWirePanel;
      Changed : Boolean;
    begin
      if ( FLinks.Count > 0) and not FVisited
      then begin
        Changed := False;
        aWp     := FindWirePanel;
        if Assigned( aWp)
        then begin
          aWp.BeginStateChange;
          FVisited := True;
          try
            with FLinks do
            begin
              while Count > 0
              do begin
                if TKnobsConnector( FLinks[ Count - 1]).DisConnectAll
                then Changed := True;
              end;
            end;
          finally
            FVisited := False;
            aWp.EndStateChange( Changed);
          end;
        end;
      end;
    end;

    procedure   TKnobsConnector.Paint; // override;
    begin
      with Canvas
      do begin
        if FHighLight or FConnected
        then begin
          Pen.Width := 1;

          if FHighLight
          then begin
            Brush.Color := CL_HIGHLIGHT;
            Pen  .Color := CL_HIGHLIGHT;
          end
          else if FConnected
          then begin
            Brush.Color := WireColor;
            Pen  .Color := clBlack;
          end;

          if Self is TKnobsOutput
          then Rectangle( 0, 0, Width, Height)
          else Ellipse( 0, 0, Width, Height);

          Pen.Color := clWhite;

          Arc(
            Width  * 3 div 4         ,
            Height * 3 div 4         ,
            Width  - Width  * 3 div 4,
            Height - Height * 3 div 4,
            Width div 2              ,
            0                        ,
            0                        ,
            Height
          );

          Pen.Color := clGray;

          Arc(
            Width  * 3 div 4         ,
            Height * 3 div 4         ,
            Width  - Width  * 3 div 4,
            Height - Height * 3 div 4,
            0                        ,
            Width div 2              ,
            Height                   ,
            0
          );
        end
        else begin
          Draw( 0, 0, FBitmap);
          Brush.Color := WireColor;
          Pen  .Color := WireColor;
          Ellipse( 3, 3, Width - 3, Height - 3);
        end;
      end;
    end;

//  public

    constructor TKnobsConnector.Create( anOwner: TComponent); // override;
    begin
      inherited;

      AssignBitmap;
      Width      := FBitmap.Width;
      Height     := FBitmap.Height;
      FLinks     := TList.Create;
      FWireColor := clBlue;
    end;

    destructor  TKnobsConnector.Destroy; // override;
    begin
      DisConnectAll;
      FLinks.Free;
      inherited;
    end;

    procedure   TKnobsConnector.Dump( var aFile: TextFile; anInd: Integer);
    begin
      WriteLnInd( aFile, anInd, Format( 'connector     : %s', [ ConnectorName( self)]));
      DumpLinks( aFile, anInd + 1);
      WriteLnInd( aFile, anInd,         'end connector');
    end;

    procedure   TKnobsConnector.DumpFlat( var aFile: TextFile; anInd: Integer);
    begin
      WriteLnInd( aFile, anInd, Format( 'links to : %s', [ ConnectorName( Self)]));
    end;

    procedure   TKnobsConnector.DumpLinks( var aFile: TextFile; anInd: Integer);
    var
      i : Integer;
    begin
      for i := 0 to FLinks.Count - 1
      do TKnobsConnector( FLinks[ i]).DumpFlat( aFile, anInd);
    end;

    procedure   TKnobsConnector.Log( const aMsg: string);
    begin
      if Parent is TKnobsCustomModule
      then TKnobsCustomModule( Parent).Log( Format( 'connector %s( %s)', [ ConnectorName( Self), aMsg]));
    end;

    function    TKnobsConnector.DistanceToOutput: Integer;
    // Return the distance to the output, there should be one at most
    // (as code elswhere should prohibit output to output connections,
    // raising an EShortCircuit on that).
    // When an output can not be reached return -1
    // When this connector is an output return 0
    begin
      Result := -1;                     // Assume no connection to an output ...
      FindDistanceToOutput( Result);
    end;


{ ========
  TInput = class( TConnector)
  protected
}

    procedure   TKnobsInput.AssignBitmap; // override;
    begin
      FBitmap := bmConIn;
      FBitmap.Transparent := True;
    end;


{ ========
  TOutput = class( TConnector)
  protected
}

    procedure   TKnobsOutput.AssignBitmap; // override;
    begin
      FBitmap := bmConOut;
      FBitmap.Transparent := False;
    end;


{ ========
  TTextLabel = class( TLabel)
  protected
    property Align;
    property Alignment;
    property Anchors;
    property AutoSize;
    property BiDiMode;
    property Caption;
    property Color;
    property Constraints;
    property DragCursor;
    property DragKind;
    property DragMode;
    property Enabled;
    property FocusControl;
    property Font;
    property ParentBiDiMode;
    property ParentColor;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowAccelChar;
    property ShowHint;
    property Transparent;
    property Layout;
    property Visible;
    property WordWrap;
  protected
}

    procedure   TKnobsTextLabel.DoDrawText( var aRect: TRect; aFlags: Longint); // override;
    var
      Text: string;
    begin
      Text   := GetLabelText;
      aFlags := aFlags or DT_NOPREFIX;
      aFlags := DrawTextBiDiModeFlags( aFlags);
      Canvas.Font := Font;
      DrawText( Canvas.Handle, PChar( Text), Length( Text), aRect, aFlags);
    end;

//  public

    constructor TKnobsTextLabel.Create( anOwner: TComponent); // override;
    begin
      inherited;

      with Font
      do begin
        Name := 'Small Fonts';
        Size := 7;
      end;

      Enabled     := False;
      Transparent := True;
    end;


{ ========
  TLabelEditor = class( TEdit)
  private
    FLabel : TEditLabel;
  private
}

    procedure   TKnobsLabelEditor.WMKillFocus( var aMessage: TWMKillFocus ); //  message WM_KILLFOCUS;
    begin
      inherited;

      Visible := False;
    end;

    procedure   TKnobsLabelEditor.Do_Exit( aSender: TObject);
    begin
      Visible := False;
    end;

    procedure   TKnobsLabelEditor.Do_KeyDown( aSender: TObject; var aKey: Word; aShift: TShiftState);
    begin
      if aShift = []
      then
        case aKey Of

          VK_RETURN:
            begin
              FLabel.SetText( Text);
              Visible := False;
              aKey    := 0;
            end;

          VK_ESCAPE :
            begin
              Visible := False;
              aKey    := 0;
            end;

        end;
    end;

//  protected

    procedure   TKnobsLabelEditor.SetLabel( aLabel: TKnobsEditLabel);
    begin
      FLabel  := aLabel;
      Parent  := aLabel.Parent;
      Text    := aLabel.Caption;
      Visible := True;
      SetFocus;
    end;

//  public

    constructor TKnobsLabelEditor.Create( anOwner: TComponent); // override;
    begin
      if Assigned( GLabelEditor)
      then Exit;

      inherited;

      ControlStyle := ControlStyle + [ csOpaque];
      Visible      := False;
      Left         := 1;
      Top          := 1;
      OnExit       := Do_Exit;
      OnKeyDown    := Do_KeyDown;
      GLabelEditor := Self;
    end;

    destructor  TKnobsLabelEditor.Destroy; // override;
    begin
      if Assigned( Owner)
      then Owner.RemoveComponent( Self);
      inherited;
    end;


{ ========
  TEditLabel = class( TTextLabel) // a label with a popup editor
  private
    FOnChanged : TOnEditLabelChanged;
  public
    property    Module: TCustomModule read GetModule;
  published
    property    OnChanged: TOnEditLabelChanged read FOnChanged write FOnChanged;
  private
}

    function    TKnobsEditLabel.GetModule: TKnobsCustomModule;
    begin
      if Parent is TKnobsCustomModule
      then Result := TKnobsCustomModule( Parent)
      else Result := nil;
    end;

//  protected

    procedure   TKnobsEditLabel.SetText( const aValue: string);
    begin
      Caption := aValue;

      if Assigned( Module)
      then Module.Title := aValue;

      if Assigned( FOnChanged)
      then FOnChanged( Self, aValue);
    end;

    procedure   TKnobsEditLabel.MouseDown( Button: TMouseButton; Shift: TShiftState; X, Y: Integer); // override;

    // We only need the double click here to start a label editor, all other
    // events should be handled by the containing module.

    var
      aModule : TKnobsCustomModule;
    begin
      if not ( csDesigning in ComponentState)
      then begin
        if ( ssDouble in Shift) and ( Button = mbLeft)
        then begin
          // Handle the double click
          NeedsLabelEditor;
          GLabelEditor.SetLabel( Self);
        end
        else begin
          // Pass it on
          aModule := Module;

          if Assigned( aModule)
          then begin
            aModule.MouseCapture := True;
            aModule.MouseDown( Button, Shift, X + Left, Y + Top);
          end;
        end;
      end;
    end;

//  public

    constructor TKnobsEditLabel.Create( anOwner: TComponent); // override;
    begin
      inherited;

      AutoSize  := False;
      Width     := 75;
      Enabled   := True;
      ShowHint  := True;
      if Owner is TKnobsCustomModule
      then Hint := TKnobsCustomModule( Owner).Title;
    end;



{ ========
  TBox = class( TBevel)
  published
    property    Enabled default False;
  public
}

    constructor TKnobsBox.Create( anOwner: TComponent); // override;
    begin
      inherited;

      Enabled := False;
    end;


{ ========
  TDrawPanel = class( TCustomControl)
   protected
}



{
  TWire = class
  private
    FSource       : TConnector;
    FDestination  : TConnector;
    FHighlight    : Boolean;
    FBezierParams : TBezierParams;
  public
    property    Source       : TConnector    read FSource;
    property    Destination  : TConnector    read FDestination;
    property    BezierParams : TBezierParams read FBezierParams;
  private
}

    function    TKnobsWire.GetSourceName: string;
    begin
      Result := ConnectorName( Source.FindOutput); // Trace to ultimate source (an output)
    end;

    function    TKnobsWire.GetDestinationName: string;
    begin
      Result := ConnectorName( Destination); // Use factual destination
    end;

    function    TKnobsWire.GetFullName: string;
    begin
      Result := Format( '%s -> %s', [ ConnectorName( Source), ConnectorName( Destination)]);
    end;

//  public

    constructor TKnobsWire.Create( aSource, aDestination: TKnobsConnector);
    begin
      inherited Create;

      FSource      := aSource;
      FDestination := aDestination;

      Wiggle;
    end;

    procedure   TKnobsWire.Wiggle;
    begin
      with FBezierParams
      do begin
        Angle1    := 15 + Random( 21);
        Strength1 := 20 + Random( 21);
        Angle2    := 20 + Random( 21);
        Strength2 := 20 + Random( 21);
      end;
    end;

    procedure   TKnobsWire.Dump( var aFile: TextFile; anInd: Integer);
    begin
      WriteLnInd( aFile, anInd, Format( 'wire : %s', [ WireName( Source, Destination)]));
    end;

    procedure   TKnobsWire.FixWire;
    // When the wire is input to input dix it such such that the Source connector is closest
    // to the originating output connector. When there is no output in the chain  just
    // do something arbitrary - which must then be fixed later once an output gets to
    // be connected.
    begin
      if ( Source is TKnobsInput) and ( Destination is TKnobsInput)
      then begin
        if Source.DistanceToOutput > Destination.DistanceToOutput
        then SwapConnectors;
      end;
    end;

    procedure   TKnobsWire.SwapConnectors;
    var
      aTmp: TKnobsConnector;
    begin
      aTmp         := FSource;
      FSource      := FDestination;
      FDestination := aTmp;
    end;


{ ========
  TWireOverlay = class( TCustomControl)
  private
    FWires            : TObjectList;
    FCurvedLines      : Boolean;
    FWireThickness    : Integer;
    FdefaultWireColor : TColor;
    FBlockCount       : Integer;
  public
    property    Wire       [ anIndex: Integer]: TWire      read GetWire;
    property    Source     [ anIndex: Integer]: TConnector read GetSource;
    property    Destination[ anIndex: Integer]: TConnector read GetDestination;
  public
    property    CurvedLines      : Boolean read FCurvedLines      write SetCurvedLines    default True;
    property    WireThickness    : Integer read FWireThickness    write SetWireThickness  nodefault;
    property    defaultWireColor : TColor  read FdefaultWireColor write FdefaultWireColor default clBlue;
    property    BlockCount       : Integer read FBlockCount;
  published
    property    Left   stored False;
    property    Top    stored False;
    property    Width  stored False;
    property    Height stored False;
  private
}

    procedure   TKnobsWireOverlay.WMNCHitTest( var aMessage: TWMNCHitTest); // message WM_NCHITTESTq;
    begin
      if csDesigning in ComponentState
      then inherited
      else aMessage.Result := HTTRANSPARENT;
    end;

    function    TKnobsWireOverlay.GetWire( anIndex: Integer): TKnobsWire;
    begin
      Result := TKnobsWire( FWires[ anIndex]);
    end;

    function    TKnobsWireOverlay.GetSource( anIndex: Integer): TKnobsConnector;
    begin
      Result := Wire[ anIndex].FSource;
    end;

    function    TKnobsWireOverlay.GetDestination( anIndex: Integer): TKnobsConnector;
    begin
      Result := Wire[ anIndex].FDestination;
    end;

    procedure   TKnobsWireOverlay.SetCurvedLines( aValue: Boolean);
    begin
      if aValue <> FCurvedLines
      then begin
        FCurvedLines := aValue;
        InvalidateWires;
      end;
    end;

    procedure   TKnobsWireOverlay.SetWireThickNess( aValue: Integer);
    begin
      if aValue <> FWireThickness
      then begin
        FWireThickness := aValue;
        InvalidateWires;
      end;
    end;

//   protected

    procedure   TKnobsWireOverlay.Paint; // override;
    var
      i      : Integer;
      O      : TKnobsConnector;
    begin
      if FBlockCount = 0
      then begin
        with Canvas
        do begin
          Pen.Width := FWireThickness + 1;

          for i := 0 to WireCount - 1
          do begin
            with Wire[ i]
            do begin
              O := FSource.FindOutput;
              if not Assigned( O)
              then Pen.Color := CL_NO_OUTPUT
              else if FHighlight
              then Pen.Color := CL_HIGHLIGHT
              else Pen.Color := O.WireColor;
            end;
            PaintWire( Handle, Wire[ i]);
          end;
        end;
      end;
    end;

    procedure   TKnobsWireOverlay.PaintWire( aDC: HDC; aWire: TKnobsWire);
    var
      Points : TKnobsBezierPoints;
    begin
      with aWire
      do begin
        if FSource.HasParent and FDestination.HasParent and ( WireThickness > 0)
        then begin
          with FSource
          do Points[ 0] := ClientToScreen( CenterPoint);

          Points[ 0] := ScreenToClient( Points[ 0]);

          with FDestination
          do Points[ 3] := ClientToScreen( CenterPoint);

          Points[ 3] := ScreenToClient( Points[ 3]);

          if CurvedLines
          then begin
            CalcBezierCurve( Points, aWire.BezierParams);
            PolyBezier( aDC, Points, 4);
          end
          else begin
            with Points[ 0]
            do MoveToEx( aDC, X, Y, nil);

            with Points[ 3]
            do LineTo  ( aDC, X, Y     );
          end;
        end;
      end;
    end;

    function    TKnobsWireOverlay.FindWire( aSource, aDestination: TKnobsConnector): Integer;
    var
      i : Integer;
    begin
      Result := -1;
      for i := 0 to WireCount - 1
      do begin
        if
          (( Wire[ i].FSource = aSource     ) and ( Wire[ i].FDestination = aDestination)) or
          (( Wire[ i].FSource = aDestination) and ( Wire[ i].FDestination = aSource     ))
        then begin
          Result := i;
          Break;
        end;
      end;
    end;

    procedure   TKnobsWireOverlay.AddWire( aSource, aDestination: TKnobsConnector);
    var
      anIndex      : Integer;
      aWire        : TKnobsWire;
      aDistanceSrc : Integer;
      aDistanceDst : Integer;
    begin
      anIndex := FindWire( aSource, aDestination);

      if anIndex < 0
      then begin
        if ( aSource is TKnobsOutput) and ( aDestination is TKnobsOutput)
        then raise EKnobsShortCircuit.Create( 'output to output connection detected ... should not be there')
        else if ( aSource is TKnobsInput) and ( aDestination is TKnobsOutput)
        then aWire := TKnobsWire.Create( aDestination, aSource)
        else if ( aSource is TKnobsOutput) and ( aDestination is TKnobsInput)
        then aWire := TKnobsWire.Create( aSource, aDestination)
        else begin
          // For input to input connections the source should be closest to originating output ..
          // when there is no output the wire might still be added with the connectors in the
          // wrong order, this must be fixed before patch compilation with a call to
          // FixAllWires - we will not do that here as the order is irrelevant for the editor
          // itself. So just add the wire here in an arbitrary way.
          aDistanceSrc := aSource     .DistanceToOutput;
          aDistanceDst := aDestination.DistanceToOutput;

          if aDistanceSrc > aDistanceDst
          then aWire := TKnobsWire.Create( aDestination, aSource)
          else aWire := TKnobsWire.Create( aSource, aDestination);

        end;
        FWires.Add( aWire);
        InvalidateWires;
      end;
    end;

    procedure   TKnobsWireOverlay.DelWire( aSource, aDestination: TKnobsConnector);
    var
      anIndex : Integer;
    begin
      anIndex := FindWire( aSource, aDestination);

      if anIndex >= 0
      then begin
        FWires.Delete( anIndex);
        InvalidateWires;
      end;
    end;

    procedure   TKnobsWireOverlay.Connect( aSource, aDestination: TKnobsConnector);
    begin
      if Assigned( aSource)
      then aSource.ConnectTo( aDestination);
    end;

    procedure   TKnobsWireOverlay.HandleHighlight;
    var
      i : Integer;
    begin
      for i := 0 to WireCount - 1
      do begin
        with Wire[ i]
        do FHighlight := FSource.HighLight or FDestination.HighLight;
      end;

      Invalidate;
    end;

    procedure   TKnobsWireOverlay.HandleColorChange( aSource: TKnobsConnector; aColor: TColor);
    begin
      InvalidateWires;
    end;

    function    TKnobsWireOverlay.WireCount: Integer;
    begin
      if Assigned( FWires)
      then Result := FWires.Count
      else Result := 0;
    end;

    procedure   TKnobsWireOverlay.WiggleWires;
    var
      i : Integer;
    begin
      if CurvedLines and Assigned( FWires)
      then begin
        for i := 0 to WireCount - 1
        do Wire[ i].Wiggle;
        InvalidateWires;
      end;
    end;

//   public

    constructor TKnobsWireOverlay.Create( anOwner: TComponent); // override;
    begin
      inherited;

      FBlockCount        := 0;
      FWires             := TObjectList.Create;
      Align              := alClient;
      FCurvedLines       := True;
      FWireThickness     := 1;
      FdefaultWireColor  := clBlue;
    // DoubleBuffered     := True;
      FWires.OwnsObjects := True;
    end;

    destructor  TKnobsWireOverlay.Destroy; // override;
    begin
      FreeAndnil( FWires);

      inherited;
    end;

    procedure   TKnobsWireOverlay.InvalidateWires;

      function CreateRgn( aDC: HDC; aWire: TKnobsWire): HRGN;
      begin
        BeginPath ( aDC);
        PaintWire ( aDC, aWire);
        EndPath   ( aDC);
        WidenPath ( aDC);
        Result := PathToRegion( aDC);
      end;

    var
      DC       : HDC;
      i        : Integer;
      HRegion1 : HRGN;
      HRegion2 : HRGN;
      SavedPen : HPEN;
      Clocks   : Int64;
    begin
      if HasParent and Parent.HasParent and ( FBlockCount = 0)
      then begin
        Inc( FPaintCount);
        Clocks := GetCpuClockCycleCount;
     // BringToFront;  // See __1 remarks - needed to keep wires on top of modules, but not here it seems ...
        with Canvas
        do begin
          Pen.Width := FWireThickness;
          DC        := CreateCompatibleDC( Handle);
          SavedPen  := SelectObject( DC, Pen.Handle);
        end;

        try
          if ( WireCount > 0) and ( WireThickness > 0)
          then begin
            HRegion1 := CreateRgn( DC, Wire[ 0]);

            for i := 1 to WireCount - 1
            do begin
              HRegion2 := CreateRgn( DC, Wire[ i]);
              CombineRgn( HRegion2, HRegion2, HRegion1, RGN_OR);
              DeleteObject( HRegion1);
              HRegion1 := HRegion2;
            end;
          end
          else HRegion1 := CreateRectRgn( 0, 0, 1, 1);

          SetWindowRgn( Handle, HRegion1, True);
          BringToFront; // see other __1 remarks - it will paint behind .. otherwise
        finally
          SelectObject( DC, SavedPen);
          DeleteDC( DC);
        end;


        Invalidate; // And tell windows we want a repaint.

        FPaintClocks := GetCpuClockCycleCount - Clocks;
      end;
    end;

    function   TKnobsWireOverlay.Disconnect( aConnector: TKnobsConnector): Boolean;
    var
      i : Integer;
    begin
      Result := False;
      for i := WireCount - 1 downto 0
      do begin
        with Wire[ i]
        do begin
          if ( FSource = aConnector) or ( FDestination = aConnector)
          then begin
            FWires.Delete( i);
            Result := True;
          end;
        end;
      end;
    end;

    procedure   TKnobsWireOverlay.Dump( var aFile: TextFile; anInd: Integer);
    var
      i : Integer;
    begin
      for i := 0 to FWires.Count - 1
      do TKnobsWire( FWires[ i]).Dump( aFile, anInd);
    end;

    procedure   TKnobsWireOverlay.Log( const aMsg: string);
    begin
      if Parent is TKnobsWirePanel
      then TKnobsWirePanel( Parent).Log( Format( 'wire overlay %s( %s)', [ Name, aMsg]));
    end;

    procedure   TKnobsWireOverlay.BlockScreenUpdates;
    begin
      Inc( FBlockCount);
    end;

    procedure   TKnobsWireOverlay.UnBlockScreenUpdates;
    begin
      if FBlockCount > 0
      then begin
        Dec( FBlockCount);
        if FBlockCount = 0
        then InvalidateWires;
      end;
    end;

    procedure   TKnobsWireOverlay.FixAllWires;
    // Fix all wires that are input to input such that the Source connector is closest
    // to the originating output connector. When there is no output in the chain  just
    // do something arbitrary - which must then be fixed later once an output gets to
    // be connected. Must be called before a patch is translated into a synth patch.
    var
      i : Integer;
    begin
      for i := 0 to WireCount - 1
      do Wire[ i].FixWire;
    end;



{ =======
  TModuleList = class( TList)
  private
    FonLog       : TKnobsOnLog;
    FControlMode : TDistanceMode;
  public
    property    Module  [ anIndex: Integer]: TKnobsCustomModule read GetModule;                     default;
    property    Selected[ anIndex: Integer]: Boolean            read GetSelected  write SetSelected;
    property    OnLog                      : TKnobsOnLog        read FOnLog       write FOnLog;
    property    ControlMode                : TDistanceMode      read FControlMode write SetControlMode;
  private
}

    function    TKnobsModuleList.GetModule( anIndex: Integer): TKnobsCustomModule;
    begin
      Result := TKnobsCustomModule( Items[ anIndex]);
    end;

    function    TKnobsModuleList.GetSelected( anIndex: Integer): Boolean;
    begin
      Result := Module[ anIndex].Selected;
    end;

    procedure   TKnobsModuleList.SetSelected( anIndex: Integer; aValue: Boolean);
    begin
      Module[ anIndex].Selected := aValue;
    end;

//  public

    procedure   TKnobsModuleList.FreeModule( anIndex: Integer);
    var
      aModule : TKnobsCustomModule;
    begin
      aModule := Module[ anIndex];
      Delete( anIndex);
      aModule.Free;
    end;

    function    TKnobsModuleList.FreeModules: Boolean;
    var
      i : Integer;
    begin
      Result := False;
      for i := Count - 1 downto 0
      do begin
        FreeModule( i);
        Result := True;
      end;
    end;

    function    TKnobsModuleList.FreeSelectedModules: Boolean;
    var
      i : Integer;
    begin
      Result := False;
      for i := Count - 1 downto 0
      do begin
        if Selected[ i]
        then begin
          FreeModule( i);
          Result := True;
        end;
      end;
    end;

    procedure   TKnobsModuleList.SelectAll;
    var
      i : Integer;
    begin
      for i := Count - 1 downto 0
      do Selected[ i] := True;
    end;

    procedure   TKnobsModuleList.UnselectAll;
    var
      i : Integer;
    begin
      for i := Count - 1 downto 0
      do Selected[ i] := False;
    end;

    procedure   TKnobsModuleList.SelectUnique( aModule: TKnobsCustomModule);
    var
      i : Integer;
    begin
      for i := 0 to Count - 1
      do Module[ i].Selected := Module[ i] = aModule;
    end;

    procedure   TKnobsModuleList.InvertSelection;
    var
      i : Integer;
    begin
      for i := Count - 1 downto 0
      do Selected[ i] := not Selected[ i];
    end;

//  public

    //* Some helpers for selector population and component registration

    function    TKnobsModuleList.FindType( aModuleType: TKnobsModuleType): TKnobsCustomModule;
    //* Finds the first enabled module of the ModuleType specified
    var
      i : Integer;
    begin
      Result := nil;

      for i := 0 to Count - 1
      do begin
        if ( Module[ i].ModuleType = aModuleType) and ( Module[ i]).Enabled
        then begin
          Result := Module[ i];
          Break;
        end;
      end;
    end;

    function TabIndexSorter( Item1, Item2: Pointer): Integer;

      function CompareModules( m1, m2: TKnobsCustomModule): Integer;
      begin
        if m1.TabOrder > m2.TabOrder
        then Result := 1
        else if m1.TabOrder < m2.TabOrder
        then Result := -1
        else Result := 0;
      end;

    var
      m1 : TKnobsCustomModule;
      m2 : TKnobsCustomModule;
      t1 : TTabSheet;
      t2 : TTabSheet;
    begin
      m1 := TKnobsCustomModule( Item1);
      m2 := TKnobsCustomModule( Item2);
      t1 := nil;
      if Assigned( m1.Parent) and ( m1.Parent is TTabSheet)
      then t1 := TTabSheet( m1.Parent);
      t2 := nil;
      if Assigned( m2.Parent) and ( m2.Parent is TTabSheet)
      then t2 := TTabSheet( m2.Parent);
      if not Assigned( t1) or not Assigned( t2)
      then Result := CompareModules( m1, m2)
      else begin
        if t1.TabIndex > t2.TabIndex
        then Result := 1
        else if t1.TabIndex < t2.TabIndex
        then Result := -1
        else Result := CompareModules( m1, m2);
      end;
    end;

    procedure   TKnobsModuleList.PopulateSelector( aModuleSelector: TKnobsModuleSelector);
    //* Populate a module selector using all the modules contained in the list
    var
      i : Integer;
    begin
      Sort( TabIndexSorter);
      if Assigned( aModuleSelector)
      then begin
        for i := 0 to Count - 1
        do begin
          if Module[ i].Enabled
          then begin
            with Module[ i]
            do aModuleSelector.RegisterModuleType( PageName, ModuleType, Title);
          end;
        end;
      end;
    end;

    function    TKnobsModuleList.CreateModule( aWirePanel: TKnobsWirePanel; aModuleType: TKnobsModuleType; MustDrag: Boolean): TKnobsCustomModule;
    //* Factory method, create a clone from the specified module type when an
    //* enabled module of that type can be found in the list.
    //* This one is called from the module selector only, it should set
    //* the one and only module into drag mode. See __2 for other cases
    //* (also in KnobParser)
    //* See Clone method .. that one fixes it
    var
      aModule : TKnobsCustomModule;
    begin
      Result  := nil;
      aModule := FindType( aModuleType);

      if Assigned( aModule)
      then Result := aModule.Clone( aWirepanel, aWirePanel, Point( 0, 0), MustDrag);
    end;

    procedure   TKnobsModuleList.RegisterComponents( doRegister: Boolean);
    //* Register or unregister all module classes and all component's classes
    //* used on all modules with Delphi so that they can be used with the
    //* component streaming mechanism.
    var
      aList : TClassList;
      i     : Integer;
      j     : Integer;
      p     : Integer;
    begin
      aList := TClassList.Create;

      try
        // Build a list having each class only once
        for i := 0 to Count - 1
        do begin
          with Module[ i]
          do begin
            // Register the module class itself
            p := aList.IndexOf( ClassType);
            if p < 0
            then aList.Add( ClassType);

            // Register component classes for all components direcly parented by
            // the module.
            for j := 0 to ControlCount - 1
            do begin
              if Controls[ j] is TPersistent
              then begin
                p := aList.IndexOf( Controls[ j].ClassType);
                if p < 0
                then aList.Add( Controls[ j].ClassType);
              end;
            end;
          end;
        end;

        // Register the classes found
        with aList
        do begin
          for i := 0 to Count - 1
          do begin
            if doRegister
            then Registerclass  ( TPersistentClass( Items[ i]))
            else UnRegisterClass( TPersistentClass( Items[ i]));
          end;
        end;
      finally
        aList.Free;
      end;
    end;

    procedure   TKnobsModuleList.Dump( var aFile: TextFile; anInd: Integer);
    var
      i : Integer;
    begin
      WriteLnInd( afile, anInd, '.');
      for i := 0 to Count - 1
      do Module[ i].Dump( aFile, anInd);
      WriteLnInd( afile, anInd, '.');
    end;

    procedure   TKnobsModuleList.Log( const aMsg: string);
    begin
      if Assigned( FOnLog)
      then FOnLog( Self, Format( 'TKnobsModuleList( %s)', [ aMsg]));
    end;

    function    TKnobsModuleList.ModuleTypeName( aModuleType: Integer): string;
    begin
      Result := 'unknown module type';
    end;


{ ========
  THistoryStack = class( TStringList)
  private
    FMaxHistory : Integer;
  public
    property    MaxHistory : Integer read FMaxHistory write SetMaxCount;
  private
}

    procedure   TKnobsHistoryStack.SetMaxHistory( aValue: Integer);
    begin
      if aValue <> FMaxHistory
      then begin
        FMaxHistory := aValue;
        while Count > MaxHistory
        do Delete( 0);
      end;
    end;

//  protected

    procedure   TKnobsHistoryStack.PushString( const S: string);
    begin
      if ( Count = 0) or ( S <> Strings[ Count - 1])
      then begin
        Add( S);
        while Count > MaxHistory
        do Delete( 0);
      end;
    end;

    function    TKnobsHistoryStack.PopString: string;
    begin
      Result := '';
      if Count > 0
      then begin
        Result := Strings[ Count - 1];
        Delete( Count - 1);
      end;
    end;

//  public

    procedure   TKnobsHistoryStack.Push( const aWirePanel: TKnobsWirePanel);
    begin
      if Assigned( aWirePanel)
      then PushString( aWirePanel.PatchWriter.WriteString( aWirePanel, '', wmAll));
    end;

    procedure   TKnobsHistoryStack.Pop( const aWirePanel: TKnobsWirePanel);
    begin
      if Count > 0
      then aWirePanel.PatchReader.ReadString( PopString, aWirePanel, Point( 0, 0), rmReplace, '');
    end;



{ ========
  TUndoRedoHistory = class( TObject)
  private
    FOnHistoryChange : TOnHistoryChange;
    FMaxHistory      : Integer;
    FLockCount       : Integer;
    FUndos           : THistoryStack;
    FRedos           : THistoryStack;
  public
    property    UndoCount       : Integer          read GetUndoCount;
    property    RedoCount       : Integer          read GetRedoCount;
    property    MaxHistory      : Integer          read FMaxHistory  write SetMaxDepth;
  public
    property    OnHistoryChange : TOnHistoryChange read FOnHistoryChange write FOnHistoryChange;
  private
}

    function    TKnobsUndoRedoHistory.GetUndoCount: Integer;
    begin
      Result := FUndos.Count;
    end;

    function    TKnobsUndoRedoHistory.GetRedoCount: Integer;
    begin
      Result := FRedos.Count;
    end;

    procedure   TKnobsUndoRedoHistory.SetmaxHistory( aValue: Integer);
    begin
      if aValue <> FMaxHistory
      then begin
        FMaxHistory := aValue;
        FUndos.MaxHistory := aValue;
        FRedos.MaxHistory := aValue;
      end;
    end;

//  public

    constructor TKnobsUndoRedoHistory.Create;
    begin
      inherited Create;
      FMaxHistory       := 1;
      FUndos            := TKnobsHistoryStack.Create;
      FRedos            := TKnobsHistoryStack.Create;
      FUndos.MaxHistory := 1;
      FRedos.MaxHistory := 1;
    end;

    destructor  TKnobsUndoRedoHistory.Destroy; // override;
    begin
      FreeAndNil( FUndos);
      FreeAndNil( FRedos);
      inherited;
    end;

    procedure   TKnobsUndoRedoHistory.HistoryChanged;
    begin
      if Assigned( FOnHistoryChange)
      then FOnHistoryChange( Self, UndoCount, RedoCount);
    end;

    procedure   TKnobsUndoRedoHistory.SaveState( const aWirePanel: TKnobsWirePanel);
    begin
      if FLockCount = 0
      then begin
        FRedos.Clear;
        FUndos.Push( aWirePanel);
        HistoryChanged;
      end;
    end;

    procedure   TKnobsUndoRedoHistory.Undo( const aWirePanel: TKnobsWirePanel);
    begin
      if UndoCount > 0
      then begin
        FRedos.Push( aWirePanel);
        try
          Lock;                    // Prevent undo from changing history
          FUndos.Pop( aWirePanel);
        finally
          UnLock;
        end;
        HistoryChanged;
      end;
    end;

    procedure   TKnobsUndoRedoHistory.Redo( const aWirePanel: TKnobsWirePanel);
    begin
      if RedoCount > 0
      then begin
        FUndos.Push( aWirePanel);
        try
          Lock;                    // Prevent redo from changing history
          FRedos.Pop( aWirePanel);
        finally
          UnLock;
        end;
        HistoryChanged;
      end;
    end;

    procedure   TKnobsUndoRedoHistory.Lock;
    begin
      Inc( FLockCount);
    end;

    procedure   TKnobsUndoRedoHistory.UnLock;
    begin
      if FLockCount > 0
      then Dec( FLockCount);
    end;

    procedure   TKnobsUndoRedoHistory.Clear;
    begin
      FUndos.Clear;
      FRedos.Clear;
      HistoryChanged;
    end;


{ ========
  TWirePanel = class( TScrollingWinControl)
  private
    FOffsetLeft        : Integer;
    FOffsetTop         : Integer;
    FTitle             : string;
    FVersion           : Integer;
    FMouseDown         : Boolean;
    FWireOverlay       : TKnobsWireOverlay;
    FModules           : TKnobsModuleList;
    FFilename          : string;
    FSnapX             : Integer;
    FSnapY             : Integer;
    FSnapActive        : Boolean;
    FPatchReader       : IKnobsPatchReader;
    FPatchWriter       : IKnobsPatchWriter;
    FOnLog             : TKnobsOnLog;
    FOnHistoryChange   : TKnobsOnHistoryChange;
    FHistory           : TKnobsUndoRedoHistory;
    FOnValueChanged    : TKnobsOnValueChange;
    FOnDisplayChanged  : TKnobsOnDisplayChanged;
    FOnFileChanged     : TKnobsOnFileChanged;
    FOnPatchChanged    : TKnobsOnPatchChanged;
  public
    property    PatchReader  : IKnobsPatchReader   read FPatchReader   write FPatchReader;
    property    PatchWriter  : IKnobsPatchWriter   read FPatchWriter   write FPatchWriter;
  public
    property    Module     [ anIndex: Integer]: TKnobsCustomModule  read GetModule;
    property    Wire       [ anIndex: Integer]: TKnobsWire          read GetWire;
    property    Source     [ anIndex: Integer]: TKnobsConnector     read GetSource;
    property    Destination[ anIndex: Integer]: TKnobsConnector     read GetDestination;
  public
    property    OnHistoryChange  : TKnobsOnHistoryChange  read FOnHistoryChange    write FOnHistoryChange;
    property    OnLog            : TKnobsOnLog            read FOnLog              write FonLog;
    property    Filename         : string                 read FFilename           write FFilename;
    property    ModuleCount      : Integer                read GetModuleCount;
    property    WireCount        : Integer                read GetWireCount;
    property    BlockCount       : Integer                read GetBlockCount;
    property    PaintCount       : Integer                read GetPaintCount;
    property    PaintClocks      : Int64                  read GetPaintClocks;
    property    ScrollOffset     : TPoint                 read GetScrollOffset;
  published
    property    MaxHistory       : Integer                read GetMaxHistory       write SetMaxHistory       default 100;
    property    OnValueChanged   : TKnobsOnValueChange    read FOnValueChanged     write FOnValueChanged;
    property    OnDisplayChanged : TKnobsOnDisplayChanged read FOnDisplayChanged   write FOnDisplayChanged;
    property    OnFileChanged    : TKnobsOnFileChanged    read FOnFileChanged      write FOnFileChanged;
    property    OnPatchChanged   : TKnobsOnPatchChanged   read FOnPatchChanged     write FOnPatchChanged;
    property    OffsetLeft       : Integer                read FOffsetLeft         write SetOffsetLeft       default 4;
    property    OffsetTop        : Integer                read FOffsetTop          write SetOffsetTop        default 4;
    property    CurvedLines      : Boolean                read GetCurvedLines      write SetCurvedLines      default True;
    property    WireThickness    : Integer                read GetWireThickness    write SetWireThickness    default 1;
    property    defaultWireColor : TColor                 read GetdefaultWireColor write SetdefaultWireColor default clBlue;
    property    Title            : string                 read FTitle              write FTitle;
    property    Version          : Integer                read FVersion            write FVersion;
    property    SnapX            : Integer                read FSnapX              write FSnapX              default 250;
    property    SnapY            : Integer                read FSnapY              write FSnapY              default 15;
    property    SnapActive       : Boolean                read FSnapActive         write FSnapActive         default True;
    property    Align;
    property    Anchors;
    property    Constraints;
    property    ParentColor;
    property    ParentShowHint;
    property    PopupMenu;
    property    ShowHint;
    property    TabOrder;
    property    TabStop;
    property    Visible;
    property    Color;
    property    Top;
    property    Left;
    property    Width;
    property    Height;
    property    HorzScrollBar;
    property    VertScrollBar;
  protected
}

    procedure   TKnobsWirePanel.UMConnectionAdd( var aMessage: TKnobsUMConnectionAdd); // message UM_CONNECTIONADD;
    begin
      with aMessage
      do FWireOverlay.AddWire( Source, Destination);
    end;

    procedure   TKnobsWirePanel.UMConnectionRemove( var aMessage: TKnobUMConnectionRemove); // message UM_CONNECTIONREMOVE;
    begin
      with aMessage
      do FWireOverlay.DelWire( Source, Destination);
    end;

    procedure   TKnobsWirePanel.UMColorChange( var aMessage: TKnobsUMColorChange); // message UM_COLORCHANGE;
    begin
      with aMessage
      do FWireOverlay.HandleColorChange( Source, Color);
    end;

    procedure   TKnobsWirePanel.UMHighlight( var aMessage: TKnobsUMHighlight); // message UM_HIGHLIGHT;
    begin
      FWireOverlay.HandleHighlight;
    end;

//  private

    function    TKnobsWirePanel.GetModule( anIndex: Integer): TKnobsCustomModule;
    begin
      Result := FModules[ anIndex];
    end;

    function    TKnobsWirePanel.GetWire( anIndex: Integer): TKnobsWire;
    begin
      if Assigned( FWireOverlay)
      then Result := FWireOverlay.Wire[ anIndex]
      else Result := nil;
    end;

    function    TKnobsWirePanel.GetSource( anIndex: Integer): TKnobsConnector;
    begin
      if Assigned( FWireOverlay)
      then Result := FWireOverlay.Source[ anIndex]
      else Result := nil;
    end;

    function    TKnobsWirePanel.GetDestination( anIndex: Integer): TKnobsConnector;
    begin
      if Assigned( FWireOverlay)
      then Result := FWireOverlay.Destination[ anIndex]
      else Result := nil;
    end;

    procedure   TKnobsWirePanel.SetOffsetLeft( aValue: Integer);
    begin
      if aValue <> FOffsetLeft
      then begin
        FOffsetLeft := aValue;
        Resnap;
        Restack( DO_RESTACK, DO_REDRAW_WIRES);
      end;
    end;

    procedure   TKnobsWirePanel.SetOffsetTop( aValue: Integer);
    begin
      if aValue <> FOffsetTop
      then begin
        FOffsetTop := aValue;
        Resnap;
        Restack( DO_RESTACK, DO_REDRAW_WIRES);
      end;
    end;

    function    TKnobsWirePanel.GetCurvedLines: Boolean;
    begin
      Result := FWireOverlay.CurvedLines;
    end;

    procedure   TKnobsWirePanel.SetCurvedLines( aValue: Boolean);
    begin
      FWireOverlay.CurvedLines := aValue;
    end;

    function    TKnobsWirePanel.GetWireThickness: Integer;
    begin
      Result := FWireOverlay.WireThickness;
    end;

    procedure   TKnobsWirePanel.SetWireThickness( aValue: Integer);
    begin
      FWireOverlay.WireThickness := aValue;
    end;

    function    TKnobsWirePanel.GetdefaultWireColor: TColor;
    begin
      Result := FWireOverlay.defaultWireColor;
    end;

    procedure   TKnobsWirePanel.SetdefaultWireColor( aValue: TColor);
    begin
      FWireOverlay.defaultWireColor := aValue;
    end;

    function    TKnobsWirePanel.GetModuleCount: Integer;
    begin
      if Assigned( FModules)
      then Result := FModules.Count
      else Result := 0;
    end;

    function    TKnobsWirePanel.GetWireCount: Integer;
    begin
      if Assigned( FWireOverlay)
      then Result := FWireOverlay.WireCount
      else Result := 0;
    end;

    function    TKnobsWirePanel.GetMaxHistory: Integer;
    begin
      Result := FHistory.MaxHistory;
    end;

    procedure   TKnobsWirePanel.SetMaxHistory( aValue: Integer);
    begin
      FHistory.MaxHistory := aValue;
    end;

    function    TKnobsWirePanel.GetBlockCount: Integer;
    begin
      if Assigned( FWireOverlay)
      then Result := FWireOverlay.BlockCount
      else Result := -1;
    end;

    function    TKnobsWirePanel.GetPaintCount: Integer;
    begin
      if Assigned( FWireOverlay)
      then Result := FWireOverlay.PaintCount
      else Result := -1;
    end;

    function    TKnobsWirePanel.GetPaintClocks: Int64;
    begin
      if Assigned( FWireOverlay)
      then Result := FWireOverlay.PaintClocks
      else Result := -1;
    end;

    function    TKnobsWirePanel.GetScrollOffset: TPoint;
    begin
      Result := Point( HorzScrollBar.Position, VertScrollBar.Position);
    end;

    procedure   TKnobsWirePanel.SetControlMode( aValue: TDistanceMode);
    begin
      if aValue <> FControlMode
      then begin
        FControlMode := aValue;
        FixControlMode;
      end;
    end;

    procedure   TKnobsWirePanel.StrToConnection( const S: string);
    const
      Nums : set of AnsiChar = [ '0' .. '9'];
      Ascs : set of AnsiChar = [ '0' .. '9', 'A' .. 'Z', 'a' .. 'z', '_'];
    var
      p        : Integer;
      i        : Integer;
      SrcModNm : string;
      SrcConNm : string;
      DstModNm : string;
      DstConNm : string;
      SrcMod   : TKnobsCustomModule;
      SrcCon   : TKnobsConnector;
      DstMod   : TKnobsCustomModule;
      DstCon   : TKnobsConnector;
    begin

      p := 1;
      i := 1;

      if Length( S) > 0
      then begin

        while ( p <= Length( S)) and CharInSet( S[ p], Ascs)
        do Inc( p);
        SrcModNm := Copy( S, i, p - i);
        if ( p > Length( S)) or ( S[ p] <> '.')
        then Exit
        else Inc( p);

        i := p;
        while ( p <= Length( S)) and CharInSet( S[ p], Ascs)
        do Inc( p);
        SrcConNm := Copy( S, i, p - i);
        if ( p > Length( S)) or ( S[ p] <> ' ')
        then Exit
        else Inc( p);

        i := p;
        while ( p <= Length( S)) and CharInSet( S[ p], Ascs)
        do Inc( p);
        DstModNm := Copy( S, i, p - i);
        if ( p > Length( S)) or ( S[ p] <> '.')
        then Exit
        else Inc( p);

        i := p;
        while ( p <= Length( S)) and CharInSet( S[ p], Ascs)
        do Inc( p);
        DstConNm := Copy( S, i, p - i);

        try
          SrcMod :=        FindComponent( SrcModNm) as TKnobsCustomModule;
          SrcCon := SrcMod.FindComponent( SrcConNm) as TKnobsConnector;
          DstMod :=        FindComponent( DstModNm) as TKnobsCustomModule;
          DstCon := DstMod.FindComponent( DstConNm) as TKnobsConnector;
          Connect( SrcCon, DstCon);
        except
          on Exception
          do
            raise
              EKnobsCorruptPatch.
                Create(
                  'The patch is corrupted or incompatible:'^M +
                  'can''t make all the connections.'
                );
        end;

      end
      else
        raise
          EKnobsCorruptPatch.
            Create(
              'The patch is corrupted, incompatible or not a patch.'^M
            );
    end;

//  protected

    procedure   TKnobsWirePanel.HandleRightClick( const aSender: TControl);
    begin
      if Assigned( FOnShowPopup)
      then FOnShowPopup( Self, aSender);
    end;

    procedure   TKnobsWirePanel.CreateParams(var Params: TCreateParams); // override
    begin
      inherited CreateParams( Params);
      with Params.WindowClass
      do Style := Style and not ( CS_HREDRAW or CS_VREDRAW);
    end;

    procedure   TKnobsWirePanel.MouseDown( Button: TMouseButton; Shift: TShiftState; X, Y: Integer); // override;
    begin
      if FMouseDown
      then Exit;

      if Button = mbRight
      then HandleRightClick( Self)
      else if not FMouseDown and ( Button = mbLeft)
      then begin
        FMouseDown := True;
        GAnchor    := Point( X, Y);
        GDistance  := Point( 0, 0);
        DragStart( GAnchor  );
        DragMove ( GDistance);
      end;
    end;

    procedure   TKnobsWirePanel.MouseUp( Button: TMouseButton; Shift: TShiftState; X, Y: Integer); // override;
    begin
      if FMouseDown
      then begin
        DragMove( GDistance);
        GDistance.x := X - GAnchor.x;
        GDistance.y := Y - GAnchor.y;
        FMouseDown := False;
        DragEnd( GDistance);
      end;
    end;

    procedure   TKnobsWirePanel.MouseMove( Shift: TShiftState; X, Y: Integer); // override;
    begin
      if FMouseDown
      then begin
        DragMove( GDistance);
        GDistance.x := X - GAnchor.x;
        GDistance.y := Y - GAnchor.y;
        DragMove( GDistance);
      end
    end;

    procedure   TKnobsWirePanel.DragStart( aPoint: TPoint); // virtual;
    begin
      // Nothing
    end;

    procedure   TKnobsWirePanel.DragEnd( aPoint: TPoint); // virtual;
    begin
      SelectModulesInRect(
        Rect(
          GAnchor.x,
          GAnchor.y,
          GAnchor.x + aPoint.x,
          GAnchor.y + aPoint.y
        )
      );
    end;

    procedure   TKnobsWirePanel.DragMove( aPoint: TPoint); // virtual;
    var
      aDC   : HDC;
      aRect : TRect;
    begin
      aDC := GetDC( 0);

      with ClientOrigin
      do IntersectClipRect( aDC, X, Y, X + ClientWidth, Y + ClientHeight);

      try
        with aRect
        do begin
          Left   := Self.ClientOrigin.x + GAnchor.x;
          Top    := Self.ClientOrigin.y + GAnchor.y;
          Right  := Left + aPoint.x;
          Bottom := Top  + aPoint.y;
          if Left > Right
          then Exchange( Left, Right);
          if Top > Bottom
          then Exchange( Bottom, Top);
        end;

        DrawFocusRect( aDC, aRect);
      finally
        ReleaseDC( 0, aDC);
      end;
    end;

    procedure   TKnobsWirePanel.SelectModulesInRect( aRect: TRect);
    var
      i : Integer;
    begin
      with aRect
      do begin
        if Left > Right
        then Exchange( Left, Right);

        if Top > Bottom
        then Exchange( Bottom, Top);
      end;

      for i := 0 to ControlCount - 1
      do begin
        if Controls[ i] is TKnobsCustomModule
        then begin
          with TKnobsCustomModule( Controls[ i])
          do Selected := RectanglesOverlap( aRect, Rect( Left, Top, Left + Width, Top + Height));
        end;
      end;
    end;

    function    TKnobsWirePanel.IndexOfModule( aModule: TKnobsCustomModule): Integer;
    begin
      Result := FModules.IndexOf( aModule);
    end;

    procedure   TKnobsWirePanel.Notification( aComponent: TComponent; anOperation: TOperation); // override;
    var
      i : Integer;
    begin
      inherited;

      if aComponent is TKnobsCustomModule
      then begin
        if anOperation = opInsert
        then begin
          i := IndexOfModule( TKnobsCustomModule( aComponent));

          if i < 0
          then FModules.Add( aComponent);
        end
        else if anOperation = opRemove
        then begin
          i := IndexOfModule( TKnobsCustomModule( aComponent));

          if i >= 0
          then begin
            with FModules
            do begin
              Delete( i);
              Pack;
            end;
          end;
        end;
      End
    end;

    procedure   TKnobsWirePanel.Connect( aSource, aDestination: TKnobsConnector);
    begin
      if Assigned( FWireOverlay)
      then FWireOverlay.Connect( aSource, aDestination);
    end;

    procedure   TKnobsWirePanel.Restack( DoRestack, RedrawWires: Boolean);
    begin
      if ModuleCount > 0
      then Module[ 0].Restack( DoRestack, RedrawWires); // Which will restack all modules
    end;

    procedure   TKnobsWirePanel.Resnap;
    var
      i : Integer;
      p : TPoint;
    begin
      for i := 0 to ModuleCount  - 1
      do begin
        with Module[ i]
        do begin
          p    := Snap( Point( Left, Top));
          Left := P.X;
          Top  := P.Y;
        end;
      end;
    end;

    procedure   TKnobsWirePanel.FixNames;
    var
      i       : Integer;
      aModule : TKnobsCustomModule;
    begin
      for i := 0 to ModuleCount - 1
      do Module[ i].Name := Format( 'a%d', [ i]);

      for i := 0 to ModuleCount - 1
      do begin
        aModule := Module[ i];
        aModule.Name := Format( '%s%d', [ Copy( aModule.ClassName, 2, Length( aModule.ClassName)), i]);
      end;
    end;

    procedure   TKnobsWirePanel.SolveNamingConflictsWith( aWirePanel: TKnobsWirePanel);
    var
      i        : Integer;
      NextName : Integer;
      aModule  : TKnobsCustomModule;
    begin
      NextName := aWirePanel.ModuleCount;

      for i := 0 to ModuleCount - 1
      do begin
        aModule := Module[ i];

        if Assigned( aWirePanel.FindModule( aModule.Name))
        then begin
          aModule.Name := Format( '%s%d', [ Copy( aModule.ClassName, 2, Length( aModule.ClassName)), NextName]);
          Inc( NextName);
        end;
      end;
    end;

    function    TKnobsWirePanel.Snap( const aPoint: TPoint): TPoint;
    begin
      if SnapActive and ( SnapX <> 0) and ( SnapY <> 0)
      then begin
        Result.x := OffsetLeft + (( aPoint.x - OffsetLeft + SnapX div 2) div SnapX) * SnapX;
        Result.y := OffsetTop  + (( aPoint.y - OffsetTop  + SnapY div 2) div SnapY) * SnapY;
      end
      else Result := aPoint;
    end;

    procedure   TKnobsWirePanel.DoHistoryChanged( aSender: TObject; anUndoCount, aRedoCount: Integer);
    begin
      if Assigned( FOnHistoryChange)
      then FOnHistoryChange( aSender, anUndoCount, aRedoCount);
    end;

    procedure   TKnobsWirePanel.ValueChanged( aSender: TObject; const aPath, aControlType: string; aValue: TSignal; IsFinal: Boolean); // virtual;
    begin
      if Assigned( FOnValueChanged)
      then FOnValueChanged( aSender, aPath, aControlType, aValue, IsFinal);
    end;

    procedure   TKnobsWirePanel.DisplayChanged( aSender: TObject; const aPath, aValue: string);
    begin
      if Assigned( FOnDisplayChanged)
      then FOnDisplayChanged( aSender, aPath, aValue);
    end;

    procedure   TKnobsWirePanel.FileChanged( aSender: TObject; const aPath, aValue: string);
    begin
      if Assigned( FOnFileChanged)
      then FOnFileChanged( aSender, aPath, aValue);
    end;

    procedure   TKnobsWirePanel.PatchChanged;
    begin
      if Assigned( FOnPatchChanged)
      then FOnPatchChanged( Self);
    end;

    procedure   TKnobsWirePanel.ScrollPointInView( aPoint: TPoint); // overload;
    begin
      ScrollPointInView( aPoint.X, aPoint.Y);
    end;

    procedure   TKnobsWirePanel.ScrollPointInView( anX, anY: LongInt); // overload;
    const
      Clearance = 33;
    var
      aRect: TRect;
    begin
      aRect.TopLeft     := Point( anX - Clearance, anY - Clearance);
      aRect.BottomRight := Point( anX + Clearance, anY + Clearance);

      Dec( aRect.Left  , HorzScrollBar.Margin);
      Inc( aRect.Right , HorzScrollBar.Margin);
      Dec( aRect.Top   , VertScrollBar.Margin);
      Inc( aRect.Bottom, VertScrollBar.Margin);

      if aRect.Left < 0
      then begin
        with HorzScrollBar
        do Position := Position + aRect.Left
      end
      else if aRect.Right > ClientWidth
      then begin
        if aRect.Right - aRect.Left > ClientWidth
        then aRect.Right := aRect.Left + ClientWidth;
        with HorzScrollBar
        do Position := Position + aRect.Right - ClientWidth;
      end;
      if aRect.Top < 0
      then begin
        with VertScrollBar
        do Position := Position + aRect.Top;
      end
      else if aRect.Bottom > ClientHeight
      then begin
        if aRect.Bottom - aRect.Top > ClientHeight
        then aRect.Bottom := aRect.Top + ClientHeight;
        with VertScrollBar
        do Position := Position + aRect.Bottom - ClientHeight;
      end;
    end;

//  public

    constructor TKnobsWirePanel.Create( anOwner: TComponent); // override;
    begin
      inherited;

      ControlStyle :=
        [
          csAcceptsControls,
          csCaptureMouse,
          csClickEvents,
          csSetCaption,
          csDoubleClicks,
          csPannable
        ];

      OffsetLeft   := 4;
      OffsetTop    := 4;
      AutoScroll   := True;
      Width        := 185;
      Height       :=  41;
      SnapX        := 250;
      SnapY        :=  15;
      SnapActive   := True;
      FModules     := TKnobsModuleList     .Create;
      FWireOverlay := TKnobsWireOverlay    .Create( Self);
      FHistory     := TKnobsUndoRedoHistory.Create;
      MaxHistory   := 100;
      FControlMode := dmHorizontal;
      ControlMode  := dmCircular;

      FWireOverlay.Parent          := Self;
      FHistory    .OnHistoryChange := DoHistoryChanged;
    end;

    destructor  TKnobsWirePanel.Destroy; // override;
    begin
      inherited;                    // This will call into "Notifiction", which
      FreeAndNil( FModules);        // accesses FModules, so free FModules last.
      FreeAndNil( FHistory);        // and do this later even ...
    end;

    procedure   TKnobsWirePanel.FixAllSynthParams;
    var
      i : Integer;
    begin
      for i := 0 to ModuleCount - 1
      do Module[ i].FixAllSynthParams;
    end;

    procedure   TKnobsWirePanel.BeginStateChange;
    begin
      if Assigned( FPatchReader) and Assigned( FPatchWriter)
      then FHistory.SaveState( Self);
      FHistory.Lock;
    end;

    procedure   TKnobsWirePanel.EndStateChange( PatchWasChanged: Boolean);
    begin
      FHistory.UnLock;
      if ( FHistory.FLockCount = 0) and PatchWasChanged
      then PatchChanged;
    end;

    procedure   TKnobsWirePanel.Undo;
    begin
      if Assigned( FPatchReader) and Assigned( FPatchWriter)
      then begin
        FHistory.Undo( Self);
        PatchChanged;
        UnSelectAllModules;
      end
    end;

    procedure   TKnobsWirePanel.Redo;
    begin
      if Assigned( FPatchReader) and Assigned( FPatchWriter)
      then begin
        FHistory.Redo( Self);
        PatchChanged;
        UnSelectAllModules;
      end;
    end;

    procedure   TKnobsWirePanel.ClearUndoRedo;
    begin
      FHistory.Clear;
    end;

    procedure   TKnobsWirePanel.LockUndoRedo;
    begin
      FHistory.Lock;
    end;

    procedure   TKnobsWirePanel.UnLockUndoRedo;
    begin
      FHistory.UnLock;
    end;

    procedure   TKnobsWirePanel.Dump( const aFileName: string; DoAppend: Boolean); // overload;
    var
      aFile: TextFile;
    begin
      AssignFile( aFile, aFileName);
      if FileExists( aFileName)
      then begin
        if DoAppend
        then Append ( aFile)
        else Rewrite( aFile);
      end
      else Rewrite( aFile);
      try
        Dump( aFile);
      finally
        CloseFile( aFile);
      end;
    end;

    procedure   TKnobsWirePanel.Dump( var aFile: TextFile); // overload;
    begin
      WriteLn     ( aFile,         '---- wirepanel.dump -----------------------------------');
      WriteLn     ( aFile, Format( '---- %s', [ FormatDateTime( 'yyyy-mm-dd hh:nn:ss', Now)]));
      DumpModules ( aFile, 1);
      WriteLn     ( aFile);
      DumpWires   ( aFile, 1);;
      WriteLn     ( aFile,         '---- end wirepanel.dump -------------------------------');
      WriteLn     ( aFile);
    end;

    procedure   TKnobsWirePanel.DumpModules( var aFile: TextFile; anInd: Integer);
    begin
      WriteLnInd( aFile, anInd, 'modules');
      if Assigned( FModules)
      then FModules.Dump( aFile, anInd + 1);
      WriteLnInd( aFile, anInd, 'end modules');
    end;

    procedure   TKnobsWirePanel.DumpWires( var aFile: TextFile; anInd: Integer);
    begin
      WriteLnInd( aFile, anInd, 'wires');
      if Assigned( FWireOverlay)
      then FWireOverlay.Dump( aFile, anInd + 1);
      WriteLnInd( aFile, anInd, 'end wires');
    end;

    procedure   TKnobsWirePanel.Log( const aMsg: string);
    begin
      if Assigned( FOnLog)
      then FOnLog( Self, aMsg);
    end;

    function    TKnobsWirePanel.ModuleTypeName( aModuleType: Integer): string;
    begin
      Result := 'todo: callback needed ...';
    end;

    function    TKnobsWirePanel.FindModule( const aName: string): TKnobsCustomModule;
    var
      i : Integer;
    begin
      Result := nil;

      for i := 0 to ModuleCount - 1
      do begin
        if SameText( Module[ i].Name, aName)
        then begin
          Result := Module[ i];
          Break;
        end;
      end;
    end;

    function    TKnobsWirePanel.FindFirstSelectedModule: TKnobsCustomModule;
    var
      i : Integer;
    begin
      Result := nil;

      for i := 0 to ModuleCount - 1
      do begin
        if Module[ i].Selected
        then begin
          Result := Module[ i];
          Break;
        end;
      end;
    end;

    procedure   TKnobsWirePanel.ChangeWireNames( const aModuleName: string; anOldType, aNewType: TKnobsModuleType);
    begin
      // todo 2 -ctype change : ChangeWireNames (module retyped)
    end;

    procedure   TKnobsWirePanel.RetypeSelectedModules( aNewType: TKnobsModuleType);
    var
      i       : Integer;
      aModule : TKnobsCustomModule;
      Changed : Boolean;
    begin
      Changed := False;
      BeginStateChange;
      try
        for i := 0 to ModuleCount - 1
        do begin
          aModule := Module[ i];
          if aModule.Selected and ( aModule.FModuleType <> aNewType)
          then begin
            ChangeWireNames( aModule.Name, aModule.FModuleType, aNewType);
            aModule.ChangeType( aNewType); // todo 2 -ctype change : hmm ... is that right? see there ... we might have to do this heere instead
            Changed := True;
          end;
        end;
        if Changed
        then FixAllWires;
      finally
        EndStateChange( Changed);
      end;
    end;

//  public

    procedure   TKnobsWirePanel.InvalidateWires;
    begin
      FWireOverlay.InvalidateWires;
    end;

    procedure   TKnobsWirePanel.Disconnect( aConnector: TKnobsConnector);
    var
      Changed : Boolean;
    begin
      if Assigned( FWireOverlay)
      then begin
        Changed := False;
        BeginStateChange;
        try
          Changed := FWireOverlay.Disconnect( aConnector);
        finally
          EndStateChange( Changed)
        end;
      end;
    end;

    procedure   TKnobsWirePanel.FreeModules;
    var
      Changed : Boolean;
    begin
      BeginStateChange;
      BlockScreenUpdates;
      Changed := False;
      try
        Changed := FModules.FreeModules;
      finally
        UnBlockScreenUpdates;
        EndStateChange( Changed);
      end;
    end;

    function    TKnobsWirePanel.HasEditPopup: Boolean;
    begin
      Result :=
        ( Assigned( GDisplayEditor) and GDisplayEditor.Visible) or
        ( Assigned( GLabelEditor  ) and GLabelEditor  .Visible)
    end;

    procedure   TKnobsWirePanel.FreeSelectedModules;
    var
      Changed : Boolean;
    begin
      if not HasEditPopup
      then begin
        BeginStateChange;
        BlockScreenUpdates;
        Changed := False;
        try
          Changed := FModules.FreeSelectedModules;
        finally
          UnBlockScreenUpdates;
          EndStateChange( Changed);
        end;
      end;
    end;

    procedure   TKnobsWirePanel.SelectAllModules;
    begin
      FModules.SelectAll;
    end;

    procedure   TKnobsWirePanel.UnSelectAllModules;
    begin
      FModules.UnSelectAll;
    end;

    procedure   TKnobsWirePanel.SelectUnique( aModule: TKnobsCustomModule);
    begin
      FModules.SelectUnique( aModule);
    end;

    procedure   TKnobsWirePanel.InvertModuleSelection;
    begin
      FModules.InvertSelection;
    end;

    procedure   TKnobsWirePanel.AddModules( aSrc: TKnobsWirePanel; aDistance: TPoint; DoRestack, RedrawWires, MustDrag: Boolean); // override;
    //* This one is called only from the KnobParser either when a patch files is
    //* loaded or when a wirepanel was pasted from the clipboard; when pasted
    //* from the clipboard the newly pasted modules should be set into drag
    //* mode, for other cases see __2 (also in KnobParser although that one
    //* will end up here and nothing special is done there except for setting
    //* the Dragging parameter to true when the Addition was the result of a
    //* paste from the clip board).
    //* Correction : also after a Ctrl+Drag operation we end up here ...
    //* in that case dragging will be false, which is the correct way to handle
    //* this ... luck ... :shock:
    //* See Clone method - that one fixes it for single module insertion
    var
      i : Integer;
      M : TKnobsCustomModule;
    begin
      if not HasEditPopup
      then begin
        BeginStateChange;          // Blocks undo/redo updates
        BlockScreenUpdates;        // Blocks screen paints
        DisableAlign;
        try
          UnSelectAllModules;
          FixNames;

          with aSrc
          do begin
            FixNames;
            SolveNamingConflictsWith( Self);

            // When dragging the following code should not be performed as this
            // represents a final insertion into the wirepanel which will move
            // any existing modules around ... this will be tricky .... __2
            // we could maybe treat this by inserting the modules a a negative
            // offset (which is not normally possible) and setting an approriate
            // distance instead of the one passed as an argument to us.
            for i := 0 to ModuleCount - 1
            do begin
              M := Module[ i];
              M.
                Clone(
                  Self,
                  Self,
                  Self.Snap(
                    AddPoints(
                      Point(
                        M.Left,
                        M.Top
                      ),
                      aDistance
                    )
                  ),
                  MustDrag
                ).
                  Restack(
                    DoRestack,
                    RedrawWires
                  );
              end;
          end;

          with aSrc
          do begin
            for i := 0 to WireCount - 1
            do begin
              Self.StrToConnection(
                Format
                (
                  '%s %s',
                  [
                    ConnectorName( Source     [ i]),
                    ConnectorName( Destination[ i])
                  ]
                )
              );
            end;
          end;
        finally
          FixControlMode;
          UnBlockScreenUpdates;
          EndStateChange( True);
          EnableAlign;
        end;
      end;
    end;

    procedure   TKnobsWirePanel.AddWire( const aSrc, aDst: string);
    begin
      BeginStateChange;
      try
        StrToConnection( Format( '%s %s', [ aSrc, aDst]));
      finally
        EndStateChange( True);
      end;
    end;

    procedure   TKnobsWirePanel.WiggleWires;
    begin
      FWireOverlay.WiggleWires;
    end;

    procedure   TKnobsWirePanel.FixAllWires;
    begin
      if Assigned( FWireOverlay)
      then FWireOverlay.FixAllWires;
    end;

    procedure   TKnobsWirePanel.FixControlMode;
    var
      i : Integer;
    begin
      for i := 0 to ModuleCount - 1
      do Module[ i].ControlMode := FControlMode;
    end;

    procedure   TKnobsWirePanel.BlockScreenUpdates;
    begin
      FWireOverlay.BlockScreenUpdates;
    end;

    procedure   TKnobsWirePanel.UnblockScreenUpdates;
    begin
      FWireOverlay.UnBlockScreenUpdates;
    end;

    procedure   TKnobsWirePanel.Search( const aValue: string);
    var
      i : Integer;
      M : TKnobsCustomModule;
      U : string;
      N : string;
    begin
      U := UpperCase( aValue);
      for i := 0 to ModuleCount - 1
      do begin
        M := Module[ i];
        N := UpperCase( M.Title);
        M.Selected := ( Pos( U, N) > 0) and ( Length( U) > 0);
      end;
    end;

//  public

    procedure   TKnobsWirePanel.ConnectorDelete( const aConnector: TKnobsConnector);
    var
      anOutput: TKnobsConnector;
    begin
      // Disconnects every connector on this tree
      if Assigned( aConnector)
      then begin
        anOutput := aConnector.FindOutput;
        if Assigned( anOutput)
        then anOutput  .DisConnectDownStreamRecursive
        else aConnector.DisConnectDownStreamRecursive;
      end;
    end;

    procedure   TKnobsWirePanel.ConnectorBreak( const aConnector: TKnobsConnector);
    begin
      // Should disconnect the downstream links, currently breaks the upstream one too
      if Assigned( aConnector)
      then aConnector.DisConnectDownStream;
    end;

    procedure   TKnobsWirePanel.ConnectorDisconnect( const aConnector: TKnobsConnector);
    begin
      // Takes this connector out of the connection tree
      if Assigned( aConnector)
      then aConnector.DisConnectAll;
    end;


{ ========
  TCustomModule = class( TCustomPanel)
  private
    FAnchor           : TPoint;
    FDistance         : TPoint;
    FTresholdPast     : Boolean;
    FFlags            : TKnobsModuleFlags;
    FTitleLabel       : TKnobsEditLabel;
    FTitle            : string;
    FComment          : string;
    FModuleType       : TKnobsModuleType;
    FPageName         : string;
    FBitmap           : TBitmap;
    FOnValueChanged   : TKnobsOnValueChange;    // Called with the new value
    FOnDisplayChanged : TKnobsOnDisplayChanged;
    FOnFileChanged    : TKnobsOnFileChanged;
    FOnLog            : TKnobsOnLog;
  public
    property    Selected         : Boolean                read GetSelected       write SetSelected;
    property    Capturing        : Boolean                read GetCapturing      write SetCapturing;
    property    Title            : string                 read FTitle            write SetTitle;
    property    Comment          : string                 read FComment          write FComment;
    property    PageName         : string                 read FPageName         write FPageName;
    property    ModuleType       : TKnobsModuleType       read FModuleType       write SetModuleType;
    property    OnLog            : TKnobsOnLog            read FOnLog            write FOnLog;
  published
    property    TitleLabel       : TKnobsEditLabel        read FTitleLabel       write SetTitleLabel;
    property    OnValueChanged   : TKnobsOnValueChange    read FOnValueChanged   write FOnValueChanged;
    property    OnDisplayChanged : TKnobsOnDisplayChanged read FOnDisplayChanged write FOnDisplayChanged;
    property    OnFileChanged    : TKnobsOnFileChanged    read FOnFileChanged    write FOnFileChanged;
  private
}

    procedure   TKnobsCustomModule.IncludeFlags( aFlags: TKnobsModuleFlags);
    begin
      FFlags := FFlags + aFlags;
    end;

    procedure   TKnobsCustomModule.ExcludeFlags( aFlags: TKnobsModuleFlags);
    begin
      FFlags := FFlags - aFlags;
    end;

    function    TKnobsCustomModule.HasFlags( aFlags: TKnobsModuleFlags): Boolean;
    begin
      Result := aFlags <= FFlags;
    end;

//  private

    function    TKnobsCustomModule.GetSelected: Boolean;
    begin
      Result := HasFlags( [ mfSelected]);
    end;

    procedure   TKnobsCustomModule.SetSelected( aValue: Boolean);
    const
      Colors : array[ Boolean] of TColor = ( clSilver, clWhite);
    begin
      if aValue <> Selected
      then begin
        if aValue
        then IncludeFlags( [ mfSelected])
        else ExcludeFlags( [ mfSelected]);

        if Assigned( FTitleLabel)
        then begin
          with FTitleLabel
          do begin
            Color       := Colors[ Selected];
            Transparent := not Selected;
          end;
        end;
      end;
    end;

    function    TKnobsCustomModule.GetCapturing: Boolean;
    begin
      Result := HasFlags( [ mfCapturing]);
    end;

    procedure   TKnobsCustomModule.SetCapturing( aValue: Boolean);
    begin
      if aValue <> Capturing
      then begin
        if aValue
        then IncludeFlags( [ mfCapturing])
        else ExcludeFlags( [ mfCapturing]);
      end;
    end;

//  private

    procedure   TKnobsCustomModule.SetModuleType( aValue : TKnobsModuleType);
    begin
      if aValue <> FModuleType
      then begin
        FModuleType := aValue;
      end;
    end;

    procedure   TKnobsCustomModule.SetTitle( const aValue: string);
    begin
      if aValue <> FTitle
      then begin
        FTitle := aValue;
        if Assigned( FTitleLabel)
        then FTitleLabel.Text := FTitle;
      end;
    end;

    procedure   TKnobsCustomModule.SetTitleLabel( aValue: TKnobsEditLabel);
    begin
      FTitleLabel := aValue;

      if Assigned( FTitleLabel) and not ( ComponentState * [ csDestroying, csLoading] = [])
      then FTitleLabel.Caption := Title;
    end;

    procedure   TKnobsCustomModule.SetControlMode( aValue: TDistanceMode);
    var
      i : Integer;
    begin
      if aValue <> FControlMode
      then begin
        FControlMode := aValue;
        for i := 0 to ControlCount - 1
        do begin
          if Controls[ i] is TKnobsKnob
          then TKnobsKnob( Controls[ i]).ControlMode := FControlMode;
        end;
      end;
    end;

//  protected

    procedure   TKnobsCustomModule.WMEraseBackground( var aMsg: TWMEraseBkgnd); // message WM_ERASEBKGND;
    begin
      aMsg.Result := -1;
    end;

    procedure   TKnobsCustomModule.MouseDown( Button: TMouseButton; Shift: TShiftState; X, Y: Integer); // override;
    var
      aWP: TKnobsWirePanel;
    begin
      inherited;

      if Capturing
      then Exit;

      if Button = mbLeft
      then begin
        HandleSelection( ssCtrl in Shift);
        SetCapture( Handle);
        Capturing     := True;
        FTresholdPast := False;
        BeginMove( Point( X, Y));
      end
      else if Button = mbRight
      then begin
        aWP := FindWirePanel;
        if Assigned( aWP)
        then aWP.HandleRightClick( Self);
      end;

      SendToBack; // See __1 remarks  - needed to keep wires on top of modules
    end;

    procedure   TKnobsCustomModule.MouseMove( Shift: TShiftState; X, Y: Integer); // override;
    begin
      inherited;

      if  Capturing
      then begin
        if FTresholdPast
        then MoveSelected( FDistance);

        FDistance := SubPoints( Point( X, Y), FAnchor);

        if not FTresholdPast
        then FTresholdPast := Distance( FDistance) > 8;

        if FTresholdPast
        then begin
          if ssCtrl in Shift
          then Selected := True;

          MoveSelected( FDistance);
        end;
      end;
    end;

    procedure   TKnobsCustomModule.MouseUp( Button: TMouseButton; Shift: TShiftState; X, Y: Integer); // override;
    begin
      inherited;

      if Capturing
      then begin
        ReleaseCapture;
        ReleasePeerCapture;

        if FTresholdPast
        then begin
          if HasParent
          then Parent.DisableAlign;

          try
            FinalMoveSelected( FDistance, ssCtrl in Shift)
          finally
            if HasParent
            then Parent.EnableAlign;
          end;
        end;

        SendToBack;  // See __1 remarks  - needed to keep wires on top of modules
      end;
    end;

//  protected

    procedure   TKnobsCustomModule.ValueChanged( aSender: TObject; const aPath, aControlType: string; aValue: TSignal; IsFinal: Boolean); // virtual;
    var
      aWp: TKnobsWirePanel;
    begin
      if Assigned( FOnValueChanged)
      then FOnValueChanged( aSender, aPath, aControlType, aValue, IsFinal)
      else begin
        aWp := FindWirePanel;
        if Assigned( aWp)
        then aWp.ValueChanged( aSender, MakePath( [ aWp.Name, aPath]), aControlType, aValue, IsFinal);
      end;
    end;

    procedure   TKnobsCustomModule.DisplayChanged( aSender: TKnobsDisplay; const aPath, aValue: string);
    var
      aWp: TKnobsWirePanel;
    begin
      if Assigned( FOnDisplayChanged)
      then FOnDisplayChanged( aSender, aPath, aValue)
      else begin
        aWp := FindWirePanel;
        if Assigned( aWp)
        then aWp.DisplayChanged( aSender, MakePath( [ aWp.Name, aPath]), aValue);
      end;
    end;

    procedure   TKnobsCustomModule.FileChanged( aSender: TKnobsFileSelector; const aPath, aValue: string);
    var
      aWp: TKnobsWirePanel;
    begin
      if Assigned( FOnFileChanged)
      then FOnFileChanged( aSender, aPath, aValue)
      else begin
        aWp := FindWirePanel;
        if Assigned( aWp)
        then aWp.FileChanged( aSender, MakePath( [ aWp.Name, aPath]), aValue);
      end;
    end;

    function    TKnobsCustomModule.FindWirePanel: TKnobsWirePanel;
    var
      aParent : TWinControl;
    begin
      Result  := nil;
      aParent := Parent;

      while Assigned( aParent) and not ( aParent is TKnobsWirePanel)
      do aParent :=  aParent.Parent;

      if aParent is TKnobsWirePanel
      then Result := TKnobsWirePanel( aParent);
    end;

    function    TKnobsCustomModule.Snap( const aPoint: TPoint): TPoint;
    var
      aWp : TKnobsWirePanel;
    begin
      aWp := FindWirePanel;

      if Assigned( aWp)
      then Result := aWp.Snap( aPoint)
      else Result := aPoint;
    end;

    procedure   TKnobsCustomModule.Paint; // override;
    begin
      inherited;

      Canvas.StretchDraw( Rect( 1, 1, Width - 2, Height - 2), FBitmap);
    end;

    procedure   TKnobsCustomModule.Loaded; // override;
    var
      p : TPoint;
    begin
      inherited;

      p := Snap( Point( Left, Top));
      Left := P.X;
      Top  := P.Y;
    end;

    procedure   TKnobsCustomModule.AssignBitmap; // virtual;
    begin
      FBitmap := bmSkin;
    end;

    procedure   TKnobsCustomModule.ScrollSelfInView;
    var
      aWp : TKnobsWirePanel;
    begin
      aWp := FindWirePanel;

      if Assigned( aWp) and ( aWp.BlockCount = 0)
      then aWp.ScrollInView( Self);
    end;

    procedure   TKnobsCustomModule.HandleSelection( CtrlKeyActive: Boolean);
    begin
      if CtrlKeyActive
      then Selected := not Selected
      else begin
        if not Selected
        then SelectUnique;
      end;

      BringToFront; // See __1 remarks around
    end;

    procedure   TKnobsCustomModule.BeginMove( anAnchor: TPoint);
    begin
      FAnchor   := anAnchor;
      FDistance := Point( 0, 0);
      MoveSelected( FDistance);
    end;

    procedure   TKnobsCustomModule.Move( aDistance: TPoint);
    var
      aWp   : TKnobsWirePanel;
      aDC   : HDC;
      aRect : TRect;
      aRgn  : HRGN;
    begin
      aWp := FindWirePanel;
      if Assigned( aWp)
      then begin
        aRect.Left   := Left + aDistance.X + 1;
        aRect.Top    := Top  + aDistance.Y + 1;
        aRect.Right  := aRect.Left + Width  - 2;
        aRect.Bottom := aRect.Top  + Height - 2;

        aDC := GetDCEx( aWp.Handle, 0, DCX_PARENTCLIP);
        try
          with aWp.ClientRect
          do aRgn := CreateRectRgn( Left, Top, Right, Bottom);
          SelectClipRgn( aDC, aRgn);
          DeleteObject( aRgn);
          DrawFocusRect( aDC, aRect);
        finally
          ReleaseDC( Parent.Handle, aDC);
        end;
      end;
    end;

    procedure   TKnobsCustomModule.MoveSelected( aDistance: TPoint);
    var
      i     : Integer;
      aWp   : TKnobsWirePanel;
      aPt   : TPoint;
      aRect : TRect;
    begin
      aWp   := FindWirePanel;
      aPt   := aDistance;
      aRect := Rect( Left, Top, Left + Width, Top + Height);
      aPt.Offset( aRect.CenterPoint);

      if Assigned( aWp)
      then begin
        with aWp
        do begin
          aWp.ScrollPointInView( aWp.ScreenToClient( ClientToScreen( aPt)));
          for i := 0 to ModuleCount - 1
          do begin
            with Module[ i]
            do begin
              if Selected
              then Move( aDistance);
            end;
          end;
        end;
      end;
    end;

    procedure   TKnobsCustomModule.FinalMoveSelected( aDistance: TPoint; CtrlKeyActive: Boolean);
    //* When CtrlKeyActive is true this is a clone operation and the set of
    //* newly created modules should be set into drag mode. See __2 for other
    //* cases (also in KnobParser). This will end up in TWirePanel.AddModules
    //* when CtrlKeyActive is true, but as we set the filename to '' and not
    //* to '<clip>' it will not go into drag mode (which was just finished
    //* for the ctrl+drag mode ...).
    var
      P   : TPoint;
      i   : Integer;
      aWp : TKnobsWirePanel;
    begin
      aWp := FindWirePanel;

      if Assigned( aWp)
      then begin
        MoveSelected( FDistance);

        with aWp
        do begin
          if CtrlKeyActive // When so this was a copy action, a move otherwise
          then PatchReader.ReadString( PatchWriter.WriteString( aWp, '', wmSelected), aWp, aDistance, rmAppend)
          else begin
            aWp.BeginStateChange;
            aWp.BlockScreenUpdates;
            DisableAlign;
            try
              for i := 0 to ModuleCount - 1
              do begin
                with Module[ i]
                do begin
                  if Selected
                  then begin
                    P := Snap( Point( Left, Top) + ScrollOffset + aDistance);
                    if P.X < OffsetLeft then P.X := OffsetLeft;
                    if P.Y < OffsetTop  then P.Y := OffsetTop;
                    P := P - ScrollOffset;
                    SetBounds( P.X, P.Y, Width, Height);
                  end;
                end;
              end;

              // todo : in another place I had to ecchange the following two lines : see ***1
              // swapped now ... after the swap the restack comes first, then then the name fixing.
              // this was a move, not a module insertion, and so the patch did not change (sound wise)
              // so the mapping between compiled modules and editor module should not be changed
              // ergo, FixNames should not be called on the editor patch here.

              ReStack( DO_RESTACK, NO_REDRAW_WIRES);
              // FixNames;

              // end todo.

            finally
              aWp.UnBlockScreenUpdates; // This will invalidate wires, when the lock count gets zero
              aWp.EndStateChange( False);
              EnableAlign;
            end;
          end;
        end;
        ScrollSelfInView;
      end;
    end;

    procedure   TKnobsCustomModule.ReleasePeerCapture;
    var
      i   : Integer;
      aWp : TKnobsWirePanel;
    begin
      aWp := FindWirePanel;
      if Assigned( aWp)
      then begin
        with aWp
        do begin
          for i := 0 to ModuleCount - 1
          do Module[ i].Capturing := False;
        end;
      end;
    end;

    procedure   TKnobsCustomModule.Notification( aComponent: TComponent; anOperation: TOperation); // override;
    var
      aWp : TKnobsWirePanel;
    begin
      inherited;

      if ( anOperation = opRemove) and ( aComponent is TKnobsConnector)
      then begin
        aWp := FindWirePanel;

        if Assigned( aWp)
        then aWp.Disconnect( TKnobsConnector( aComponent));
      end;
    end;

    procedure   TKnobsCustomModule.FixOwnership( aRoot, aControl: TWinControl);
    //
    // Recursively make all controls parented by 'aControl' to be owned by
    // 'aRoot', this in order to allow 'aRoot' to write such controls on
    // a stream easily.
    //
    var
      i : Integer;
    begin
      if Assigned( aControl) and Assigned( aRoot)
      then begin
        with aControl
        do begin
          for i := 0 to ControlCount - 1
          do begin
            with Controls[ i]
            do begin
              if ( Owner <> aRoot) and ( Parent = aRoot)
              then begin
                Owner.RemoveComponent( Controls[ i]);
                aRoot.InsertComponent( Controls[ i]);

                if Controls[ i] is TWinControl
                then FixOwnerShip( aRoot, TWinControl( Controls[ i]));
              end;
            end;
          end;
        end;
      end;
    end;

    procedure   TKnobsCustomModule.FixNames;
    var
      aWp : TKnobsWirePanel;
    begin
      aWp := FindWirePanel;
      if Assigned( aWp)
      then aWp.FixNames;
    end;

    function    TKnobsCustomModule.FindValuedControl( const aName: string): TKnobsValuedControl;
    var
      i : Integer;
    begin
      Result := nil;

      for i := 0 to ComponentCount - 1
      do begin
        if ( Components[ i] is TKnobsValuedControl) and SameText( aName, Components[ i].Name)
        then begin
          Result := TKnobsValuedControl( Components[ i]);
          break;
        end;
      end;
    end;

    function    TKnobsCustomModule.FindDisplay( const aName: string): TKnobsDisplay;
    var
      i : Integer;
    begin
      Result := nil;

      for i := 0 to ComponentCount - 1
      do begin
        if ( Components[ i] is TKnobsDisplay) and SameText( aName, Components[ i].Name)
        then begin
          Result := TKnobsDisplay( Components[ i]);
          break;
        end;
      end;
    end;

    function    TKnobsCustomModule.FindFileSelector( const aName: string): TKnobsFileSelector;
    var
      i : Integer;
    begin
      Result := nil;

      for i := 0 to ComponentCount - 1
      do begin
        if ( Components[ i] is TKnobsFileSelector) and SameText( aName, Components[ i].Name)
        then begin
          Result := TKnobsFileSelector( Components[ i]);
          break;
        end;
      end;
    end;

//  public

    constructor TKnobsCustomModule.Create( anOwner: TComponent); // override;
    begin
      inherited;

      ControlStyle :=
        [
          csAcceptsControls,
          csCaptureMouse,
          csClickEvents,
          csDoubleClicks,
          csOpaque
        ];

      AssignBitmap;
      FBitmap.Transparent := False;

      Width          := 250;
      Height         :=  60;
      Name           := '';
      FTitle         := '';
      DoubleBuffered := True;
    end;

    destructor  TKnobsCustomModule.Destroy; // override;
    begin
      Disconnect;
      inherited;
    end;

    procedure   TKnobsCustomModule.FixAllSynthParams;
    var
      i : Integer;
    begin
      for i := 0 to ControlCount - 1
      do begin
        if Controls[ i] is TKnobsValuedControl
        then TKnobsValuedControl( Controls[ i]).ValueChanged( True)
        else if ( Controls[ i] is TKnobsDisplay) and TKnobsDisplay( Controls[ i]).HasEditor
        then begin
          with TKnobsDisplay( Controls[ i])
          do DisplayChanged( Caption, NO_NOTIFY);
        end
        else if ( Controls[ i] is TKnobsFileSelector)
        then begin
          with TKnobsFileSelector( Controls[ i])
          do FileChanged( WaveName, NO_NOTIFY);
        end;
      end;
    end;

    function    TKnobsCustomModule.Clone( anOwner: TComponent; aParent: TWinControl; anOffset: TPoint; MustDrag: Boolean): TKnobsCustomModule;
    //
    // Make all children parented by self to be owned by self, so they will be
    // streamed out. Then write self on a stream and read it back into a new
    // TCustomModule. Finally make the clone owned and parented as indicated in
    // the parameters passed, then position it relative to it's parent's (top, left).
    //
    var
      S : TStream;
      i : Integer;
    begin
      FixOwnerShip( Self, Self);
      S := TMemoryStream.Create;

      try
        S.WriteComponent( Self);

        S.Seek( 0, 0);
        Result := TKnobsCustomModule( S.ReadComponent( nil));

        // ReadComponent fux up zero values in TValuedControls ... fix 'm manually then ...

        for i := 0 to ControlCount - 1
        do begin
          if Controls[ i] is TKnobsValuedControl
          then ( Result.Controls[ i] as TKnobsValuedControl).KnobPosition := TKnobsValuedControl( Controls[ i]).KnobPosition;
        end;

        if Assigned( anOwner)
        then anOwner.InsertComponent( Result);

        with Result
        do begin
          if MustDrag
          then begin
            Left     := anOffset.X;
            Top      := anOffset.Y - 1000;
          end
          else begin
            Left     := anOffset.X;
            Top      := anOffset.Y;
          end;
          Parent   := aParent;
          Selected := True;
        end;

        with Result
        do begin
          if MustDrag and HasParent and Parent.HasParent
          then begin
            SetCapture( Handle);
            Capturing := True;
          end;
          SelectUnique;
        end;
      finally
        S.Free;
      end;
    end;

    procedure   TKnobsCustomModule.SetKnobValue( const aType, aName: string; aValue: Integer);
    var
      aValuedControl : TKnobsValuedControl;
    begin
      if
        SameText( aType, 'TKnobsSelector'     ) or
        SameText( aType, 'TKnobsKnob'         ) or
        SameText( aType, 'TKnobsSmallKnob'    ) or
        SameText( aType, 'TKnobsNoKnob'       ) or
        SameText( aType, 'TKnobsValuedButton' ) or
        SameText( aType, 'TKnobsValuedButtons')
      then begin
        aValuedControl := FindValuedControl( aName);

        if Assigned( aValuedControl)
        then aValuedControl.KnobPosition := aValue;
      end;
    end;

    procedure   TKnobsCustomModule.SetDetailValue( const aType, aName: string; const aValue: string);
    var
      aDisplay      : TKnobsDisplay;
      aFileSelector : TKnobsFileSelector;
    begin
      if SameText( aType, 'TKnobsDisplay')
      then begin
        if SameText( aName, 'waveplayer_display_filename')   // Some trickery to be able to read old patches
        then begin                                            // which did not have the file selector component
          aFileSelector := FindFileSelector( 'waveplayer_fileselector_filename');

          if Assigned( aFileSelector)
          then aFileSelector.SetText( aValue);
        end
        else begin
          aDisplay := FindDisplay( aName);

          if Assigned( aDisplay)
          then aDisplay.SetText( aValue);
        end;
      end
      else if SameText( aType, 'TKnobsFileSelector')
      then begin
        aFileSelector := FindFileSelector( aName);

        if Assigned( aFileSelector)
        then aFileSelector.SetText( aValue);
      end;
    end;

    function CleanControlName( const aName: string): string;
    var
      aParts : TStringList;
    begin
      aParts := Explode( aName, '_');
      try
        if aParts.Count >= 0
        then Result := aParts[ aParts.Count - 1]
        else Result := '';
      finally
        aParts.Free;
      end;
    end;

    procedure   TKnobsCustomModule.SetLight( const aName: string; aValue: TSignal);
    var
      i          : Integer;
      anIndicator: TKnobsIndicator;
      anIndBar   : TKnobsIndicatorBar;
      anIndText  : TKnobsIndicatorText;
    begin
      for i := 0 to ControlCount - 1
      do begin
        if Controls[ i] is TKnobsIndicator
        then begin
          anIndicator := TKnobsIndicator( Controls[ i]);
          if SameText( aName, CleanControlName( anIndicator.Name))
          then anIndicator.Active := aValue > 0;
        end
        else if Controls[ i] is TKnobsIndicatorBar
        then begin
          anIndBar := TKnobsIndicatorBar( Controls[ i]);
          if SameText( aName, CleanControlName( anIndBar.Name))
          then anIndBar.Value := aValue * TKnobsIndicatorBar( Controls[ i]).IndicatorCount;
        end
        else if Controls[ i] is TKnobsIndicatorText
        then begin
          anIndText := TKnobsIndicatorText( Controls[ i]);
          if SameText( aName, CleanControlName( anIndText.Name))
          then anIndText.Value := aValue;
        end;
      end;
    end;

    procedure   TKnobsCustomModule.SetData( const aName: string; const aData: TSignalArray);
    var
      i           : Integer;
      aDataViewer : TKnobsDataViewer;
    begin
      for i := 0 to ControlCount - 1
      do begin
        if Controls[ i] is TKnobsDataViewer
        then begin
          aDataViewer := TKnobsDataViewer( Controls[ i]);
          if SameText( aName, CleanControlName( aDataViewer.Name))
          then aDataViewer.Data := aData;
        end;
      end;
    end;

    procedure   TKnobsCustomModule.SetInfo( const aName: string; const aValue: string);
    var
      i        : Integer;
      aDisplay : TKnobsDisplay;
    begin
      for i := 0 to ControlCount - 1
      do begin
        if Controls[ i] is TKnobsDisplay
        then begin
          aDisplay := TKnobsDisplay( Controls[ i]);
          if SameText( aName, CleanControlName( aDisplay.Name))
          then aDisplay.Caption := aValue;
        end
      end;
    end;

    procedure   TKnobsCustomModule.Dump( var aFile: TextFile; anInd: Integer);
    var
      i : Integer;
    begin
      WriteLnInd( aFile, anInd, Format( 'module : %s', [ Name]));
      for i := 0 to ControlCount - 1
      do begin
        if Controls[ i] is TKnobsConnector
        then TKnobsConnector( Controls[ i]).Dump( aFile, anInd + 1)
        else if Controls[ i] is TKnobsValuedControl
        then TKnobsValuedControl( Controls[ i]).Dump( aFile, anInd + 1);
      end;
      WriteLnInd( aFile, anInd, Format( 'end module : %s', [ Name]));
    end;

    procedure   TKnobsCustomModule.Log( const aMsg: string);
    begin
      if Parent is TKnobsWirePanel
      then TKnobsWirePanel( Parent).Log( Format( 'custom module %s( %s)', [ Name, aMsg]))
      else if Assigned( FOnLog)
      then FOnLog( Self, Format( 'custom module %s( %s)', [ Name, aMsg]));
    end;

    procedure   TKnobsCustomModule.LogFmt( const aFmt: string; const anArgs: array of const);
    begin
      Log( Format( aFmt, anArgs));
    end;

    function    TKnobsCustomModule.AsBitmap: TBitmap;
    begin
      Result := TBitmap.Create;
      Result.Width  := Width;
      Result.Height := Height;
      PaintTo( Result.Canvas, 0, 0);
    end;

//  public

    procedure   TKnobsCustomModule.Restack( DoRestack, RedrawWires: Boolean);

      function DoSort( Item1, Item2: Pointer): Integer;

        function CompareModulePositions( F1, F2: TKnobsCustomModule): Integer;
        begin
          if F1.Left < F2.Left
          then Result := -1
          else if F1.Left > F2.Left
          then Result := 1
          else begin
            if F1.Top < F2.Top
            then Result := -1
            else if F1.Top > F2.Top
            then Result := 1
            else Result := 0;
          end;
        end;

      begin
        Result := CompareModulePositions( TKnobsCustomModule( Item1), TKnobsCustomModule( Item2));
      end;

    var
      i           : Integer;
      CurrentTop  : Integer;
      CurrentLeft : Integer;
      MinTop      : Integer;
      aWirePanel  : TKnobsWirePanel;
    begin
      if not DoRestack
      then Exit;

      aWirePanel := FindWirePanel;
      if Assigned( aWirePanel) and aWirePanel.SnapActive
      then begin
        with aWirePanel
        do begin

          CurrentLeft := OffsetLeft - ScrollOffset.X;
          CurrentTop  := OffsetTop  - ScrollOffset.Y;

          for i := 0 to ModuleCount - 1
          do begin
            with Module[ i]
            do begin
              if Top < CurrentTop
              then CurrentTop := Top;
              if Left < CurrentLeft
              then CurrentLeft := Left;
            end;
          end;

          MinTop := CurrentTop;

          FModules.Sort( @ DoSort);
          for i := 0 to ModuleCount - 1
          do begin
            with Module[ i]
            do begin
              TabOrder := i;

              if Left > CurrentLeft
              then begin
                CurrentLeft := Left;
                CurrentTop  := MinTop;
              end;

              if Top < CurrentTop
              then Top := CurrentTop;

              CurrentTop := Top + Height;

            end;
          end;

          if RedrawWires
          then InvalidateWires;
        end;
      end;
    end;

    procedure   TKnobsCustomModule.FixupInsertion;
    begin
      // todo : this may go wrong in other places too? see ***1
      // done: To fix naming mismatches beteen editor and compiled patch the
      // following two code lines were swapped. So after the swap restack comes
      // then then the name fixing.
      Restack( DO_RESTACK, DO_REDRAW_WIRES);
      FixNames;
    end;

    procedure   TKnobsCustomModule.FixDisplays;
    var
      i : Integer;
    begin
      for i := 0 to ControlCount - 1
      do begin
        if Controls[ i] is TKnobsValuedControl
        then TKnobsValuedControl( Controls[ i]).FixDisplay;
      end;
    end;

    procedure   TKnobsCustomModule.Disconnect;
    var
      i : Integer;
    begin
      for i := 0 to ControlCount - 1
      do begin
        if Controls[ i] is TKnobsConnector
        then TKnobsConnector( Controls[ i]).DisConnectAll;
      end;
    end;

    procedure   TKnobsCustomModule.SelectUnique;
    var
      aWp : TKnobsWirePanel;
    begin
      aWp := FindWirePanel;

      if Assigned( aWp) and ( aWp.BlockCount = 0)
      then aWp.SelectUnique( Self);
    end;

    procedure   TKnobsCustomModule.CollectInputsWithBaseName( var anInputs: TKnobsConnectors; const aBaseName: string);
    var
      i        : Integer;
      aParts   : TStringList;
      aResults : TStringList;
    begin
      SetLength( anInputs, 0);
      aResults := TStringList.Create;
      try
        aResults.Sorted := True;
        for i := 0 to ControlCount - 1
        do begin
          if Controls[ i] is TKnobsInput
          then begin
            aParts := Explode( Controls[ i].Name, '_');
            try
              if ( aParts.Count = 2) and ( Pos( aBaseName, aParts[ 1]) = 1)
              then aResults.AddObject( aParts[ 1], Controls[ i]);
            finally
              aParts.Free;
            end;
          end;
        end;
        for i := 0 to aResults.Count - 1
        do begin
          SetLength( anInputs, Length( anInputs) + 1);
          anInputs[ Length( anInputs) - 1] := TKnobsOutput( aResults.Objects[ i]);
        end;
      finally
        aResults.Free;
      end;
    end;

    procedure   TKnobsCustomModule.CollectOutputsWithBaseName( var anOutputs: TKnobsConnectors; const aBaseName: string);
    var
      i        : Integer;
      aParts   : TStringList;
      aResults : TStringList;
    begin
      SetLength( anOutputs, 0);
      aResults := TStringList.Create;
      try
        aResults.Sorted := True;
        for i := 0 to ControlCount - 1
        do begin
          if Controls[ i] is TKnobsOutput
          then begin
            aParts := Explode( Controls[ i].Name, '_');
            try
              if ( aParts.Count = 2) and ( Pos( aBaseName, aParts[ 1]) = 1)
              then aResults.AddObject( aParts[ 1], Controls[ i]);
            finally
              aParts.Free;
            end;
          end;
        end;
        for i := 0 to aResults.Count - 1
        do begin
          SetLength( anOutputs, Length( anOutputs) + 1);
          anOutputs[ Length( anOutputs) - 1] := TKnobsOutput( aResults.Objects[ i]);
        end;
      finally
        aResults.Free;
      end;
    end;

    procedure   TKnobsCustomModule.ChangeType( aNewType: TKnobsModuleType);
    begin
      // todo 2 -ctype change : change the actual module type
      // -- change the type ID
      // -- fix the details such that controls with the same name suffix will have the same values
      //    while controls not existing before will have some default values.
      // .. hmm ... does this need to be done here .. seems like the patch could better do this ...
    end;



{ ========
  TModuleButton = class( TSpeedButton)
  private
    FPageName   : string;
    FModuleType : TModuleType;
    FModuleName : string;
  public
    property    PageName   : string       read FPageName;
    property    Moduleclass: TModuleclass read FModuleclass;
    property    ModuleName : string       read FNoduleName;
  public
}

    procedure   TKnobsModuleButton.LoadGlyph( const aGlyph: TBitmap);
    var
      aFileName : string;
    begin
      if Assigned( aGlyph)
      then Glyph.Assign( aGlyph)
      else begin
        aFileName := ApplicationPath + 'bitmaps\' + ModuleName + '.bmp';

        if FileExists( aFileName)
        then
          with Glyph
          do begin
            LoadFromFile( aFileName);
            NumGlyphs := Width div Height;
          end
        else Caption := IntToStr( ModuleType);
      end;
    end;

    procedure   TKnobsModuleButton.Initialize( const aPageName: string; aModuleType: TKnobsModuleType; const aModuleName: string; const aGlyph: TBitmap);
    begin
      FPageName   := aPageName;
      FModuleType := aModuleType;
      FModuleName := aModuleName;
      AllowAllUp  := True;
      GroupIndex  := 1;
      Flat        := True;
      LoadGlyph( aGlyph);
    end;


{ ========
  TmoduleSelector = class( TPageControl)
  private
    FRegisteredModules   : TModuleList;
    FOnModuleButtonClick : TModuleButtonClick;
    FOnGetGlyph          : TOnGetGlyph;
    FButtonsEnabled      : Boolean;
    FAllowDuplicates     : Boolean;
    FTimer               : TTimer;
  published
    property    OnModuleButtonClick : TModuleButtonClick read FonModuleButtonClick write FOnModuleButtonClick;
    property    OnGetGlyph          : TOnGetGlyph        read FOnGetGlyph          write FonGetGlyph;
    property    ButtonsEnabled      : Boolean            read FButtonsEnabled      write SetButtonsEnabled;
    property    AllowDuplicates     : Boolean            read FAllowDuplicates     write FAllowDuplicates     default True;
  private
}

    procedure   TKnobsModuleSelector.StartTimeout;
    begin
      if not Assigned( FTimer)
      then FTimer := TTimer.Create( nil);
      if Assigned( FTimer)
      then begin
        FTimer.Enabled  := False;
        FTimer.OnTimer  := TimeoutFired;
        FTimer.Interval := 1000;
        FTimer.Enabled  := True;
      end;
    end;

    procedure   TKnobsModuleSelector.StopTimeout;
    begin
      if Assigned( FTimer)
      then begin
        FTimer.Enabled := False;
        FTimer.OnTimer := nil;
        FreeAndNil( FTimer);
      end;
    end;

    procedure   TKnobsModuleSelector.TimeOutFired( aSender: TObject);
    begin
      AllButtonsUp;
      StopTimeout;
    end;

    procedure   TKnobsModuleSelector.AllButtonsUp;
    var
      i : Integer;
    begin
      with FRegisteredModules
      do begin
        for i := 0 to Count - 1
        do begin
          TKnobsModuleButton( Items[ i]).Down := True;
          TKnobsModuleButton( Items[ i]).Down := False;
        end;
      end;
    end;

//  private

    procedure   TKnobsModuleSelector.SetButtonsEnabled( aValue: Boolean);
    var
      i : Integer;
    begin
      if aValue <> FButtonsEnabled
      then begin
        FButtonsEnabled := aValue;

        with FRegisteredModules
        do begin
          for i := 0 to Count - 1
          do TKnobsModuleButton( Items[ i]).Enabled := FButtonsEnabled;
        end;
      end;
    end;

    function    TKnobsModuleSelector.AddPage( const aName: string): TTabSheet;
    begin
      Result := TTabSheet.Create( Self);

      with Result
      do begin
        Visible     := True;
        Caption     := aName;
        PageControl := Self;
      end;
    end;

    function    TKnobsModuleSelector.PageButtonCount( const aName: string): Integer;
    var
      i : Integer;
    begin
      Result := 0;
      if PageExists( aName)
      then begin
        with PageByName( aName)
        do begin
          for i := 0 to ControlCount - 1
          do begin
            if Controls[ i] is TKnobsModuleButton
            then Inc( Result);
          end;
        end;
      end;
    end;

    function    TKnobsModuleSelector.PageExists( const aName: string): Boolean;
    begin
      Result := PageByName( aName) <> nil;
    end;

    function    TKnobsModuleSelector.PageByName( const aName: string): TTabSheet;
    var
      i : Integer;
    begin
      Result := nil;

      for i := 0 to PageCount - 1
      do begin
        if Pages[ i].Caption = aName
        then begin
          Result := Pages[ i];
          Break;
        end;
      end;
    end;

    function    TKnobsModuleSelector.FindModulePage( aModuleType: TKnobsModuleType): TTabSheet;
    var
      i : Integer;
    begin
      Result := nil;

      with FRegisteredModules
      do begin
        for i := 0 to Count - 1
        do begin
          with TKnobsModuleButton( Items[ i])
          do begin
            if ModuleType = aModuleType
            then begin
              Result := PageByName( PageName);
              Break;
            end;
          end;
        end;
      end;
    end;

    function    TKnobsModuleSelector.FindModuleButton( aModuleType: TKnobsModuleType): TKnobsModuleButton;
    var
      aPage : TTabSheet;
      i     : Integer;
    begin
      aPage := FindModulePage( aModuleType);
      Result := nil;
      if Assigned( aPage)
      then begin
        with aPage
        do begin
          for i := 0 to ControlCount - 1
          do begin
            if Controls[ i] is TKnobsModuleButton
            then begin
              with TKnobsModuleButton( Controls[ i])
              do begin
                if ModuleType = aModuleType
                then begin
                  Result := TKnobsModuleButton( Controls[ i]);
                  Break;
                end;
              end;
            end;
          end;
        end;
      end;
    end;

    procedure   TKnobsModuleSelector.DoModuleMouseDown( aSender: TObject; aButton: TMouseButton; aShift: TShiftState; X, Y: Integer);
    begin
      if Assigned( FOnModuleButtonClick) and ( aSender is TKnobsModuleButton) and ( aShift = [ ssLeft])
      then begin
        FOnModuleButtonClick(
          TKnobsModuleButton( aSender),
          TKnobsModuleButton( aSender).ModuleType
        );
        StartTimeout;  // should release the button after a while ... button stays down though ... grrr .. this sort of works ...
      end;
    end;

    procedure   TKnobsModuleSelector.FormatPage( aPage: TTabSheet);
    var
      LastLeft : Integer;
      i        : Integer;
    begin
      Assert(
        Assigned( aPage),
        'TKnobsModuleSelecteor.FormatPage : can''t format an unassigned <nil> page.'
      );

      if PageButtonCount( aPage.Caption) = 0
      then begin
        aPage.Free;
        if PageCount > 0
        then ActivePage := Pages[ 0];
      end
      else begin
        with aPage
        do begin
          LastLeft := 0;

          for i := 0 to ControlCount - 1
          do begin
            if Controls[ i] is TKnobsModuleButton
            then begin
              with TKnobsModuleButton( Controls[ i])
              do begin
                Left := LastLeft;
                if Controls[ i] is TKnobsModuleSeparator
                then Inc( LastLeft, 12)
                else Inc( LastLeft, Width + 2);
              end;
            end;
          end;
        end;
      end;
    end;

{  protected }

    procedure   TKnobsModuleSelector.Notification( aComponent: TComponent; anOperation: TOperation); // override;
    begin
      if aComponent is TKnobsModuleButton
      then begin
        if anOperation = opInsert
        then FRegisteredModules.Add   ( aComponent)
        else FRegisteredModules.Remove( aComponent);
      end;
    end;

{   public }

    constructor TKnobsModuleSelector.Create( anOwner: TComponent); // override;
    begin
      inherited;

      FRegisteredModules := TKnobsModuleList.Create;
      HotTrack           := True;
      FAllowDuplicates   := True;
    end;

    destructor  TKnobsModuleSelector.Destroy; // override;
    begin
      StopTimeout;
      inherited;
      FRegisteredModules.Free;
     end;

    procedure   TKnobsModuleSelector.RegisterModuleType( const aPageName: string; aModuleType: TKnobsModuleType; const aModuleName: string);
    var
      aPage         : TTabSheet;
      aModuleButton : TKnobsModuleButton;
      aGlyph        : TBitmap;
    begin
      if not FAllowDuplicates
      then begin
        if Assigned( FindModuleButton( aModuleType)) and ( aModuleType >= 0)
        then begin
          raise
            EKnobsNoDuplicates.
              CreateFmt(
                'A module type %d with name ''%s''is already present on page ''%s''',
                [
                  aModuleType,
                  aModuleName,
                  FindModulePage( aModuleType).Caption
                ]
              );
        end;
      end;

      aPage := PageByName( aPageName);

      if not Assigned( aPage)
      then aPage := AddPage( aPageName);

      ActivePage := aPage;
      ActivePage := Pages[ 0];
      if aModuleType >= 0
      then aModuleButton := TKnobsModuleButton   .Create( Self)
      else aModuleButton := TKnobsModuleSeparator.Create( Self);

      if Assigned( FOnGetGlyph)
      then aGlyph := FOnGetGlyph( Self, aModuleName)
      else aGlyph := nil;

      with aModuleButton
      do begin
        Parent   := aPage;
        Left     := 0;
        Top      := 0;
        Flat     := True;
        Hint     := Format( '%s~%d~', [ aModuleName, aModuleType]);
        ShowHint := True;
        Enabled  := FButtonsEnabled;
        Visible  := aModuleType >= 0;
        Initialize( aPageName, aModuleType, aModuleName, aGlyph);
        OnMouseDown := DoModuleMouseDown;
      end;

      FormatPage( aPage);
    end;

    procedure   TKnobsModuleSelector.UnregisterModuleType( aModuleType: TKnobsModuleType);
    var
      aModuleButton : TKnobsModuleButton;
      aPage         : TTabSheet;
    begin
      aModuleButton := FindModuleButton( aModuleType);
      aPage         := FindModulePage  ( aModuleType);

      if Assigned( aModuleButton) and Assigned( aPage)
      then begin
        aModuleButton.Free;
        FormatPage( aPage); // This will remove the page when it beccomes empty.
      end;
    end;

{ ========
  TKnobsHintWindow = class( THintWindow)
  private
    FModuleType : string;
    FBitmap     : TBitmap;
    FComment    : string;
  published
    property    Caption;
  protected
}

    function    TKnobsHintWindow.MakeBitmap( const aModuleType: string): TBitmap;
    var
      aModuleTypeInt : TKnobsModuleType;
    begin
      aModuleTypeInt := StrToIntDef( aModuleType, -1);
      if aModuleTypeInt >= 0
      then begin
        Result   := CreateModuleBitmap( aModuleTypeInt);
        FComment := ReadModuleComment ( aModuleTypeInt);
      end
      else begin
        Result   := nil;
        FComment := '';
      end;
    end;

    procedure   TKnobsHintWindow.ParseHint( var aHintStr: string);
    type
      TParseState = ( psNothing, psTilde);
    var
      aHint      : string;
      i          : Integer;
      ParseState : TParseState;
    begin
      FModuleType   := '';
      aHint         := '';
      ParseState    := psNothing;

      for i := 1 to Length( aHintStr)
      do begin

        case ParseState of

          psNothing :
            begin
              if aHintStr[ i] = '~'
              then ParseState := psTilde
              else aHint := aHint + aHintStr[ i];
            end;

          psTilde :
            begin
              if aHintStr[ i] = '~'
              then ParseState := psNothing
              else FModuleType := FModuleType + aHintStr[ i];
            end;
        end;

      end;

      if Assigned( FBitmap)
      then FreeAndNil( FBitmap);

      FBitmap := MakeBitmap( FModuleType);

      aHintStr := FComment;
    end;

    procedure   TKnobsHintWindow.Paint; // override;
    var
      R          : TRect;
      aTextRect  : TRect;
      aTopMargin : Integer;
    const
      TEXT_MARGIN_LEFT =  2;
      IMG_MARGIN_TOP   =  4;
      TEXT_MARGIN_TOP  =  2;
    begin
      if Assigned( FBitmap)
      then aTopMargin := IMG_MARGIN_TOP + TEXT_MARGIN_TOP + FBitmap.Height
      else aTopMargin := TEXT_MARGIN_TOP;

      R := ClientRect;
      Inc( R.Left, 2);
      Inc( R.Top , 2);

      with Canvas
      do begin
        Rectangle( 0, 0, R.Width + 2, R.Bottom + 2);
        if Assigned( FBitmap)
        then Draw( 2, IMG_MARGIN_TOP, FBitmap);
        aTextRect := R;
        Inc( aTextRect.Top , aTopMargin);
        Inc( aTextRect.Left, TEXT_MARGIN_LEFT);
        DrawText( Handle, PChar( Caption), -1, aTextRect, dt_NoClip or dt_WordBreak);
      end;
    end;

//  public

    constructor TKnobsHintWindow.Create( anOwner: TComponent); // override;
    begin
      inherited;

      with Canvas.Font do
      begin
        Name  := 'Ariel';
        Color := clBlack;
      end;
    end;

    destructor  TKnobsHintWindow.Destroy; // override;
    begin
      if Assigned( FBitmap)
      then FreeAndNil( FBitmap);
      inherited;
    end;

    procedure   TKnobsHintWindow.ActivateHint( aRect: TRect; const aHint: string); // override;
    const
      EXTRA_WIDTH  =  8;
      EXTRA_HEIGHT =  8;
    var
      anExtraHeight : Integer;
      aWidth        : Integer;
      aHintStr      : string;
    begin
      aHintStr := aHint;
      ParseHint( aHintStr);
      Caption := aHintStr;

      if Assigned( FBitmap)
      then begin
        anExtraHeight := EXTRA_HEIGHT + FBitmap.Height;
        aWidth        := EXTRA_WIDTH  + FBitmap.Width;
      end
      else begin
        anExtraHeight := EXTRA_HEIGHT;
        aWidth        := EXTRA_WIDTH + aRect.Width;
      end;

      Inc( aRect.Bottom, anExtraHeight);
      aRect.Right := aRect.Left + aWidth;
      UpdateBoundsRect( aRect);

      if aRect.Top + Height > Screen.DesktopHeight
      then aRect.Top := Screen.DesktopHeight - Height;

      if aRect.Left + Width > Screen.DesktopWidth
      then aRect.Left := Screen.DesktopWidth - Width;

      if aRect.Left < Screen.DesktopLeft
      then aRect.Left := Screen.DesktopLeft;

      if aRect.Bottom < Screen.DesktopTop
      then aRect.Bottom := Screen.DesktopTop;

      SetWindowPos( Handle, HWND_TOPMOST, aRect.Left, aRect.Top, Width, Height, SWP_SHOWWINDOW or SWP_NOACTIVATE);
      Invalidate;
    end;


initialization

  CreateBitmaps;

finalization

  FreeBitmaps;

end.

